<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://n-ware.hu
 * @since             0.8.1
 * @package           billzone.eu Invoice
 *
 * @wordpress-plugin
 * Plugin Name:       billzone.eu Invoice
 * Plugin URI:        https://billzone.eu
 * Description:       WooCommerce PDF Invoices by billzone.eu
 * Version:           0.8.1
 * Author:            N-Ware
 * Author URI:        https://n-ware.hu
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       billzone-eu-invoice
 * Domain Path:       /languages
 * 
 * WC requires at least: 4.7
 * WC tested up to: 6.0.0
 * 
 */

//include all common variables, constants and other data
require_once 'include/billzone-eu-invoice-common.php';


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Exit if accessed directly
*/
if ( ! defined( 'ABSPATH' ) ) { 
    exit; 
}
//---------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add billzone.eu Invoice plugin stylesheet which customize the look of the settings tab, error messages and other elements
 * @access public
*/
function billzone_eu_invoice_add_stylesheet()  
{
    wp_enqueue_style( 'billzone-eu-invoice', BILLZONE_EU_INVOICE_PLUGIN_DIR . '/css/billzone-eu-invoice-admin.min.css', array(), BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM );
}
add_action( 'admin_enqueue_scripts', 'billzone_eu_invoice_add_stylesheet' ); 
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add billzone.eu Invoice plugin javascript file which contains all needed js scripts
 * @access public
*/
function billzone_eu_invoice_add_javascript()  
{
    wp_enqueue_script( 'billzone-eu-invoice', BILLZONE_EU_INVOICE_PLUGIN_DIR . '/js/billzone-eu-invoice.min.js', array(), BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM );
}
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Prepare translations
 * @access public
*/
function billzone_eu_invoice_load_plugin_textdomain() { 
    load_plugin_textdomain( BILLZONE_EU_INVOICE_PLUGIN_NAME, false, BILLZONE_EU_INVOICE_PLUGIN_NAME.'/languages' );
  }
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
// Error messages

/**
 * Function to generate error message body
 * @access public
*/
function billzone_eu_invoice_requirements_error_message_body($bei_message) {
    echo 
    '<div class="notice error billzone-eu-invoice-woocommerce-missing-error">
        <div>
            <span class="dashicons dashicons-warning"></span>
        </div>
        <div>
            <strong>'.$bei_message.'</strong>
        </div>
	</div>';
}

/**
 * Wrong PHP version error
 * @access public
*/
function billzone_eu_invoice_wrong_php_version_message() { 
    billzone_eu_invoice_requirements_error_message_body(__('The billzone.eu Invoice plugin requires PHP 7.0 or greater to work properly! For help ask your hosting admin, or check your hosting control panel. After upgrade, please delete and reinstall the plugin.',BILLZONE_EU_INVOICE_PLUGIN_NAME));
}

/**
 * WooCommerce missing error
 * @access public
*/
function billzone_eu_invoice_missing_woocommerce_message() { 
    billzone_eu_invoice_requirements_error_message_body(__('The billzone.eu Invoice plugin requires WooCommerce to be installed and activated! After WooCommerce activation, please delete and reinstall the plugin.',BILLZONE_EU_INVOICE_PLUGIN_NAME));
}

/**
 * Open SSL PHP extension missing error
 * @access public
*/
function billzone_eu_invoice_missing_openssl_message() { 
    billzone_eu_invoice_requirements_error_message_body(__('The billzone.eu Invoice plugin requires PHP OpenSSL extension to be installed! For help ask your hosting admin, or check your hosting control panel. After enabling the extension, please delete and reinstall the plugin.',BILLZONE_EU_INVOICE_PLUGIN_NAME));
}

/**
 * PHP SOAP extension missing error
 * @access public
*/
function billzone_eu_invoice_missing_soap_message() { 
    billzone_eu_invoice_requirements_error_message_body(__('The billzone.eu Invoice plugin requires PHP SOAP extension to be installed! For help ask your hosting admin, or check your hosting control panel. After enabling the extension, please delete and reinstall the plugin.',BILLZONE_EU_INVOICE_PLUGIN_NAME));
}

/**
 * PHP XSL extension missing error
 * @access public
*/
function billzone_eu_invoice_missing_xsl_message() { 
    billzone_eu_invoice_requirements_error_message_body(__('The billzone.eu Invoice plugin requires PHP XSL extension to be installed! For help ask your hosting admin, or check your hosting control panel. After enabling the extension, please delete and reinstall the plugin.',BILLZONE_EU_INVOICE_PLUGIN_NAME));
}
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Check for requirements to run billzone.eu Invoice and then after run the plugin
 * @access public
*/
function billzone_eu_invoice_check_requirements_and_load_plugin(){

    if (billzone_eu_invoice_all_requirements_are_met()) {
        
        // Prepare translations
        billzone_eu_invoice_load_plugin_textdomain();

        // Action to call billzone_eu_invoice_add_javascript() function to include all neccessary javascripts
        add_action('admin_enqueue_scripts', 'billzone_eu_invoice_add_javascript');

        include_once 'include/billzone-eu-invoice-main.php';
    }
}

add_action( 'init', 'billzone_eu_invoice_check_requirements_and_load_plugin' );
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Create "xxx_billzone_eu_invoice" and "xxx_billzone_eu_invoice_log" table (if the table is already created, the content will be erased) and
 * add plugin version to settings table (xxx is same as WP prefix)
 * The table will not be created if WooCommerce is missing or all requirements are not met
 * @access public
*/
function billzone_eu_invoice_create_table_and_insert_version(){
    if (billzone_eu_invoice_all_requirements_are_met()) {
        bei_create_all_table_and_insert_version();
    }
}
register_activation_hook( __FILE__, 'billzone_eu_invoice_create_table_and_insert_version' );
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Clear all data related to billzone.eu Invoice plugin when uninstall plugin
 * @access public
*/
function billzone_eu_invoice_uninstall(){
    
    // delete all option stored in xxx_wp_option database table
    foreach ( BILLZONE_EU_INVOICE_ALL_OPTION_STORED_IN_DATABASE as $bei_opt ) {
        if ( !is_bool(get_option( $bei_opt )) ) {
            delete_option( $bei_opt );
        }
    }
    
    // delete all table
    global $wpdb;
    $bei_table_name = BILLZONE_EU_INVOICE_TABLE_NAME;
    $bei_log_table_name = BILLZONE_EU_INVOICE_LOG_TABLE_NAME;
    $wpdb->query( 'DROP TABLE IF EXISTS '.$bei_table_name.','.$bei_log_table_name );
    
    
}
register_uninstall_hook(__FILE__, 'billzone_eu_invoice_uninstall');
//----------------------------------------------------------------------------------------------------------------------------------------------