<?php

/**
 * This file contains API calling class and functions
 *
 * 
 * @link              https://n-ware.hu
 * @since             0.8.1
 * @package           billzone.eu Invoice
 *
 */

include_once 'billzone-eu-invoice-log.php';

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Exit if accessed directly
*/
if ( ! defined( 'ABSPATH' ) ) { 
    exit; 
}
//---------------------------------------------------------------------------------------------------------------------------------------------

if (!class_exists("BEI_INVOICE")) {
    
/**
 * Main class for creating/getting billzone.eu invoice
*/
    class BEI_INVOICE{

        public $TransactionId;
        public $SourceSystem;
        public $RequestId;
        public $SecurityToken;
        public $OrderNumber;

        public $Name;
        public $TaxNumber;
        public $EUTaxNumber;
        public $AddressPostalCode;
        public $AddressCity;
        public $AddressCountryId;
        public $AddressLine1;

        public $HasHungarianTaxNumber;
        public $LegalEntityWithoutTaxNumber;
        
        public $BankIdentifier;
        public $AccountBlockPrefix;

        public $FulfillmentDate;
        public $PaymentDueDate;
  
        public $InvoiceDescription;

        public $SendInvoiceToCustomer;
        public $SendInvoiceToEmailAddress;

        public $LocalForeignCurrencyExchangeRate;

        public $Lines = array();
        

        public function __construct($bei_API_invoicing_service_source) {

            billzone_eu_invoice_logme(__FILE__,'API - OK','#1: class BEI_INVOICE > START');

            // set API source (production or sandbox)
            include_once $bei_API_invoicing_service_source;

            billzone_eu_invoice_logme(__FILE__,'API - OK','#2: class BEI_INVOICE > END');
             
        }
               

        /**
         * Function to get invoice data internaly
	     * @access private
	     * @return object
	    */
        private function get_invoice_data_internal() {

            billzone_eu_invoice_logme(__FILE__,'API - OK','#3: class BEI_INVOICE::get_invoice_data_internal > START');

            //Creating InvoicingService object
	        $InvoicingService = new InvoicingService();

	        //Creating GetInvoiceDataQuery object and fill up
	        $GetInvoiceDataQuery = new GetInvoiceDataQuery();
		
        	//Creating GetInvoiceDataRequest object and fill up
	        $GetInvoiceDataRequest = new GetInvoiceDataRequest();
	        $GetInvoiceDataRequest -> GetInvoiceDataQuery = $GetInvoiceDataQuery;
	        $GetInvoiceDataRequest -> RequestId = $this->RequestId;
	
	        //Company SecurityToken
	        $GetInvoiceDataRequest -> SecurityToken = $this->SecurityToken;	
	
	        //Creating GetInvoiceData object and fill up
	        $GetInvoiceData = new GetInvoiceData();
	        $GetInvoiceData -> request = $GetInvoiceDataRequest;		
	
        	//Calling the GetInvoiceData function with previously setted values
            $GetInvoiceDataResponse = $InvoicingService -> GetInvoiceData($GetInvoiceData);
            
            billzone_eu_invoice_logme(__FILE__,'API - OK','#4: class BEI_INVOICE::get_invoice_data_internal > END');

            return $GetInvoiceDataResponse;
        }
        
        /**
         * Function to check security token
	     * @access public
	     * @return object
	    */
        public function check_security_token() {

            billzone_eu_invoice_logme(__FILE__,'API - OK','#5: class BEI_INVOICE::check_security_token > START');

            $GetInvoiceDataResponse = $this->get_invoice_data_internal();
            
            $GetInvoiceDataResponse->GetInvoiceDataResult->RequestResult->Code == 61001 ? $bei_response['status'] = 'OK' : $bei_response['status'] = 'ERROR';
            $bei_response['code'] = $GetInvoiceDataResponse->GetInvoiceDataResult->RequestResult->Code;

            billzone_eu_invoice_logme(__FILE__,'API - OK','#6: class BEI_INVOICE::check_security_token > END');

            return $bei_response;
        }

        /**
         * Function to get invoice data public
	     * @access public
	     * @return array
	    */
        public function get_invoice_data() {

            billzone_eu_invoice_logme(__FILE__,'API - OK','#7: class BEI_INVOICE::get_invoice_data > START');

            $beiGetInvoiceDataResponse = $this->get_invoice_data_internal();
            $array_beiGetInvoiceDataResponse = (array)$beiGetInvoiceDataResponse;

            
            if ($array_beiGetInvoiceDataResponse['GetInvoiceDataResult']->RequestResult->Code == BILLZONE_EU_INVOICE_OK_SECURITY_CODE) {
                
                // If the adddress country is equal to Hungary (namely HU) then get current exchange rate via API request from Hungarian Central Bank (Magyar Nemzeti Bank)
                if (strtoupper($array_beiGetInvoiceDataResponse['GetInvoiceDataResult']->QueryResult->CompanyData->AddressCountryId) == "HU"){
                    $array_beiGetInvoiceDataResponse['MNBCurrentExchangeRates'] = bei_get_MNB_current_exchange_rates_via_API();
                }
                else {
                    $array_beiGetInvoiceDataResponse['MNBCurrentExchangeRates'] = NULL;
                }

                // Get default currency for country defined by invoice issuer address
                $array_beiGetInvoiceDataResponse['InvoiceIssuerCountryDefaultCurrency'] = bei_get_country_default_currency(strtoupper($array_beiGetInvoiceDataResponse['GetInvoiceDataResult']->QueryResult->CompanyData->AddressCountryId));
            }
            else {
                $array_beiGetInvoiceDataResponse['MNBCurrentExchangeRates'] = NULL;
                $array_beiGetInvoiceDataResponse['InvoiceIssuerCountryDefaultCurrency'] = NULL;
            }
            
            billzone_eu_invoice_logme(__FILE__,'API - OK','#8: class BEI_INVOICE::get_invoice_data > END');
            
            return $array_beiGetInvoiceDataResponse;
        }

        /**
         * Function o create invoice
	     * @access public
	     * @return array
	    */
        public function create_invoice() {

            billzone_eu_invoice_logme(__FILE__,'API - OK','#9: class BEI_INVOICE::create_invoice > START');
        
            //Creating InvoicingService object
            $InvoicingService = new InvoicingService();	
        
            //Creating Customer object and fill up
            $Customer = new Customer();
            $Customer -> Name = $this->Name;
            $Customer -> TaxNumber = $this->TaxNumber;
            $Customer -> EUTaxNumber = $this->EUTaxNumber;
            $Customer -> AddressPostalCode = $this->AddressPostalCode;
            $Customer -> AddressCity = $this->AddressCity;
            $Customer -> AddressCountryId = $this->AddressCountryId;
            $Customer -> AddressLine1 = $this->AddressLine1;
            $Customer -> HasHungarianTaxNumber = $this->HasHungarianTaxNumber;
            $Customer -> LegalEntityWithoutTaxNumber = $this->LegalEntityWithoutTaxNumber;
            
            //Creating Bank object and fill up
            $Bank = new BankIdentifier();
            $Bank -> Identifier = $this->BankIdentifier;
    
            //Creating InvoiceHeader object and fill up
            $InvoiceHeader = new InvoiceHeader();
            $InvoiceHeader -> AccountBlockPrefix = $this->AccountBlockPrefix;
    
            $InvoiceHeader -> Bank = $Bank;
            $InvoiceHeader -> Currency = new DefaultCurrency();
            $InvoiceHeader -> IssuerAddress = new DefaultIssuerAddress();
            $InvoiceHeader -> Customer = $Customer;
            $InvoiceHeader -> FulfillmentDate = date("Y-m-d",strtotime($this->FulfillmentDate));
            $InvoiceHeader -> PaymentDueDate = date("Y-m-d",strtotime($this->PaymentDueDate));
            $InvoiceHeader -> InterEUVatExempt = 0;
            $InvoiceHeader -> InvoiceDescription = $this->InvoiceDescription;
            $InvoiceHeader -> OrderNumber = $this->OrderNumber;
            $InvoiceHeader -> LocalForeignCurrencyExchangeRate = bei_number($this->LocalForeignCurrencyExchangeRate);

            //Creating CreateInvoiceTransaction object and fill up
            $CreateInvoiceTransaction = new CreateInvoiceTransaction();
            $CreateInvoiceTransaction -> TransactionId = $this->TransactionId;
            $CreateInvoiceTransaction -> SourceSystem = $this->SourceSystem;
            $CreateInvoiceTransaction -> Header = $InvoiceHeader;
            $CreateInvoiceTransaction -> Lines = $this->Lines;
            

            if ($this->SendInvoiceToCustomer === TRUE) {
                // Send order email to customer
                $CreateInvoiceTransaction -> SendInvoiceToCustomer = false;
                $CreateInvoiceTransaction -> InvoiceCourierTypeId = 1;
            }
            else  {
                // The order email link will be returned in response
                $CreateInvoiceTransaction -> SendInvoiceToCustomer = FALSE;
                $CreateInvoiceTransaction -> InvoiceCourierTypeId = 2;
            }
        
            $CreateInvoiceTransaction -> SendInvoiceToEmailAddress = $this->SendInvoiceToEmailAddress;
        
            //Creating CreateInvoiceRequest object and fill up
            $CreateInvoiceRequest = new CreateInvoiceRequest();
            $CreateInvoiceRequest -> CreateInvoiceTransaction = $CreateInvoiceTransaction; 
            $CreateInvoiceRequest -> RequestId = $this->RequestId;
    
            //Company SecurityToken
            $CreateInvoiceRequest -> SecurityToken = $this->SecurityToken;
            
            //Creating CreateInvoice object and fill up
            $CreateInvoice = new CreateInvoice();
            $CreateInvoice -> request = $CreateInvoiceRequest;
        
            //Creating CreateInvoiceResponse object and fill up
            $CreateInvoiceResponse = new CreateInvoiceResponse();
            $CreateInvoiceResponse = $InvoicingService -> CreateInvoice($CreateInvoice);
                   
            //Get InvoiceNumber from response
            $InvoiceNumber = $CreateInvoiceResponse -> CreateInvoiceResult -> TransactionResult -> InvoiceNumber;

            billzone_eu_invoice_logme(__FILE__,'API - OK','#10: class BEI_INVOICE::create_invoice > END');
        
            return $CreateInvoiceResponse;
        
        }
    }
}
