<?php

/**
 * This file is store all admin options and admin option stored in database (xxx_options table) where xxx is the database prefix defined when WP was installed.
 *
 *
 * @link              https://n-ware.hu
 * @since             0.8.1
 * @package           billzone.eu Invoice
 *
 */

include_once 'billzone-eu-invoice-log.php';

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Exit if accessed directly
 */
if (!defined('ABSPATH')) {
    exit;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Default menu order for billzone.eu Invoice
 * @var integer
 */
$bei_admin_sidebar_menu_order = 1;
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Invoice type
 * @var array
 */
$bei_invoice_type = array(
    'normal' => 'normal',
    'void' => 'void',
    'credit_memo' => 'credit_memo',
    'debit_memo' => 'debit_memo',
);
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Constant for invoice type
 */
if (!defined('BILLZONE_EU_INVOICE_INVOICE_TYPE')) {
    define('BILLZONE_EU_INVOICE_INVOICE_TYPE', $bei_invoice_type);
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Constant for API WSDL path (production server)
 */
if (!defined('BILLZONE_EU_INVOICE_API_WSDL_PATH')) {
    define('BILLZONE_EU_INVOICE_API_WSDL_PATH', 'https://www.billzone.eu/billgate?wsdl');
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Constant for API WSDL path (sandbox server)
 */
if (!defined('BILLZONE_EU_INVOICE_SANDBOX_API_WSDL_PATH')) {
    define('BILLZONE_EU_INVOICE_SANDBOX_API_WSDL_PATH', 'https://sandbox.billzone.eu/billgate?wsdl');
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * All options name ID defined in admin panel and stored in database
 * @var array
 */
$bei_options = array(
    'billzone_eu_invoice_company_location_id' => 'billzone_eu_invoice_company_location_id',
    'billzone_eu_invoice_date_of_delivery_plus_days' => 'billzone_eu_invoice_date_of_delivery_plus_days',
    'billzone_eu_invoice_payment_due_plus_days' => 'billzone_eu_invoice_payment_due_plus_days',
    'billzone_eu_invoice_invoice_notes' => 'billzone_eu_invoice_invoice_notes',
    'billzone_eu_invoice_invoice_number_prefix' => 'billzone_eu_invoice_invoice_number_prefix',
    'billzone_eu_invoice_is_sandbox' => 'billzone_eu_invoice_is_sandbox',
    'billzone_eu_invoice_security_code' => 'billzone_eu_invoice_security_code',
    'billzone_eu_invoice_send_email' => 'billzone_eu_invoice_send_email',
    'billzone_eu_invoice_version' => 'billzone_eu_invoice_version',
    'billzone_eu_invoice_security_code_status' => 'billzone_eu_invoice_security_code_status',
    'billzone_eu_invoice_all_billzone_data_list' => 'billzone_eu_invoice_all_billzone_data_list',
    'billzone_eu_invoice_default_pricing_unit' => 'billzone_eu_invoice_default_pricing_unit',
    'billzone_eu_invoice_default_payment_methode' => 'billzone_eu_invoice_default_payment_methode',
    'billzone_eu_invoice_get_exchange_rate' => 'billzone_eu_invoice_get_exchange_rate',
    'billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code' => 'billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code',
    'billzone_eu_invoice_log_is_on' => 'billzone_eu_invoice_log_is_on',
);
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Constant for storing all options
 */
if (!defined('BILLZONE_EU_INVOICE_ALL_OPTION_STORED_IN_DATABASE')) {
    define('BILLZONE_EU_INVOICE_ALL_OPTION_STORED_IN_DATABASE', $bei_options);
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * UN/EDIFACT codes
 * @var array
 */
$bei_UN_EDIFACT_codes = array(
    'S' => 27,
    'AA-2' => 18,
    'AA-1' => 5,
    'AB-AAH1' => 0,
    'AB-AAH1' => 0,
    'AE-AAO' => 0,
    'E-AAB1' => 0,
    'E-AAB2' => 0,
    'E-AAC' => 0,
    'E-AAF' => 0,
    'E-AAG1' => 0,
    'E-AAG2' => 0,
    'E-AAG3' => 0,
    'E-AAI' => 0,
    'E-AAM' => 0,
    'E-AAN' => 0,
    'G-AAA1' => 0,
    'G-AAA2' => 0,
    'G-AAA3' => 0,
    'G-AAA4' => 0,
    'O' => 0,
);
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
global $wpdb;
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Constant for database table name to store invoice numbers
 */
if (!defined('BILLZONE_EU_INVOICE_TABLE_NAME')) {
    define('BILLZONE_EU_INVOICE_TABLE_NAME', $wpdb->prefix . 'billzone_eu_invoice');
}

/**
 * Constant for database log table name to store log informations
 */
if (!defined('BILLZONE_EU_INVOICE_LOG_TABLE_NAME')) {
    define('BILLZONE_EU_INVOICE_LOG_TABLE_NAME', $wpdb->prefix . 'billzone_eu_invoice_log');
}

/**
 * Constant for plugin path
 */
if (!defined('BILLZONE_EU_INVOICE_PLUGIN_DIR')) {
    define('BILLZONE_EU_INVOICE_PLUGIN_DIR', str_replace('/include', '', plugins_url('', __FILE__)));
}

/**
 * Constant for readable plugin name
 */
if (!defined('BILLZONE_EU_INVOICE_READABLE_PLUGIN_NAME')) {
    define('BILLZONE_EU_INVOICE_READABLE_PLUGIN_NAME', 'billzone.eu Invoice');
}

/**
 * Constant for plugin name
 */
if (!defined('BILLZONE_EU_INVOICE_PLUGIN_NAME')) {
    define('BILLZONE_EU_INVOICE_PLUGIN_NAME', str_replace('/include', '', trim(dirname(plugin_basename(__FILE__)), '/')));
}

/**
 * billzone.eu Invoice version key to store in database
 */
if (!defined('BILLZONE_EU_INVOICE_PLUGIN_VERSION_KEY')) {
    define('BILLZONE_EU_INVOICE_PLUGIN_VERSION_KEY', $bei_options['billzone_eu_invoice_version']);
}

/**
 * billzone.eu Invoice version number to store in database
 */
if (!defined('BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM')) {
    define('BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM', '0.8.1');
}

/**
 * Constant for empty security code
 */
if (!defined('BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE')) {
    define('BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE', 99999);
}

/**
 * Constant for empty security code
 */
if (!defined('BILLZONE_EU_INVOICE_OK_SECURITY_CODE')) {
    define('BILLZONE_EU_INVOICE_OK_SECURITY_CODE', 61001);
}

/**
 * Constant for SystemSource API field
 */
if (!defined('BILLZONE_EU_INVOICE_SYSTEM_SOURCE')) {
    define('BILLZONE_EU_INVOICE_SYSTEM_SOURCE', 'Woo-BEI v' . BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM);
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Replace comma and quotes with entity codes
 * @access public
 * @return string
 */
function bei_rep($bei_str_in)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_rep > START');
    $toSearch = array("'", '"');
    $toReplace = array("&#39;", "&#34;");
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_rep > END');
    return str_replace($toSearch, $toReplace, $bei_str_in);
}

/**
 * Round function to show numbers with 2 decimals (as billzone.eu works)
 * @access public
 * @return string
 */
function bei_b_round($bei_number_in)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_b_round > START_END');
    return bei_rep(number_format($bei_number_in, 2, wc_get_price_decimal_separator(), wc_get_price_thousand_separator()));
}

/**
 * Round function to show numbers with 0 decimals
 * @access public
 * @return string
 */
function bei_z_round($bei_number_in)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_z_round > START_END');
    return bei_rep(number_format($bei_number_in, 0, wc_get_price_decimal_separator(), wc_get_price_thousand_separator()));
}

/**
 * Round function to show numbers according to woocomerce settings
 * @access public
 * @return string
 */
function bei_w_round($bei_number_in)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_w_round > START_END');
    return bei_rep(number_format($bei_number_in, wc_get_price_decimals(), wc_get_price_decimal_separator(), wc_get_price_thousand_separator()));
}

/**
 * Round function to show numbers according to woocomerce settings using bankers round rule
 * @access public
 * @return string
 */
function bei_wb_round($bei_number_in)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_wb_round > START_END');
    return bei_rep(number_format(round($bei_number_in, wc_get_price_decimals(), PHP_ROUND_HALF_UP), wc_get_price_decimals(), wc_get_price_decimal_separator(), wc_get_price_thousand_separator()));
}

/**
 * Transform decimal comma separated string into decimal period float
 * @access public
 * @return string
 */
function bei_number($string_in)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_number > START_END');
    return (!empty($string_in)) ? floatval(str_replace(',', '.', $string_in)) : null;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * GUID generator
 * @access public
 * @return number
 */
function bei_create_GUID()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_create_GUID > START');
    if (function_exists('com_create_guid')) { // for Windows base server

        billzone_eu_invoice_logme(__FILE__, 'OK', '#2A: function bei_create_GUID > END Windows system');
        return com_create_guid();
    } else { // for Linux base server
        $bei_charid = bin2hex(openssl_random_pseudo_bytes(16));
        $bei_hyphen = '-';
        $bei_uuid = '{'
        . substr($bei_charid, 0, 8)
        . $bei_hyphen . substr($bei_charid, 8, 4)
        . $bei_hyphen . substr($bei_charid, 12, 4)
        . $bei_hyphen . substr($bei_charid, 16, 4)
        . $bei_hyphen . substr($bei_charid, 20, 12)
            . '}';

        billzone_eu_invoice_logme(__FILE__, 'OK', '#2B: function bei_create_GUID > END Linux base system');
        return trim($bei_uuid);
    }

}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return all country code available in Woocommerce plus Northern Ireland
 * @return array
 */
function bei_all_country_code_base()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_all_country_code_base > START');

    $bei_country_code = [];
    foreach (WC()->countries->countries as $bei_country_code_key => $bei_country_code_value) {
        $bei_country_code[strtoupper($bei_country_code_key)] = $bei_country_code_value;
    }

    if (!array_key_exists("XI",$bei_country_code)) {
        switch (strtoupper(get_locale())){
            case "HU_HU":
                $bei_country_code["XI"] = "Észak-Írország";
                break;
            case "DE_CH":
            case "DE_DE":
                $bei_country_code["XI"] = "Nordirland";
                break;
            case "AF":
                $bei_country_code["XI"] = "Noord-Ierland";
                break;
            case "FR_BE":
            case "FR_FR":
                $bei_country_code["XI"] = "Irlande du Nord";
                break;
            case "GA":
                $bei_country_code["XI"] = "Tuaisceart Éireann";
                break;
            case "HR":
                $bei_country_code["XI"] = "Sjeverna Irska";
                break;
            case "RO_RO":
                $bei_country_code["XI"] = "Irlanda de Nord";
                break;
            case "GD":
                $bei_country_code["XI"] = "Èirinn a Tuath";
                break;
            case "SK_SK":
                $bei_country_code["XI"] = "Severné Írsko";
                break;
            case "SI_SI":
                $bei_country_code["XI"] = "Severna Irska";
                break;
            case "SR_RS":
                $bei_country_code["XI"] = "Северна Ирска";
                break;
            case "ES_AR":
            case "ES_CL":
            case "ES_CO":
            case "ES_MX":
            case "ES_PE":
            case "ES_PR":
            case "ES_ES":
            case "ES_VE":
                $bei_country_code["XI"] = "Irlanda del Norte";
                break;
            case "SV_SE":
                $bei_country_code["XI"] = "Norra Irland";
                break;
            case "UK":
                $bei_country_code["XI"] = "Північна Ірландія";
                break;
            case "CY":
                $bei_country_code["XI"] = "Gogledd Iwerddon";
                break;
            case "SQ":
                $bei_country_code["XI"] = "Irlanada Veriore";
                break;
            case "BEL":
                $bei_country_code["XI"] = "Паўночная Ірландыя";
                break;
            case "BS_BA":
                $bei_country_code["XI"] = "Sjeverna Irska";
                break;
            case "BG_BG":
                $bei_country_code["XI"] = "Северна Ирландия";
                break;
            case "BAL":
            case "CA":
                $bei_country_code["XI"] = "Irlanda del Nord";
                break;
            case "CS_CZ":
                $bei_country_code["XI"] = "Severní Irsko";
                break;
            case "DA_DK":
                $bei_country_code["XI"] = "Nordirland";
                break;
            case "NL_BE":
            case "NL_NL":
                $bei_country_code["XI"] = "Noord-Ierland";
                break;
            case "ET":
                $bei_country_code["XI"] = "Põhja-Iirimaa";
                break;
            case "FI":
                $bei_country_code["XI"] = "Pohjois-Irlanti";
                break;
            case "EL":
                $bei_country_code["XI"] = "Βόρεια Ιρλανδία";
                break;
            case "IS_IS":
                $bei_country_code["XI"] = "Norður Írland";
                break;
            case "IT_IT":
                $bei_country_code["XI"] = "Irlanda del Nord";
                break;
            case "LT_LT":
                $bei_country_code["XI"] = "Šiaurės Airija";
                break;
            case "LB_LU":
                $bei_country_code["XI"] = "Nordirland";
                break;
            case "LV":
                $bei_country_code["XI"] = "Ziemeļīrija";
                break;
            case "MK_MK":
                $bei_country_code["XI"] = "Северна Ирска";
                break;
            case "NN_NO":
            case "NB_NO":
                $bei_country_code["XI"] = "Nord-Irland";
                break;
            case "PL_PL":
                $bei_country_code["XI"] = "Irlandia Północna";
                break;
            case "PT_BR":
            case "PT_PT":
                $bei_country_code["XI"] = "Irlanda do Norte";
                break;
            default:
                $bei_country_code["XI"] = "Northern Ireland";
        }
    }
   
    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function bei_all_country_code_base > END');

    return $bei_country_code;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return all country code in JSON format
 * @access public
 * @return JSON
 */
function bei_all_country_code_JSON()
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_all_country_code_JSON > START_END');
    return json_encode(bei_all_country_code_base());
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return all country code
 * @access public
 * @return array
 */
function bei_get_all_country_code()
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_get_all_country_code > START_END');
    return bei_all_country_code_base();
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to check required PHP version and extensions (Open SSL, SOAP and XSL) and presence of WooCommerce
 * @access public
*/
function billzone_eu_invoice_all_requirements_are_met() {

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_all_requirements_are_met > START');

    $bei_requirements_are_met = true;
    
    if (intval(substr(phpversion(),0,1)) < 7) {
        $bei_requirements_are_met = false;
        add_action('admin_notices', 'billzone_eu_invoice_wrong_php_version_message' );
    }

    if ( !extension_loaded('openssl') ){
        $bei_requirements_are_met = false;
        add_action('admin_notices', 'billzone_eu_invoice_missing_openssl_message' );
        
    }
        
	if ( !extension_loaded('soap') ){
        $bei_requirements_are_met = false;
        add_action('admin_notices', 'billzone_eu_invoice_missing_soap_message' );
	}
    
    if ( !extension_loaded('xsl') ){
        $bei_requirements_are_met = false;
        add_action('admin_notices', 'billzone_eu_invoice_missing_xsl_message' );
    }

    if(!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))){ 
        $bei_requirements_are_met = false;
        add_action('admin_notices', 'billzone_eu_invoice_missing_woocommerce_message' );
    }
    
    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_all_requirements_are_met > END');

    return $bei_requirements_are_met;
    
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Create "xxx_billzone_eu_invoice" and "xxx_billzone_eu_invoice_log" table and add plugin version to settings table
 * @access public
 */
function bei_create_all_table_and_insert_version()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_create_all_table_and_insert_version > START');

    global $wpdb;

    // create new table named "xxx_billzone_eu_invoice" where xxx is defined by the WP $wpdb->prefix
    $bei_table_exist = $wpdb->get_results("SHOW TABLES LIKE '" . BILLZONE_EU_INVOICE_TABLE_NAME . "'");
    if (empty($bei_table_exist)) {

        $bei_table_name = BILLZONE_EU_INVOICE_TABLE_NAME;

        $bei_charset_collate = $wpdb->get_charset_collate();
        $bei_sql = "CREATE TABLE $bei_table_name (
	                id BIGINT UNSIGNED NOT NULL auto_increment,
	                order_id BIGINT UNSIGNED NOT NULL,
                    order_key VARCHAR(100) DEFAULT '' NOT NULL,
                    request_id VARCHAR(40) DEFAULT '' NOT NULL,
                    transaction_id VARCHAR(40) DEFAULT '' NOT NULL,
	                invoice_number VARCHAR(100) DEFAULT '' NOT NULL,
                    invoice_type VARCHAR(40) DEFAULT '',
                    invoice_url TEXT DEFAULT '',
                    invoice_creation_date DATETIME,
	                PRIMARY KEY  (id),
                    UNIQUE (invoice_number),
                    UNIQUE (request_id)
                    ) $bei_charset_collate;";
        $wpdb->query($bei_sql);

        add_option(BILLZONE_EU_INVOICE_PLUGIN_VERSION_KEY, BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM);
    }

    // create new table named "xxx_billzone_eu_invoice_log" where xxx is defined by the WP $wpdb->prefix
    $bei_log_table_exist = $wpdb->get_results("SHOW TABLES LIKE '" . BILLZONE_EU_INVOICE_LOG_TABLE_NAME . "'");
    if (empty($bei_log_table_exist)) {

        $bei_log_table_name = BILLZONE_EU_INVOICE_LOG_TABLE_NAME;

        $bei_charset_collate = $wpdb->get_charset_collate();
        $bei_sql = "CREATE TABLE $bei_log_table_name (
	                id BIGINT UNSIGNED NOT NULL auto_increment,
	                log_creation_date DATETIME,
                    log_type VARCHAR(10) DEFAULT '',
                    log_text TEXT DEFAULT '',
                    PRIMARY KEY  (id)
                    ) $bei_charset_collate;";
        $wpdb->query($bei_sql);

    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function bei_create_all_table_and_insert_version > END');
}
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Check if the plugin install was correctly done
 * @access public
 * @return bool
 */
function bei_check_if_install_is_OK() {

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_create_all_table_and_insert_version > START');

    global $wpdb;

    $install_is_OK = false;
    $bei_log_table_exist = $wpdb->get_results("SHOW TABLES LIKE '" . BILLZONE_EU_INVOICE_LOG_TABLE_NAME . "'");
    if (billzone_eu_invoice_all_requirements_are_met() && !(empty($bei_log_table_exist))) {
        $install_is_OK = true;
    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function bei_create_all_table_and_insert_version > END');
    return $install_is_OK;
}
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return European countries default currency
 * @access public
 * @var string $bei_country_code two digit string country code
 * @return string
 */
function bei_get_country_default_currency($bei_country_code)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_get_country_default_currency > START');

    switch ($bei_country_code) {
        case 'AL':
            $bei_default_currency = 'ALL';
            break;
        case 'AM':
            $bei_default_currency = 'AMD';
            break;
        case 'AZ':
            $bei_default_currency = 'AZN';
            break;
        case 'BA':
            $bei_default_currency = 'BAM';
            break;
        case 'BG':
            $bei_default_currency = 'BGN';
            break;
        case 'BY':
            $bei_default_currency = 'BYN';
            break;
        case 'LI':
            $bei_default_currency = 'CHF';
            break;
        case 'CH':
            $bei_default_currency = 'CHF';
            break;
        case 'CZ':
            $bei_default_currency = 'CZK';
            break;
        case 'DK':
            $bei_default_currency = 'DKK';
            break;
        case 'AD':
        case 'AT':
        case 'BE':
        case 'CY':
        case 'EE':
        case 'FI':
        case 'FR':
        case 'DE':
        case 'GR':
        case 'IE':
        case 'IT':
        case 'LV':
        case 'LT':
        case 'LU':
        case 'MT':
        case 'MC':
        case 'ME':
        case 'NL':
        case 'PT':
        case 'SM':
        case 'SK':
        case 'SI':
        case 'ES':
        case 'VA':
            $bei_default_currency = 'EUR';
            break;
        case 'GB':
            $bei_default_currency = 'GBP';
            break;
        case 'GE':
            $bei_default_currency = 'GEL';
            break;
        case 'HR':
            $bei_default_currency = 'HRK';
            break;
        case 'HU':
            $bei_default_currency = 'HUF';
            break;
        case 'IS':
            $bei_default_currency = 'ISK';
            break;
        case 'MD':
            $bei_default_currency = 'MDL';
            break;
        case 'MK':
            $bei_default_currency = 'MKD';
            break;
        case 'NO':
            $bei_default_currency = 'NOK';
            break;
        case 'PL':
            $bei_default_currency = 'PLN';
            break;
        case 'RO':
            $bei_default_currency = 'RON';
            break;
        case 'RS':
            $bei_default_currency = 'RSD';
            break;
        case 'RU':
            $bei_default_currency = 'RUB';
            break;
        case 'SE':
            $bei_default_currency = 'SEK';
            break;
        case 'TR':
            $bei_default_currency = 'TRY';
            break;
        case 'UA':
            $bei_default_currency = 'UAH';
            break;
        default:
            $bei_default_currency = "HUF";
            break;
    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function bei_get_country_default_currency > END');

    return $bei_default_currency;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return true if the customer qualifies as a Hungarian taxable entity
 * @access public
 * @var object $bei_entity_to_check the input entity to check
 * @return bool
 */
function bei_Is_Hungarian_Tax_Entity($bei_entity_to_check){

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_Is_Hungarian_Tax_Entity > START');

    $bei_check_result = false;
    if ($bei_entity_to_check['AddressCountryId'] == 'HU') {
        $bei_check_result = true;
    }
    if (!empty($bei_entity_to_check['EUTaxNumber']) && (strlen($bei_entity_to_check['EUTaxNumber']) >= 2) && (strtoupper(substr($bei_entity_to_check['EUTaxNumber'], 0, 2)) == 'HU')) {
        $bei_check_result = true;
    }
    $bei_regexp_pattern_to_check_HU_tax_nr = '/^\d{8}\-[1-5]\-(?:0[2-9]|[13][0-9]|2[02-9]|4[0-4]|51)$/';
    if (preg_match($bei_regexp_pattern_to_check_HU_tax_nr, $bei_entity_to_check['TaxNumber'])) {
        $bei_check_result = true;
    }
    
    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function bei_Is_Hungarian_Tax_Entity > END');

    return $bei_check_result;
}

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return API errors string
 * @access public
 * @var int $bei_error_code the input integer error code returned by billzone.eu API
 * @return array ['code'] -> error code, ['message'] -> error message (translatable), ['error_type'] - > error type
 */
function bei_get_API_error($bei_error_code)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_get_API_error > START');

    switch ($bei_error_code) {
        case BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE:
            $bei_error['code'] = BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE;
            $bei_error['message'] = __('EMPTY', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case BILLZONE_EU_INVOICE_OK_SECURITY_CODE:
            $bei_error['code'] = BILLZONE_EU_INVOICE_OK_SECURITY_CODE;
            $bei_error['message'] = __('OK', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General message';
            break;
        case 50332:
            $bei_error['code'] = 50332;
            $bei_error['message'] = __('The maximum length of the ProductTextIdentifier field must be 20 characters!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 50619:
            $bei_error['code'] = 50619;
            $bei_error['message'] = __('The custom data you entered is incorrect, the exact error is in the response message (ResultCode.Message)', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CustomData error message';
            break;
        case 60001:
            $bei_error['code'] = 60001;
            $bei_error['message'] = __('Setting invoice sequence group is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60002:
            $bei_error['code'] = 60002;
            $bei_error['message'] = __('Currency is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60003:
            $bei_error['code'] = 60003;
            $bei_error['message'] = __('PaymentDueDate is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60004:
            $bei_error['code'] = 60004;
            $bei_error['message'] = __('FulfillmentDate is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60005:
            $bei_error['code'] = 60005;
            $bei_error['message'] = __('FulfillmentDate cannot be 15 days earlier than today', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60006:
            $bei_error['code'] = 60006;
            $bei_error['message'] = __('PaymentMethodId is required.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60007:
            $bei_error['code'] = 60007;
            $bei_error['message'] = __('Payment method connection is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60008:
            $bei_error['code'] = 60008;
            $bei_error['message'] = __('At least one bet line is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60009:
            $bei_error['code'] = 60009;
            $bei_error['message'] = __('Taxation within the Community: VAT can only be 0%', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60010:
            $bei_error['code'] = 60010;
            $bei_error['message'] = __('Currency other than the issuing country. VAT can only be 0%.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60011:
            $bei_error['code'] = 60011;
            $bei_error['message'] = __('The customer name is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60012:
            $bei_error['code'] = 60012;
            $bei_error['message'] = __('The country code of the buyer is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60013:
            $bei_error['code'] = 60013;
            $bei_error['message'] = __('The city is required in the address of the buyer', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60014:
            $bei_error['code'] = 60014;
            $bei_error['message'] = __('In the address of the buyer, the address bar is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60015:
            $bei_error['code'] = 60015;
            $bei_error['message'] = __('The postal code is required in the buyers address', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60016:
            $bei_error['code'] = 60016;
            $bei_error['message'] = __('If there is intra-community taxation, the customers tax number is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60017:
            $bei_error['code'] = 60017;
            $bei_error['message'] = __('If the buyers EU tax number is filled in, it must be formally correct.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60018:
            $bei_error['code'] = 60018;
            $bei_error['message'] = __('If the customers tax number is filled, then it must be formally correct', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60019:
            $bei_error['code'] = 60019;
            $bei_error['message'] = __('If we send a letter to the buyer, you must enter the email address', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60020:
            $bei_error['code'] = 60020;
            $bei_error['message'] = __('If the email address is filled out, it should be formally correct', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60021:
            $bei_error['code'] = 60021;
            $bei_error['message'] = __('The name of the issuer is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60022:
            $bei_error['code'] = 60022;
            $bei_error['message'] = __('The country code of the issuer is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60023:
            $bei_error['code'] = 60023;
            $bei_error['message'] = __('The city is obligatory in the address of the issuer', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60024:
            $bei_error['code'] = 60024;
            $bei_error['message'] = __('The address line is required in the issuers address', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60025:
            $bei_error['code'] = 60025;
            $bei_error['message'] = __('The postal code is required in the issuers address', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60026:
            $bei_error['code'] = 60026;
            $bei_error['message'] = __('OrderNumber field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60027:
            $bei_error['code'] = 60027;
            $bei_error['message'] = __('Notes field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60028:
            $bei_error['code'] = 60028;
            $bei_error['message'] = __('InvoiceDescription field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60029:
            $bei_error['code'] = 60029;
            $bei_error['message'] = __('BankDescription field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60030:
            $bei_error['code'] = 60030;
            $bei_error['message'] = __('CustomerIdentifier field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60031:
            $bei_error['code'] = 60031;
            $bei_error['message'] = __('CustomerName field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60032:
            $bei_error['code'] = 60032;
            $bei_error['message'] = __('CustomerAddressState field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60033:
            $bei_error['code'] = 60033;
            $bei_error['message'] = __('CustomerAddressPostalCode field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60034:
            $bei_error['code'] = 60034;
            $bei_error['message'] = __('CustomerAddressCity field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60035:
            $bei_error['code'] = 60035;
            $bei_error['message'] = __('CustomerAddressLine1 field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60036:
            $bei_error['code'] = 60036;
            $bei_error['message'] = __('CustomerAddressLine2 field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60037:
            $bei_error['code'] = 60037;
            $bei_error['message'] = __('CustomerEUTaxNumber field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60038:
            $bei_error['code'] = 60038;
            $bei_error['message'] = __('CustomerTaxNumber field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60039:
            $bei_error['code'] = 60039;
            $bei_error['message'] = __('CustomerEmailAddress field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60040:
            $bei_error['code'] = 60040;
            $bei_error['message'] = __('ProductName field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60041:
            $bei_error['code'] = 60041;
            $bei_error['message'] = __('ProductStatisticalCode field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60042:
            $bei_error['code'] = 60042;
            $bei_error['message'] = __('If there is EU community taxation, the community tax number of the issuer is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60043:
            $bei_error['code'] = 60043;
            $bei_error['message'] = __('Wrong line item', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60044:
            $bei_error['code'] = 60044;
            $bei_error['message'] = __('In the case of domestic currency billing, you must specify the exchange rate', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60045:
            $bei_error['code'] = 60045;
            $bei_error['message'] = __('In the case of a foreign currency account, at the domestic settlement, the exchange rate must be given', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60046:
            $bei_error['code'] = 60046;
            $bei_error['message'] = __('In the case of a foreign currency account, not in the case of domestic settlement, the exchange rate cannot be specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60047:
            $bei_error['code'] = 60047;
            $bei_error['message'] = __('In the case of a foreign currency account, in the case of non-domestic execution, if the buyer is a company with a Community tax number, the items cannot contain VAT content.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60048:
            $bei_error['code'] = 60048;
            $bei_error['message'] = __('In the case of local currency, no exchange rate can be given', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60049:
            $bei_error['code'] = 60049;
            $bei_error['message'] = __('If the exchange box is displayed, the exchange rate is mandatory!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60050:
            $bei_error['code'] = 60050;
            $bei_error['message'] = __('The maximum length of the group member tax number must be 20 characters!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60051:
            $bei_error['code'] = 60051;
            $bei_error['message'] = __('In the case of a legal entity partner without a tax number, it is forbidden to provide any partner tax number!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60052:
            $bei_error['code'] = 60052;
            $bei_error['message'] = __('The country of the buyer cannot be within the Community (EU).', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60053:
            $bei_error['code'] = 60053;
            $bei_error['message'] = __('In the case of an EU community tax or export invoice, the buyer cannot be domestic.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60054:
            $bei_error['code'] = 60054;
            $bei_error['message'] = __('The country of the buyer and the issuer must be within the Community (EU).', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60055:
            $bei_error['code'] = 60055;
            $bei_error['message'] = __('If you use the EU VAT rate, you have incorrectly stated above that you are not billing within the Community (EU).', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60057:
            $bei_error['code'] = 60057;
            $bei_error['message'] = __('Discount VAT approval field cannot be filled.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60058:
            $bei_error['code'] = 60058;
            $bei_error['message'] = __('In case of reverse charge, the customer must be home.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60059:
            $bei_error['code'] = 60059;
            $bei_error['message'] = __('In the case of reverse charge, the tax number of the buyer is mandatory and the third digit from the back must be 2, 3, 4 or 5.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60060:
            $bei_error['code'] = 60060;
            $bei_error['message'] = __('Use the EU VAT rate when invoicing within the EU community!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60072:
            $bei_error['code'] = 60072;
            $bei_error['message'] = __('The InvoiceLineNote field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60073:
            $bei_error['code'] = 60073;
            $bei_error['message'] = __('ReferenceInvoiceNumber field is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60074:
            $bei_error['code'] = 60074;
            $bei_error['message'] = __('Referenced invoice not exist and for inexistent invoice de delete operation is not permited', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60075:
            $bei_error['code'] = 60075;
            $bei_error['message'] = __('In case of invalidation, the ReferenceInvoiceFulfillmentDate field is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60076:
            $bei_error['code'] = 60076;
            $bei_error['message'] = __('The InvoiceSummary field for the referenced invoice is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60077:
            $bei_error['code'] = 60077;
            $bei_error['message'] = __('Defective Document Type! The invoice document type can only be Void if the final amount is the total of the original invoice with a negative sign.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60078:
            $bei_error['code'] = 60078;
            $bei_error['message'] = __('Defective Document Type! The invoice document type can only be Credit for a total amount of 0 or less if you make a change on the line item.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60079:
            $bei_error['code'] = 60079;
            $bei_error['message'] = __('Defective Document Type: The invoice document type can only be Modifier for a total amount greater than 0 if you made a change on the line item.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60080:
            $bei_error['code'] = 60080;
            $bei_error['message'] = __('If the VAT content is greater than 2M, the customers tax number is mandatory!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60081:
            $bei_error['code'] = 60081;
            $bei_error['message'] = __('There is more than one tax-exempt VAT rate on your account!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60082:
            $bei_error['code'] = 60082;
            $bei_error['message'] = __('The start of the period must be specified on the line if the value of IsPeriodic is true!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60083:
            $bei_error['code'] = 60083;
            $bei_error['message'] = __('The end of the period must be specified on the line if the value of IsPeriodic is true!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60084:
            $bei_error['code'] = 60084;
            $bei_error['message'] = __('The start of the period on the line cannot be specified if the value of IsPeriodic is false!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60085:
            $bei_error['code'] = 60085;
            $bei_error['message'] = __('The end of the period on the line cannot be specified if the value of IsPeriodic is false!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60086:
            $bei_error['code'] = 60086;
            $bei_error['message'] = __('The end of the period should not be less than the beginning!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60087:
            $bei_error['code'] = 60087;
            $bei_error['message'] = __('According to EU directive nr. 2011/83/EU and hungarian ÁFA law nr. § 45/A§, the buyer must be within the EU!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60090:
            $bei_error['code'] = 60090;
            $bei_error['message'] = __('Missing clause parameter!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60091:
            $bei_error['code'] = 60091;
            $bei_error['message'] = __('Only one clause from the same group can be used!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60095:
            $bei_error['code'] = 60095;
            $bei_error['message'] = __('Buyer group ID number is too long!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60110:
            $bei_error['code'] = 60110;
            $bei_error['message'] = __('The size of the invoice attachments exceeded the limit set in the package', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60111:
            $bei_error['code'] = 60111;
            $bei_error['message'] = __('Invoice attachment file name too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60112:
            $bei_error['code'] = 60112;
            $bei_error['message'] = __('The invoice attachment name contains invalid characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60120:
            $bei_error['code'] = 60120;
            $bei_error['message'] = __('The invoice is obliged to provide NAV data based on the data content. To do this, you need to enter the NAV Online Account registration username, replacement key and signature key information on the Nav Data Service Settings page.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60121:
            $bei_error['code'] = 60121;
            $bei_error['message'] = __('Transaction ID is too long!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60122:
            $bei_error['code'] = 60122;
            $bei_error['message'] = __('The invoice template you choose requires a higher subscription!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60123:
            $bei_error['code'] = 60123;
            $bei_error['message'] = __('In the case of item prepayment clause, the transaction is also mandatory!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60124:
            $bei_error['code'] = 60124;
            $bei_error['message'] = __('The total gross amount of items in the line item group exceeds the maximum allowed value!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60125:
            $bei_error['code'] = 60125;
            $bei_error['message'] = __('The total amount of VAT in the items in the line item group exceeds the maximum allowed value!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60126:
            $bei_error['code'] = 60126;
            $bei_error['message'] = __('The sum total of items in the line item group exceeds the maximum allowable value!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60127:
            $bei_error['code'] = 60127;
            $bei_error['message'] = __('All line items are required to have a GroupIndex greater than zero if the invoice has a line item group associated with it!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60128:
            $bei_error['code'] = 60128;
            $bei_error['message'] = __('There is no Item Line Group with the GroupIndex specified in the Item Line!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60129:
            $bei_error['code'] = 60129;
            $bei_error['message'] = __('If there is no item line group associated with the account, you must not enter a group index greater than zero on the item line!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60130:
            $bei_error['code'] = 60130;
            $bei_error['message'] = __('GroupIndex must be a positive number greater than zero in the line item group!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60131:
            $bei_error['code'] = 60131;
            $bei_error['message'] = __('There is no item with the GroupIndex specified in the item row group!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60132:
            $bei_error['code'] = 60132;
            $bei_error['message'] = __('BankName is mandatory when using BankData payment method!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60134:
            $bei_error['code'] = 60134;
            $bei_error['message'] = __('The invoice to be canceled/credited/modified has not yet been finalized in the system! Check the NAV status of the referenced invoice!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60135:
            $bei_error['code'] = 60135;
            $bei_error['message'] = __('If you enter a group member tax number, you must also enter a tax number!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'InvoiceValidator error message';
            break;
        case 60200:
            $bei_error['code'] = 60200;
            $bei_error['message'] = __('It is not possible to specify a rate for the same item line!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'ConvertExternalInvoiceValidator error message';
            break;
        case 60201:
            $bei_error['code'] = 60201;
            $bei_error['message'] = __('It is mandatory to specify the exchange rate in the line item!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'ConvertExternalInvoiceValidator error message';
            break;
        case 60202:
            $bei_error['code'] = 60202;
            $bei_error['message'] = __('You cannot enter a rate for the same currency!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'ConvertExternalInvoiceValidator error message';
            break;
        case 60203:
            $bei_error['code'] = 60203;
            $bei_error['message'] = __('Exchange rate is required!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'ConvertExternalInvoiceValidator error message';
            break;
        case 61001:
            $bei_error['code'] = 61001;
            $bei_error['message'] = __('OK', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 61002:
            $bei_error['code'] = 61002;
            $bei_error['message'] = __('Unknown error', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 61003:
            $bei_error['code'] = 61003;
            $bei_error['message'] = __('SecurityToken is wrong', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Request ResultCode error message';
            break;
        case 61004:
            $bei_error['code'] = 61004;
            $bei_error['message'] = __('SelfBillingSecurityToken is broken', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Request ResultCode error message';
            break;
        case 61005:
            $bei_error['code'] = 61005;
            $bei_error['message'] = __('Incorrect RequestId', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Request ResultCode error message';
            break;
        case 61006:
            $bei_error['code'] = 61006;
            $bei_error['message'] = __('Request cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Request ResultCode error message';
            break;
        case 61007:
            $bei_error['code'] = 61007;
            $bei_error['message'] = __('The account number you entered does not include an invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Request ResultCode error message';
            break;
        case 61502:
            $bei_error['code'] = 61502;
            $bei_error['message'] = __('Internal communication failure', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 61503:
            $bei_error['code'] = 61503;
            $bei_error['message'] = __('Billgate cannot be used in the current subscription package', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Subcsription error message';
            break;
        case 61504:
            $bei_error['code'] = 61504;
            $bei_error['message'] = __('You have reached a subscription limit, you can not issue an invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Subcsription error message';
            break;
        case 61505:
            $bei_error['code'] = 61505;
            $bei_error['message'] = __('You do not have the right to use the self-billing feature with your current subscription', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Subcsription error message';
            break;
        case 61506:
            $bei_error['code'] = 61506;
            $bei_error['message'] = __('Timestamp provider error', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 61507:
            $bei_error['code'] = 61507;
            $bei_error['message'] = __('Incorrect NAV login data! You can find more information in the Message!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 61508:
            $bei_error['code'] = 61508;
            $bei_error['message'] = __('NAV communication error! You can find more information in the Message!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 61509:
            $bei_error['code'] = 61509;
            $bei_error['message'] = __('The system is under maintenance, please try again later!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'General error message';
            break;
        case 62001:
            $bei_error['code'] = 62001;
            $bei_error['message'] = __('Unknown IssuerAddress type', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62002:
            $bei_error['code'] = 62002;
            $bei_error['message'] = __('Unknown Bank Type', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62003:
            $bei_error['code'] = 62003;
            $bei_error['message'] = __('The bank could not be loaded based on the specified ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62004:
            $bei_error['code'] = 62004;
            $bei_error['message'] = __('Could not load default bank', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62005:
            $bei_error['code'] = 62005;
            $bei_error['message'] = __('The site could not be loaded based on the specified ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62006:
            $bei_error['code'] = 62006;
            $bei_error['message'] = __('Could not load default location', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62007:
            $bei_error['code'] = 62007;
            $bei_error['message'] = __('Unknown currency', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62008:
            $bei_error['code'] = 62008;
            $bei_error['message'] = __('The currency could not be loaded based on the specified ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62009:
            $bei_error['code'] = 62009;
            $bei_error['message'] = __('Could not load default currency', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62010:
            $bei_error['code'] = 62010;
            $bei_error['message'] = __('Unknown invoice sequence group prefix', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62011:
            $bei_error['code'] = 62011;
            $bei_error['message'] = __('No currency specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62012:
            $bei_error['code'] = 62012;
            $bei_error['message'] = __('No bank specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62013:
            $bei_error['code'] = 62013;
            $bei_error['message'] = __('No location specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62014:
            $bei_error['code'] = 62014;
            $bei_error['message'] = __('There is no invoice item specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62015:
            $bei_error['code'] = 62015;
            $bei_error['message'] = __('The quantitative unit is unknown', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62016:
            $bei_error['code'] = 62016;
            $bei_error['message'] = __('Product ID unknown', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62017:
            $bei_error['code'] = 62017;
            $bei_error['message'] = __('No customer specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62018:
            $bei_error['code'] = 62018;
            $bei_error['message'] = __('The buyer could not be loaded based on the specified ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62019:
            $bei_error['code'] = 62019;
            $bei_error['message'] = __('Unknown customer type', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62020:
            $bei_error['code'] = 62020;
            $bei_error['message'] = __('Unknown VAT key', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62021:
            $bei_error['code'] = 62021;
            $bei_error['message'] = __('It should be an electronic invoice sequence group', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62023:
            $bei_error['code'] = 62023;
            $bei_error['message'] = __('Incorrect TransactionId', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62024:
            $bei_error['code'] = 62024;
            $bei_error['message'] = __('The transaction ID you entered already has a transaction but is still being processed', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62025:
            $bei_error['code'] = 62025;
            $bei_error['message'] = __('Product VAT cannot be 0%, use the VAT code (VatTaxRateCode) on InvoiceLine2!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62026:
            $bei_error['code'] = 62026;
            $bei_error['message'] = __('VAT code is not valid!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62029:
            $bei_error['code'] = 62029;
            $bei_error['message'] = __('VAT grouper cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62030:
            $bei_error['code'] = 62030;
            $bei_error['message'] = __('You cannot use an invoice sequence group type of imported account numbers', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62031:
            $bei_error['code'] = 62031;
            $bei_error['message'] = __('Only imported invoice sequence group type is permited', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62032:
            $bei_error['code'] = 62032;
            $bei_error['message'] = __('Only net or gross unit price can be specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62033:
            $bei_error['code'] = 62033;
            $bei_error['message'] = __('Net or gross unit price is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62034:
            $bei_error['code'] = 62034;
            $bei_error['message'] = __('The gross unit price must not be between -1 and 0, or between 0 and 1', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62035:
            $bei_error['code'] = 62035;
            $bei_error['message'] = __('SourceSystem field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62040:
            $bei_error['code'] = 62040;
            $bei_error['message'] = __('The given invoice number already exists', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62041:
            $bei_error['code'] = 62041;
            $bei_error['message'] = __('Referenced invoice not exist and for inexistent invoice de delete operation is not permited', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62042:
            $bei_error['code'] = 62042;
            $bei_error['message'] = __('The specified InvoiceDocumentType is invalid', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62043:
            $bei_error['code'] = 62043;
            $bei_error['message'] = __('The reference invoice delivery time has not been filled in and the referenced invoice has not been found', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62044:
            $bei_error['code'] = 62044;
            $bei_error['message'] = __('ReferenceInvoiceNumber field is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62045:
            $bei_error['code'] = 62045;
            $bei_error['message'] = __('InvoiceCourierTypeId field can only add value if SendInvoiceToCustomer is false', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62046:
            $bei_error['code'] = 62046;
            $bei_error['message'] = __('Unknown clause identifier', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62047:
            $bei_error['code'] = 62047;
            $bei_error['message'] = __('Missing clause parameter', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62048:
            $bei_error['code'] = 62048;
            $bei_error['message'] = __('Inappropriate clause type', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62049:
            $bei_error['code'] = 62049;
            $bei_error['message'] = __('RegisterExternalInvoice: UnitIdentifier or UnitText cannot be specified at the same time', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62050:
            $bei_error['code'] = 62050;
            $bei_error['message'] = __('IssuerSoftware/SoftwareId is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62051:
            $bei_error['code'] = 62051;
            $bei_error['message'] = __('IssuerSoftware/SoftwareId is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62052:
            $bei_error['code'] = 62052;
            $bei_error['message'] = __('IssuerSoftware/SoftwareName is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62053:
            $bei_error['code'] = 62053;
            $bei_error['message'] = __('IssuerSoftware.SoftwareOperation is not valid', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62054:
            $bei_error['code'] = 62054;
            $bei_error['message'] = __('IssuerSoftware/SoftwareMainVersion is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62055:
            $bei_error['code'] = 62055;
            $bei_error['message'] = __('IssuerSoftware/SoftwareDevName is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62056:
            $bei_error['code'] = 62056;
            $bei_error['message'] = __('IssuerSoftware/SoftwareDevContact is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62057:
            $bei_error['code'] = 62057;
            $bei_error['message'] = __('IssuerSoftware/SoftwareDevCountryCode is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62058:
            $bei_error['code'] = 62058;
            $bei_error['message'] = __('IssuerSoftware/SoftwareDevTaxNumber is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62059:
            $bei_error['code'] = 62059;
            $bei_error['message'] = __('OtherInvoiceData is missing from RegisterExternalInvoiceTransaction', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62060:
            $bei_error['code'] = 62060;
            $bei_error['message'] = __('IssuerSoftware is missing from RegisterExternalInvoiceTransaction/ConvertExternalInvoiceTransaction', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62061:
            $bei_error['code'] = 62061;
            $bei_error['message'] = __('RegisterExternalInvoiceTransaction Faulty Bank Data Type (BankData Type Only)', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62062:
            $bei_error['code'] = 62062;
            $bei_error['message'] = __('RegisterExternalInvoiceTransaction Faulty Site Data Type (IssuerAddressData Type Only)', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62063:
            $bei_error['code'] = 62063;
            $bei_error['message'] = __('RegisterExternalInvoice-type invoice sequence group is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62064:
            $bei_error['code'] = 62064;
            $bei_error['message'] = __('BankIdentifierWithAccountNumber can only be used for ConvertExternalInvoice', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62065:
            $bei_error['code'] = 62065;
            $bei_error['message'] = __('Attachments are not allowed for paper invoices', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62066:
            $bei_error['code'] = 62066;
            $bei_error['message'] = __('Invoice attachment, file name not specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62067:
            $bei_error['code'] = 62067;
            $bei_error['message'] = __('Invoice attachment, file is empty', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62068:
            $bei_error['code'] = 62068;
            $bei_error['message'] = __('In the case of foreign currency invoces, only Hungarian companies may request exchange rates', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62100:
            $bei_error['code'] = 62100;
            $bei_error['message'] = __('No referenced invoice specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62101:
            $bei_error['code'] = 62101;
            $bei_error['message'] = __('The specified invoice could not be found', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62102:
            $bei_error['code'] = 62102;
            $bei_error['message'] = __('No currency specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62103:
            $bei_error['code'] = 62103;
            $bei_error['message'] = __('Could not load currency with the given ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62104:
            $bei_error['code'] = 62104;
            $bei_error['message'] = __('No prefix prefix specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62105:
            $bei_error['code'] = 62105;
            $bei_error['message'] = __('Unknown prefix prefix', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62106:
            $bei_error['code'] = 62106;
            $bei_error['message'] = __('The total amount cannot be 0 or negative', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62107:
            $bei_error['code'] = 62107;
            $bei_error['message'] = __('No customer specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62108:
            $bei_error['code'] = 62108;
            $bei_error['message'] = __('The buyer could not be loaded based on the specified ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62109:
            $bei_error['code'] = 62109;
            $bei_error['message'] = __('No payment method specified', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62110:
            $bei_error['code'] = 62110;
            $bei_error['message'] = __('The payment method could not be loaded based on the identifier provided', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62111:
            $bei_error['code'] = 62111;
            $bei_error['message'] = __('Payment method is not available for cash deposit', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62112:
            $bei_error['code'] = 62112;
            $bei_error['message'] = __('In OtherInvoiceLineData, you must specify LineNumberReference for a cancel/change/credit invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62113:
            $bei_error['code'] = 62113;
            $bei_error['message'] = __('OtherInvoiceLineData forbidden to specify LineNumberReference for normal account (no cancellation/credit/modifier)', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62114:
            $bei_error['code'] = 62114;
            $bei_error['message'] = __('Xml validation error during invoice creation, check the Message tag!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62115:
            $bei_error['code'] = 62115;
            $bei_error['message'] = __('Receiver tax number, NAV validation error', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62116:
            $bei_error['code'] = 62116;
            $bei_error['message'] = __('Issuing tax number, NAV validation error', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62117:
            $bei_error['code'] = 62117;
            $bei_error['message'] = __('Receiver number, NAV query error', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62118:
            $bei_error['code'] = 62118;
            $bei_error['message'] = __('Issuers tax number, NAV query error', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62119:
            $bei_error['code'] = 62119;
            $bei_error['message'] = __('Technical invalidation of RegisterExternal invoice, referenced invoice not found', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62121:
            $bei_error['code'] = 62121;
            $bei_error['message'] = __('For self-billing, the recipients tax number must match the tax number of the company requesting the request', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 62122:
            $bei_error['code'] = 62122;
            $bei_error['message'] = __('PEPPOL XML validation error while creating invoice, check Message tag!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 63001:
            $bei_error['code'] = 63001;
            $bei_error['message'] = __('The invoice number cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoice ResultCode error message';
            break;
        case 63002:
            $bei_error['code'] = 63002;
            $bei_error['message'] = __('There is no invoice for the specified number', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoice ResultCode error message';
            break;
        case 63003:
            $bei_error['code'] = 63003;
            $bei_error['message'] = __('The query cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoice ResultCode error message';
            break;
        case 63004:
            $bei_error['code'] = 63004;
            $bei_error['message'] = __('Incorrect document type', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoice ResultCode error message';
            break;
        case 63006:
            $bei_error['code'] = 63006;
            $bei_error['message'] = __('The requested type of attachment does not exist for the invoice (if you set up the EN16931 CII XML Generation for your invoice sequence group, then an XML based attachment will be generated for accounts created with this account array)', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoices ResultCode error message';
            break;
        case 63007:
            $bei_error['code'] = 63007;
            $bei_error['message'] = __('RegisterExternal does not create an invoice in Billzone.eu.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoice ResultCode error message';
            break;
        case 63100:
            $bei_error['code'] = 63100;
            $bei_error['message'] = __('No transaction ID provided.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 63122:
            $bei_error['code'] = 63122;
            $bei_error['message'] = __('The VAT Total field converted to the local currency (InvoiceLocalVatTotalAmount) is required if the invoice currency is different from the local currency.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 63123:
            $bei_error['code'] = 63123;
            $bei_error['message'] = __('The VAT Value Value field (LocalVatAmount) converted to local currency VAT grouping is required if the invoice currency is different from the local currency.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 63124:
            $bei_error['code'] = 63124;
            $bei_error['message'] = __('Quantity Unit Identifier (UnitIdentifier) ​​and Quantity Unit Name (UnitText) cannot be used in a line item!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 63125:
            $bei_error['code'] = 63125;
            $bei_error['message'] = __('It is mandatory to enter a Quantity Unit Identifier (UnitIdentifier) ​​or Name (UnitText) in the item line!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Transaction ResultCode error message';
            break;
        case 63126:
            $bei_error['code'] = 63126;
            $bei_error['message'] = __('SoftwareDevName is required!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'ConvertExternalInvoiceValidator error message';
            break;
        case 63127:
            $bei_error['code'] = 63127;
            $bei_error['message'] = __('SoftwareMainVersion is required!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'ConvertExternalInvoiceValidator error message';
            break;
        case 63128:
            $bei_error['code'] = 63128;
            $bei_error['message'] = __('SoftwareDevName is required if you are using a client version other than Nav v1.1!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'RegisterExternalInvoiceValidator error message';
            break;
        case 63129:
            $bei_error['code'] = 63129;
            $bei_error['message'] = __('SoftwareMainVersion is required if you are using a client version other than Nav v1.1!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'RegisterExternalInvoiceValidator error message';
            break;
        case 63130:
            $bei_error['code'] = 63130;
            $bei_error['message'] = __('Wrong Nav client version!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'RegisterExternalInvoiceValidator error message';
            break;
        case 64001:
            $bei_error['code'] = 64001;
            $bei_error['message'] = __('The specified filter specification is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoices ResultCode error message';
            break;
        case 64002:
            $bei_error['code'] = 64002;
            $bei_error['message'] = __('The query cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadInvoices ResultCode error message';
            break;
        case 65000:
            $bei_error['code'] = 65000;
            $bei_error['message'] = __('The receipt block is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CashReceiptValidator error message';
            break;
        case 65001:
            $bei_error['code'] = 65001;
            $bei_error['message'] = __('The display name field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CashReceiptValidator error message';
            break;
        case 65002:
            $bei_error['code'] = 65002;
            $bei_error['message'] = __('The cashier name field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CashReceiptValidator error message';
            break;
        case 65003:
            $bei_error['code'] = 65003;
            $bei_error['message'] = __('The Payer Name field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CashReceiptValidator error message';
            break;
        case 65004:
            $bei_error['code'] = 65004;
            $bei_error['message'] = __('The document number field identifying the contributor is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CashReceiptValidator error message';
            break;
        case 65005:
            $bei_error['code'] = 65005;
            $bei_error['message'] = __('The buyer name field is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CashReceiptValidator error message';
            break;
        case 66001:
            $bei_error['code'] = 66001;
            $bei_error['message'] = __('The receipt number cannot be zero', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadCashReceipt ResultCode error message';
            break;
        case 66002:
            $bei_error['code'] = 66002;
            $bei_error['message'] = __('The specified serial number does not include a receipt', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadCashReceipt ResultCode error message';
            break;
        case 66003:
            $bei_error['code'] = 66003;
            $bei_error['message'] = __('The query cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'DownloadCashReceipt ResultCode error message';
            break;
        case 67000:
            $bei_error['code'] = 67000;
            $bei_error['message'] = __('The query cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'GetNavOnlineszamlaStatus ResultCode error message';
            break;
        case 67001:
            $bei_error['code'] = 67001;
            $bei_error['message'] = __('The invoice number cannot be null', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'GetNavOnlineszamlaStatus ResultCode error message';
            break;
        case 67002:
            $bei_error['code'] = 67002;
            $bei_error['message'] = __('The specified invoice could not be found', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'GetNavOnlineszamlaStatus ResultCode error message';
            break;
        case 94800:
            $bei_error['code'] = 94800;
            $bei_error['message'] = __('Only aggregate parent companies are allowed the RegisterChildCompany method', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 94801:
            $bei_error['code'] = 94801;
            $bei_error['message'] = __('The RegisterCompany method is only allowed if there is a separate agreement with the operator of Billzone.eu', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 94810:
            $bei_error['code'] = 94810;
            $bei_error['message'] = __('The information provided has already registered the company in the system', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 94850:
            $bei_error['code'] = 94850;
            $bei_error['message'] = __('The acceptance of the General Terms and Conditions is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 94851:
            $bei_error['code'] = 94851;
            $bei_error['message'] = __('Adoption of data management regulations is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95000:
            $bei_error['code'] = 95000;
            $bei_error['message'] = __('The company name field is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95001:
            $bei_error['code'] = 95001;
            $bei_error['message'] = __('The company name you entered cannot be longer than 60 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95002:
            $bei_error['code'] = 95002;
            $bei_error['message'] = __('Incorrect business name! If you are a sole proprietor, enter your name as required by law: János Kovács e.v. (No. 12345678)', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95004:
            $bei_error['code'] = 95004;
            $bei_error['message'] = __('A valid ID is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95005:
            $bei_error['code'] = 95005;
            $bei_error['message'] = __('The business list number cannot be longer than 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95006:
            $bei_error['code'] = 95006;
            $bei_error['message'] = __('The specified community tax number has the wrong format', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95007:
            $bei_error['code'] = 95007;
            $bei_error['message'] = __('The community tax number cannot be longer than 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95008:
            $bei_error['code'] = 95008;
            $bei_error['message'] = __('Contact email is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95010:
            $bei_error['code'] = 95010;
            $bei_error['message'] = __('Contact email address incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95011:
            $bei_error['code'] = 95011;
            $bei_error['message'] = __('The contact email address cannot be longer than 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95012:
            $bei_error['code'] = 95012;
            $bei_error['message'] = __('A contact phone number is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95014:
            $bei_error['code'] = 95014;
            $bei_error['message'] = __('Wrong contact phone number', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95015:
            $bei_error['code'] = 95015;
            $bei_error['message'] = __('The contact phone number you entered cannot be longer than 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95016:
            $bei_error['code'] = 95016;
            $bei_error['message'] = __('The contact name field is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95017:
            $bei_error['code'] = 95017;
            $bei_error['message'] = __('The contact name cannot be longer than 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95046:
            $bei_error['code'] = 95046;
            $bei_error['message'] = __('The email address on the issued invoice is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95054:
            $bei_error['code'] = 95054;
            $bei_error['message'] = __('A tax number is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95055:
            $bei_error['code'] = 95055;
            $bei_error['message'] = __('The tax number cannot be longer than 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95056:
            $bei_error['code'] = 95056;
            $bei_error['message'] = __('Incorrect tax number format', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95060:
            $bei_error['code'] = 95060;
            $bei_error['message'] = __('The county cannot be longer than 30 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95061:
            $bei_error['code'] = 95061;
            $bei_error['message'] = __('The zip code field is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95062:
            $bei_error['code'] = 95062;
            $bei_error['message'] = __('The specified zip code cannot be longer than 20 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95063:
            $bei_error['code'] = 95063;
            $bei_error['message'] = __('City is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95064:
            $bei_error['code'] = 95064;
            $bei_error['message'] = __('The city field cannot be longer than 40 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95065:
            $bei_error['code'] = 95065;
            $bei_error['message'] = __('Heading 1 is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95066:
            $bei_error['code'] = 95066;
            $bei_error['message'] = __('Headline 1 field cannot be longer than 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95068:
            $bei_error['code'] = 95068;
            $bei_error['message'] = __('Headline 2 fields cannot be longer than 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95070:
            $bei_error['code'] = 95070;
            $bei_error['message'] = __('Incorrect country id', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95071:
            $bei_error['code'] = 95071;
            $bei_error['message'] = __('Wrong business logo. Supported types: jpg, bmp, png. Supported sizes are 217x85px and 434x170px.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95072:
            $bei_error['code'] = 95072;
            $bei_error['message'] = __('Invalid user unique identifier', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95074:
            $bei_error['code'] = 95074;
            $bei_error['message'] = __('The email address you entered and the email address of the user associated with the unique ID do not match', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95076:
            $bei_error['code'] = 95076;
            $bei_error['message'] = __('Email is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95077:
            $bei_error['code'] = 95077;
            $bei_error['message'] = __('The email address you entered is not in valid format', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95078:
            $bei_error['code'] = 95078;
            $bei_error['message'] = __('The email address you entered is too long', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95079:
            $bei_error['code'] = 95079;
            $bei_error['message'] = __('Your message is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95080:
            $bei_error['code'] = 95080;
            $bei_error['message'] = __('A role is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95081:
            $bei_error['code'] = 95081;
            $bei_error['message'] = __('Incorrect role ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95083:
            $bei_error['code'] = 95083;
            $bei_error['message'] = __('An invoice sequence group prefix is ​​required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95085:
            $bei_error['code'] = 95085;
            $bei_error['message'] = __('The specified invoice sequence group array prefix format is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95086:
            $bei_error['code'] = 95086;
            $bei_error['message'] = __('The specified invoice array prefix cannot be longer than 8 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95088:
            $bei_error['code'] = 95088;
            $bei_error['message'] = __('The number of fixed digits in the serial number is mandatory', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95090:
            $bei_error['code'] = 95090;
            $bei_error['message'] = __('The sequence number must be between 3 and 8 fixed digits', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95092:
            $bei_error['code'] = 95092;
            $bei_error['message'] = __('Invalid invoice template ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95094:
            $bei_error['code'] = 95094;
            $bei_error['message'] = __('There is no such type of invoice block', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95096:
            $bei_error['code'] = 95096;
            $bei_error['message'] = __('The maximum length of a bank name is 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95097:
            $bei_error['code'] = 95097;
            $bei_error['message'] = __('An interface payment method ID is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95098:
            $bei_error['code'] = 95098;
            $bei_error['message'] = __('The maximum length of the interface payment method identifier is 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95100:
            $bei_error['code'] = 95100;
            $bei_error['message'] = __('The maximum length of a bank account number is 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95102:
            $bei_error['code'] = 95102;
            $bei_error['message'] = __('The specified IBAN is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95104:
            $bei_error['code'] = 95104;
            $bei_error['message'] = __('The maximum length of an IBAN is 80 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95106:
            $bei_error['code'] = 95106;
            $bei_error['message'] = __('Payment method is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95108:
            $bei_error['code'] = 95108;
            $bei_error['message'] = __('The maximum length of the payment method is 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95110:
            $bei_error['code'] = 95110;
            $bei_error['message'] = __('Your payment method code is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95112:
            $bei_error['code'] = 95112;
            $bei_error['message'] = __('Your payment method code cannot be longer than 3 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95114:
            $bei_error['code'] = 95114;
            $bei_error['message'] = __('The maximum display name for a bank is 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95116:
            $bei_error['code'] = 95116;
            $bei_error['message'] = __('Quantity unit name is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95118:
            $bei_error['code'] = 95118;
            $bei_error['message'] = __('The maximum unit name is 50 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95119:
            $bei_error['code'] = 95119;
            $bei_error['message'] = __('The specified Quantity Unit ID already exists!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95120:
            $bei_error['code'] = 95120;
            $bei_error['message'] = __('The maximum length of the unit identifier is 20 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95121:
            $bei_error['code'] = 95121;
            $bei_error['message'] = __('The specified NAV Quantity Unit ID is incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95122:
            $bei_error['code'] = 95122;
            $bei_error['message'] = __('Username is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95124:
            $bei_error['code'] = 95124;
            $bei_error['message'] = __('The username cannot be longer than 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95126:
            $bei_error['code'] = 95126;
            $bei_error['message'] = __('Password is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95127:
            $bei_error['code'] = 95127;
            $bei_error['message'] = __('The maximum length of the password is 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95128:
            $bei_error['code'] = 95128;
            $bei_error['message'] = __('Signing key is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95129:
            $bei_error['code'] = 95129;
            $bei_error['message'] = __('The signature key you entered cannot be longer than 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95130:
            $bei_error['code'] = 95130;
            $bei_error['message'] = __('A replacement key is required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95131:
            $bei_error['code'] = 95131;
            $bei_error['message'] = __('The replacement key cannot be longer than 100 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95132:
            $bei_error['code'] = 95132;
            $bei_error['message'] = __('Enabled integration IP address (s) are required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95133:
            $bei_error['code'] = 95133;
            $bei_error['message'] = __('The maximum length of IPs allowed for integration IPs is 250 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95134:
            $bei_error['code'] = 95134;
            $bei_error['message'] = __('The provided integration IP address (s) are invalid', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95135:
            $bei_error['code'] = 95135;
            $bei_error['message'] = __('Enabled self-billing IP address (s) are required', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95136:
            $bei_error['code'] = 95136;
            $bei_error['message'] = __('The maximum length of allowed self-billing IP address (s) is 250 characters', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95137:
            $bei_error['code'] = 95137;
            $bei_error['message'] = __('The self-billing IP address (s) you provided are incorrect', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95138:
            $bei_error['code'] = 95138;
            $bei_error['message'] = __('The email address displayed on the specified API accounts is malformed', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95139:
            $bei_error['code'] = 95139;
            $bei_error['message'] = __('Invoice sequence group type cannot be changed', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95140:
            $bei_error['code'] = 95140;
            $bei_error['message'] = __('You cannot change the number of fixed digits in an invoice block number', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95141:
            $bei_error['code'] = 95141;
            $bei_error['message'] = __('There is no form of payment with the ID you provided', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95142:
            $bei_error['code'] = 95142;
            $bei_error['message'] = __('There is no site with the specified ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95143:
            $bei_error['code'] = 95143;
            $bei_error['message'] = __('NAV technical data can only be interpreted in the case of a company based in Hungary.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95144:
            $bei_error['code'] = 95144;
            $bei_error['message'] = __('In the case of a paper-based invoice sequence group, the number of copies can be one of the following values: 2, 3, 4!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95145:
            $bei_error['code'] = 95145;
            $bei_error['message'] = __('Invalid subscription package ID!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95146:
            $bei_error['code'] = 95146;
            $bei_error['message'] = __('You cannot create multiple invoice sequence group with the same prefix!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        case 95147:
            $bei_error['code'] = 95147;
            $bei_error['message'] = __('At least one user must be assigned to the company!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'CompanyValidator error message';
            break;
        default:
            $bei_error['code'] = $bei_error_code;
            $bei_error['message'] = __('Unknown error code', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_error['error_type'] = 'Unknown error code';
            break;
    }
       
    
    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function bei_get_API_error > END');

    return $bei_error;

}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * XML parser function
 * @access public
 * @param string $XML xml type input string
 * @return array
 */
function bei_XMLtoArray($XML)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#99A: function bei_XMLtoArray > START');

    $xml_parser = xml_parser_create();
    xml_parse_into_struct($xml_parser, $XML, $vals);
    xml_parser_free($xml_parser);

    $_tmp = '';
    foreach ($vals as $xml_elem) {
        $x_tag = $xml_elem['tag'];
        $x_level = $xml_elem['level'];
        $x_type = $xml_elem['type'];
        if ($x_level != 1 && $x_type == 'close') {
            if (isset($multi_key[$x_tag][$x_level])) {
                $multi_key[$x_tag][$x_level] = 1;
            } else {
                $multi_key[$x_tag][$x_level] = 0;
            }
        }
        if ($x_level != 1 && $x_type == 'complete') {
            if ($_tmp == $x_tag) {
                $multi_key[$x_tag][$x_level] = 1;
            }
            $_tmp = $x_tag;
        }
    }

    foreach ($vals as $xml_elem) {
        $x_tag = $xml_elem['tag'];
        $x_level = $xml_elem['level'];
        $x_type = $xml_elem['type'];
        if ($x_type == 'open') {
            $level[$x_level] = $x_tag;
        }
        $start_level = 1;
        $php_stmt = '$xml_array';
        if ($x_type == 'close' && $x_level != 1) {
            $multi_key[$x_tag][$x_level]++;
        }
        while ($start_level < $x_level) {
            $php_stmt .= '[$level[' . $start_level . ']]';
            if (isset($multi_key[$level[$start_level]][$start_level]) && $multi_key[$level[$start_level]][$start_level]) {
                $php_stmt .= '[' . ($multi_key[$level[$start_level]][$start_level] - 1) . ']';
            }
            $start_level++;
        }
        $add = '';
        if (isset($multi_key[$x_tag][$x_level]) && $multi_key[$x_tag][$x_level] && ($x_type == 'open' || $x_type == 'complete')) {
            if (!isset($multi_key2[$x_tag][$x_level])) {
                $multi_key2[$x_tag][$x_level] = 0;
            } else {
                $multi_key2[$x_tag][$x_level]++;
            }
            $add = '[' . $multi_key2[$x_tag][$x_level] . ']';
        }
        if (isset($xml_elem['value']) && trim($xml_elem['value']) != '' && !array_key_exists('attributes', $xml_elem)) {
            if ($x_type == 'open') {
                $php_stmt_main = $php_stmt . '[$x_type]' . $add . '[\'EXCHANGERATE\'] = $xml_elem[\'value\'];';
            } else {
                $php_stmt_main = $php_stmt . '[$x_tag]' . $add . ' = $xml_elem[\'value\'];';
            }
            eval($php_stmt_main);
        }
        if (array_key_exists('attributes', $xml_elem)) {
            if (isset($xml_elem['value'])) {
                $php_stmt_main = $php_stmt . '[$x_tag]' . $add . '[\'EXCHANGERATE\'] = $xml_elem[\'value\'];';
                eval($php_stmt_main);
            }
            foreach ($xml_elem['attributes'] as $key => $value) {
                $php_stmt_att = $php_stmt . '[$x_tag]' . $add . '[$key] = $value;';
                eval($php_stmt_att);
            }
        }
    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#99B: function bei_XMLtoArray > END');
    return $xml_array;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Escape some special HTML characters
 * @access public
 * @param string $bei_in_data input HTML string
 * @return string
 */
function bei_esc_html($bei_in_data)
{
    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function bei_esc_html > START_END');
    return str_replace("/", "\/", str_replace("'", "\'", str_replace('"', '\"', $bei_in_data)));
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Return all billzone.eu API returned country code
 * @access public
 * @param object $bei_all_tax_type object input
 * @return array
 */
function bei_get_all_API_returned_tax_country_code($bei_all_tax_type)
{

    billzone_eu_invoice_logme(__FILE__, 'API - OK', '#1: function bei_get_all_API_returned_tax_country_code > START');

    // Collect all available tax country code
    $bei_all_available_tax_country_code = array();
    foreach ($bei_all_tax_type as $bei_one_tax_type) {
        if (!array_key_exists(strtoupper($bei_one_tax_type->CountryId), $bei_all_available_tax_country_code)) {
            $bei_all_available_tax_country_code[strtoupper($bei_one_tax_type->CountryId)] = strtoupper($bei_one_tax_type->CountryId);
        }
    }

    billzone_eu_invoice_logme(__FILE__, 'API - OK', '#2: function bei_get_all_API_returned_tax_country_code > END');
    return $bei_all_available_tax_country_code;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Get all MNB API returned exchange rate
 * @access public
 * @return array
 */
function bei_get_MNB_current_exchange_rates_via_API()
{

    billzone_eu_invoice_logme(__FILE__, 'MNB Exchange API - START', '#77A: function bei_get_MNB_current_exchange_rates_via_API - START');

    try {

        $beiMNBExchangeRatesAPIResponse = new SoapClient("http://www.mnb.hu/arfolyamok.asmx?wsdl");
        $beiMNBExchangeRatesXML = $beiMNBExchangeRatesAPIResponse->GetCurrentExchangeRates()->GetCurrentExchangeRatesResult;
        $beiMNBExchangeRates = bei_XMLtoArray($beiMNBExchangeRatesXML);

        billzone_eu_invoice_logme(__FILE__, 'MNB Exchange API - OK', '#77B: function bei_get_MNB_current_exchange_rates_via_API');

    } catch (Exception $e) {

        $beiMNBExchangeRates = null;

        billzone_eu_invoice_logme(__FILE__, 'MNB Exchange API - ERROR', '#77C: function bei_get_MNB_current_exchange_rates_via_API: ' . $e->getMessage());

    }

    billzone_eu_invoice_logme(__FILE__, 'MNB Exchange API - END', '#77D: function bei_get_MNB_current_exchange_rates_via_API - END');

    return $beiMNBExchangeRates;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Get all MNB API returned exchange rate
 * @access public
 * @param string $bei_currency three digit currency
 * @return float
 */
function bei_get_currency_exchange_rate($bei_currency_being_searched)
{

    billzone_eu_invoice_logme(__FILE__, 'Exchange rate - START', '#88A: function bei_get_currency_exchange_rate(' . $bei_currency_being_searched . ') -> START');

    $bei_all_currency = bei_get_MNB_current_exchange_rates_via_API();

    $bei_currency = array();

    if (!empty($bei_all_currency) && count($bei_all_currency) > 0) {

        $bei_all_currency_array = $bei_all_currency['MNBCURRENTEXCHANGERATES']['DAY']['RATE'];
        foreach ($bei_all_currency_array as $bei_one_currency) {

            if (strtoupper($bei_one_currency['CURR']) == strtoupper($bei_currency_being_searched)) {
                $bei_currency['UNIT'] = $bei_one_currency['UNIT'];
                $bei_currency['CURR'] = $bei_one_currency['CURR'];
                $bei_currency['EXCHANGERATE'] = $bei_one_currency['EXCHANGERATE'];
            }

        }

    }

    billzone_eu_invoice_logme(__FILE__, 'Exchange rate - END', '#88A: function bei_get_currency_exchange_rate(' . $bei_currency_being_searched . ') -> END');
    return $bei_currency;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Main class to extract all necessary data from woomommerce order
 * @access public
 */
if (!class_exists("BEI_ORDER")) {
    class BEI_ORDER
    {

        private $order_data;
        private $order;

        public function __construct($bei_order)
        {

            billzone_eu_invoice_logme(__FILE__, 'OK', '#1: class BEI_ORDER > START');

            global $bei_options;
            global $bei_UN_EDIFACT_codes;
            global $wpdb;

            $this->order = $bei_order;

            // order data, billing info etc.
            $this->order_data = $this->order->get_data();

            billzone_eu_invoice_logme(__FILE__, 'OK', '#2: class BEI_ORDER > AFTER GETTING ORDER DATA | Order ID:' . $this->order_data['id']);

            // escape all special characters from text fields
            $this->order_data['billing']['first_name'] = htmlspecialchars($this->order_data['billing']['first_name'], ENT_QUOTES);
            $this->order_data['billing']['last_name'] = htmlspecialchars($this->order_data['billing']['last_name'], ENT_QUOTES);
            $this->order_data['billing']['company'] = htmlspecialchars($this->order_data['billing']['company'], ENT_QUOTES);
            $this->order_data['billing']['address_1'] = htmlspecialchars($this->order_data['billing']['address_1'], ENT_QUOTES);
            $this->order_data['billing']['address_2'] = htmlspecialchars($this->order_data['billing']['address_2'], ENT_QUOTES);
            $this->order_data['billing']['city'] = htmlspecialchars($this->order_data['billing']['city'], ENT_QUOTES);
            $this->order_data['billing']['postcode'] = htmlspecialchars($this->order_data['billing']['postcode'], ENT_QUOTES);
            $this->order_data['billing']['email'] = htmlspecialchars($this->order_data['billing']['email'], ENT_QUOTES);
            $this->order_data['billing']['phone'] = htmlspecialchars($this->order_data['billing']['phone'], ENT_QUOTES);

            $this->order_data['shipping']['first_name'] = htmlspecialchars($this->order_data['shipping']['first_name'], ENT_QUOTES);
            $this->order_data['shipping']['last_name'] = htmlspecialchars($this->order_data['shipping']['last_name'], ENT_QUOTES);
            $this->order_data['shipping']['company'] = htmlspecialchars($this->order_data['shipping']['company'], ENT_QUOTES);
            $this->order_data['shipping']['address_1'] = htmlspecialchars($this->order_data['shipping']['address_1'], ENT_QUOTES);
            $this->order_data['shipping']['address_2'] = htmlspecialchars($this->order_data['shipping']['address_2'], ENT_QUOTES);
            $this->order_data['shipping']['city'] = htmlspecialchars($this->order_data['shipping']['city'], ENT_QUOTES);

            // get security code status
            $bei_security_code_status = get_option($bei_options['billzone_eu_invoice_security_code_status']);

            // all UN/EDIFACT codes, all Unit Of Measure and all Invoice prefixes and invoice currency
            // if security code is OK then will be extracted from database otherwise we use the global $bei_UN_EDIFACT_codes array (only hungarian)
            // (only tax code with country ID equal to current Wordpress language will be extracted)
            $this->order_data['all_invoice_prefixes_and_associated_currency'] = array();
            $this->order_data['all_unit_of_measure'] = array();

            if (!empty($bei_security_code_status) && ($bei_security_code_status == 61001)) {

                $bei_get_all_billzone_data_list_json = get_option($bei_options['billzone_eu_invoice_all_billzone_data_list']);

                if (!empty($bei_get_all_billzone_data_list_json)) {

                    $bei_get_all_billzone_data_list_json_decoded = json_decode($bei_get_all_billzone_data_list_json);

                    // Collect invoice issuer data
                    $bei_get_invoice_issuer_data = array();
                    $bei_get_invoice_issuer_data['AddressCountryId'] = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressCountryId;
                    $bei_get_invoice_issuer_data['Name'] = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->Name;
                    $bei_get_invoice_issuer_data['Address'] = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressCountryId . '-' . $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressPostalCode . ' ' . $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressCity . ', ' . $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressLine1 . ' ' . $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressLine2;
                    $bei_get_invoice_issuer_data['TaxNumber'] = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->TaxNumber;
                    $bei_get_invoice_issuer_data['EUTaxNumber'] = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->EUTaxNumber;
                    $bei_get_invoice_issuer_data['RegistrationNumber'] = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->RegistrationNumber;
                    $bei_get_invoice_issuer_data['CountryDefaultCurrency'] = $bei_get_all_billzone_data_list_json_decoded->InvoiceIssuerCountryDefaultCurrency;
                    $this->order_data['invoice_issuer_data'] = $bei_get_invoice_issuer_data;

                    // All tax list from settings
                    $bei_get_all_tax_list = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->Taxes->TaxesQueryResultItem;

                    // Collect all available tax country code
                    $bei_get_all_available_tax_country_code = bei_get_all_API_returned_tax_country_code($bei_get_all_tax_list);

                    // The default country code for UN/EDICAT codes is taken from Worpdress user profile language settings
                    $bei_tax_lang = trim(substr(strtoupper(get_bloginfo("language")), 0, 2));
                    // If the language code of the Wordpress is EN or US then we will change it to GB (beiause of the API constrain)
                    if ($bei_tax_lang == 'EN' || $bei_tax_lang == 'US') {
                        $bei_tax_lang = 'GB';
                    }

                    // If we have UN/EDIFACT codes country code set in billzone settings, then we will take them from there
                    $bei_tax_country_from_billzone_settings = get_option($bei_options['billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code']);
                    if (!empty($bei_tax_country_from_billzone_settings)) {
                        $bei_tax_lang = $bei_tax_country_from_billzone_settings;
                    }

                    // If the WordPress or any other country code is not available on billzone tax list then we use the Hungarian code (HU) by default
                    if (!array_key_exists($bei_tax_lang, $bei_get_all_available_tax_country_code)) {
                        $bei_tax_lang = 'HU';
                    }

                    $bei_tax_list_for_actual_language_code = array();
                    foreach ($bei_get_all_tax_list as $bei_get_tax) {

                        if (strtoupper($bei_get_tax->CountryId) == $bei_tax_lang) {
                            $bei_tax_list_for_actual_language_code[$bei_get_tax->TaxRateCode] = (float) $bei_get_tax->Percentage;

                        }

                    }

                    $this->order_data['UN_EDIFACT_codes'] = $bei_tax_list_for_actual_language_code;
                    $this->order_data['UN_EDIFACT_country_code'] = $bei_tax_lang;

                    $bei_get_all_unit_of_measure = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->Units->UnitsQueryResultItem;
                    foreach ($bei_get_all_unit_of_measure as $bei_unit_of_measure) {

                        $this->order_data['all_unit_of_measure'][$bei_unit_of_measure->UnitIdentifier] = $bei_unit_of_measure->Name;

                    }

                    $all_invoice_prefixes_and_associated_currency = $bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->AccountBlocks->AccountBlocksQueryResultItem;
                    foreach ($all_invoice_prefixes_and_associated_currency as $invoice_prefixes_and_associated_currency) {

                        $this->order_data['all_invoice_prefixes_and_associated_currency'][$invoice_prefixes_and_associated_currency->AccountBlockPrefix] = $invoice_prefixes_and_associated_currency->CurrencyShortName;

                    }

                    // Get MNB exchange rates if the issuer address code is for Hungary (HU) and the invoice type is domestic currency billing, aka the currency is different from HUF
                    $this->order_data['current_exchange_rate'] = array();
                    $bei_invoice_sq = get_option($bei_options['billzone_eu_invoice_invoice_number_prefix']);
                    $bei_curr = $this->order_data['all_invoice_prefixes_and_associated_currency'][strtoupper(sanitize_text_field(trim($bei_invoice_sq)))];
                    if (strtoupper($bei_get_all_billzone_data_list_json_decoded->GetInvoiceDataResult->QueryResult->CompanyData->AddressCountryId) == 'HU' && !empty($bei_invoice_sq)) {

                        if ($bei_curr != 'HUF') {

                            $bei_cur_temp = array();
                            if (strtoupper(get_option($bei_options['billzone_eu_invoice_get_exchange_rate'])) == 'YES') {
                                $bei_invoice_currency_exchange_rate = bei_get_currency_exchange_rate($bei_curr);
                                if (!empty($bei_invoice_currency_exchange_rate) && count($bei_invoice_currency_exchange_rate) > 0) {
                                    $this->order_data['current_exchange_rate'] = $bei_invoice_currency_exchange_rate;
                                } else {
                                    $bei_cur_temp['UNIT'] = 1;
                                    $bei_cur_temp['CURR'] = $bei_curr;
                                    $bei_cur_temp['EXCHANGERATE'] = null;
                                    $this->order_data['current_exchange_rate'] = $bei_cur_temp;
                                }
                            } else {
                                $bei_cur_temp['UNIT'] = 1;
                                $bei_cur_temp['CURR'] = $bei_curr;
                                $bei_cur_temp['EXCHANGERATE'] = null;
                                $this->order_data['current_exchange_rate'] = $bei_cur_temp;
                            }

                        } else {
                            $bei_cur_temp['UNIT'] = 1;
                            $bei_cur_temp['CURR'] = $bei_curr;
                            $bei_cur_temp['EXCHANGERATE'] = null;
                            $this->order_data['current_exchange_rate'] = $bei_cur_temp;
                        }

                    } else {
                        $bei_cur_temp['UNIT'] = 1;
                        $bei_cur_temp['CURR'] = $bei_curr;
                        $bei_cur_temp['EXCHANGERATE'] = null;
                        $this->order_data['current_exchange_rate'] = $bei_cur_temp;
                    }

                } else {

                    $this->order_data['UN_EDIFACT_codes'] = $bei_UN_EDIFACT_codes;

                    $this->order_data['all_unit_of_measure']['DARAB'] = 'Pcs';

                    $this->order_data['invoice_issuer_data'] = null;

                }

            } else {

                $this->order_data['UN_EDIFACT_codes'] = $bei_UN_EDIFACT_codes;

                $this->order_data['all_unit_of_measure']['DARAB'] = 'Pcs';

                $this->order_data['invoice_issuer_data'] = null;

            }

            // set base unit of measure
            $bei_base_unit_of_measure = array();
            $bei_base_unit_of_measure['Code'] = 'DARAB';
            $bei_base_unit_of_measure['Text'] = __('Pcs', BILLZONE_EU_INVOICE_PLUGIN_NAME);

            // set default unit of measure
            $bei_default_unit_of_measure = array();

            if (!empty($bei_security_code_status) && ($bei_security_code_status == 61001)) {

                $bei_get_default_unit_of_measure = trim(get_option($bei_options['billzone_eu_invoice_default_pricing_unit']));

                if (empty($bei_get_default_unit_of_measure)) {
                    $bei_default_unit_of_measure['Code'] = $bei_base_unit_of_measure['Code'];
                    $bei_default_unit_of_measure['Text'] = $bei_base_unit_of_measure['Text'];
                } else {
                    $bei_default_unit_of_measure['Code'] = $bei_get_default_unit_of_measure;
                    $bei_default_unit_of_measure['Text'] = $this->order_data['all_unit_of_measure'][$bei_get_default_unit_of_measure];
                }

            } else {

                $bei_default_unit_of_measure['Code'] = $bei_base_unit_of_measure['Code'];
                $bei_default_unit_of_measure['Text'] = $bei_base_unit_of_measure['Text'];

            }

            // trimming billing fields
            foreach ($this->order_data['billing'] as $bei_key => $bei_value) {
                $this->order_data['billing'][$bei_key] = trim($bei_value);
            }

            // customer compound name
            if (trim($this->order_data['billing']['company']) != '') {
                $this->order_data['billing']['customer_name'] = trim($this->order_data['billing']['company']);
            } elseif ($this->order_data['billing']['company'] == 'HU') {
                $this->order_data['billing']['customer_name'] = $this->order_data['billing']['last_name'] . ' ' . $this->order_data['billing']['first_name'];
            } else {
                $this->order_data['billing']['customer_name'] = $this->order_data['billing']['first_name'] . ' ' . $this->order_data['billing']['last_name'];
            }
            $this->order_data['billing']['customer_name'] = $this->order_data['billing']['customer_name'];

            // customer compound address (addres_1 + address_2)
            $this->order_data['billing']['address_concatenated'] = $this->order_data['billing']['address_1'] . ' ' . $this->order_data['billing']['address_2'];
            $this->order_data['billing']['address_concatenated'] = $this->order_data['billing']['address_concatenated'];

            // VAT numbers
            $this->order_data['billing']['VAT_number'] = '';
            $this->order_data['billing']['EU_VAT_number'] = '';

            // all tax rate type
            $bei_all_tax_rate_type = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}woocommerce_tax_rates");
            
            if (!empty($bei_all_tax_rate_type) && count($bei_all_tax_rate_type)) {
                foreach ($bei_all_tax_rate_type as $bei_tax_type) {
                    $bei_tax_type_array = (array) $bei_tax_type;
                    $this->order_data['all_tax_type'][$bei_tax_type_array['tax_rate_id']] = $bei_tax_type_array;
                }
            }


            // set zero tax rate (without tax)
            // the default value is the first value in the array if there is no zero-valued VAT code
            $this->order_data['default_zero_tax_rate_id'] = reset($bei_all_tax_rate_type)->tax_rate_id;
            $this->order_data['default_zero_tax_rate'] = 0;
            if (!empty($bei_all_tax_rate_type) && count($bei_all_tax_rate_type)) {
                foreach ($bei_all_tax_rate_type as $bei_tax_type) {
                    $bei_tax_type_array = (array) $bei_tax_type;
                    if ($bei_tax_type_array['tax_rate'] == 0) {
                        $this->order_data['default_zero_tax_rate_id'] = $bei_tax_type_array['tax_rate_id'];
                        $this->order_data['default_zero_tax_rate'] = 0;
                    }
                }
            }
            
            // default tax rate ID
            $this->order_data['default_tax_rate_id'] = reset($bei_all_tax_rate_type)->tax_rate_id;

            // default tax rate
            $this->order_data['default_tax_rate'] = reset($bei_all_tax_rate_type)->tax_rate;

            // all country codes available in Woocommerce
            $this->order_data['all_country_codes'] = $this->bei_get_all_country_code();

            // all order statuses
            $this->order_data['all_status_codes'] = wc_get_order_statuses();

            // order status translated
            if (array_key_exists('wc-' . trim($this->order_data['status']), $this->order_data['all_status_codes'])) {
                $this->order_data['status_title'] = $this->order_data['all_status_codes']['wc-' . trim($this->order_data['status'])];
            } else {
                $this->order_data['status_title'] = '---';
            }
            $this->order_data['status_title'] = htmlspecialchars($this->order_data['status_title'], ENT_QUOTES);

            // if no payment method then set the default payment methode from billzone.eu invoice settings page
            if (empty($this->order_data['payment_method'])) {
                $bei_def_payment_meth = get_option($bei_options['billzone_eu_invoice_default_payment_methode']);
                if (!empty($bei_def_payment_meth)) {
                    $this->order_data['payment_method'] = strtoupper(trim($bei_def_payment_meth));
                }
            }

            // all payment methode
            $this->order_data['all_payment_method_type'] = null;
            $all_available_payment_method = WC()->payment_gateways->payment_gateways();
            if (isset($all_available_payment_method) && count($all_available_payment_method) > 0) {
                foreach ($all_available_payment_method as $bei_payment_method_value) {
                    $this->order_data['all_payment_method_type'][$bei_payment_method_value->id]['enabled'] = (strtoupper($bei_payment_method_value->enabled) == 'YES') ? 1 : 0;
                    $this->order_data['all_payment_method_type'][$bei_payment_method_value->id]['id'] = $bei_payment_method_value->id;
                    $this->order_data['all_payment_method_type'][$bei_payment_method_value->id]['method_title'] = htmlspecialchars($bei_payment_method_value->method_title, ENT_QUOTES);
                    $this->order_data['all_payment_method_type'][$bei_payment_method_value->id]['method_description'] = htmlspecialchars($bei_payment_method_value->method_description, ENT_QUOTES);
                    if ($this->order_data['payment_method'] == $bei_payment_method_value->id) {
                        $this->order_data['payment_method_title'] = htmlspecialchars($bei_payment_method_value->method_title, ENT_QUOTES);
                    }
                }
            }

            // all tax lines
            $this->bei_get_lines_data_and_meta_data('tax_lines', '');

            // all coupon lines
            $this->bei_get_lines_data_and_meta_data('coupon_lines', $bei_base_unit_of_measure);

            // initialize all discount by TAX category if there is any coupon
            $this->order_data['discount_per_tax'] = null;
            if (!empty($this->order_data['coupon_lines'])) {
                if (!empty($this->order_data['tax_lines'])) {
                    // categorizad by TAX code
                    foreach ($this->order_data['tax_lines'] as $bei_tax_line_key => $bei_tax_line_value) {
                        $this->order_data['discount_per_tax'][$bei_tax_line_value['rate_id']]['tax_percentage_id'] = $bei_tax_line_value['rate_id'];
                        $this->order_data['discount_per_tax'][$bei_tax_line_value['rate_id']]['tax_percentage'] = $this->order_data['all_tax_type'][$bei_tax_line_value['rate_id']]['tax_rate'];
                        $this->order_data['discount_per_tax'][$bei_tax_line_value['rate_id']]['total_discount_per_tax'] = 0;
                        $this->order_data['discount_per_tax'][$bei_tax_line_value['rate_id']]['total_discount_tax_per_tax'] = 0;
                        $this->order_data['discount_per_tax'][$bei_tax_line_value['rate_id']]['unit_of_measure'] = $bei_base_unit_of_measure['Code'];
                        $this->order_data['discount_per_tax'][$bei_tax_line_value['rate_id']]['unit_of_measure_text'] = htmlspecialchars($bei_base_unit_of_measure['Text'], ENT_QUOTES);
                    }

                }
                else {
                    // no TAX codes (typiccaly when selling is set to no tax)
                    $this->order_data['discount_per_tax'][$this->order_data['default_zero_tax_rate_id']]['tax_percentage_id'] = $this->order_data['default_zero_tax_rate_id'];
                    $this->order_data['discount_per_tax'][$this->order_data['default_zero_tax_rate_id']]['tax_percentage'] = $this->order_data['all_tax_type'][$this->order_data['default_zero_tax_rate_id']]['tax_rate'];
                    $this->order_data['discount_per_tax'][$this->order_data['default_zero_tax_rate_id']]['total_discount_per_tax'] = 0;
                    $this->order_data['discount_per_tax'][$this->order_data['default_zero_tax_rate_id']]['total_discount_tax_per_tax'] = 0;
                    $this->order_data['discount_per_tax'][$this->order_data['default_zero_tax_rate_id']]['unit_of_measure'] = $bei_base_unit_of_measure['Code'];
                    $this->order_data['discount_per_tax'][$this->order_data['default_zero_tax_rate_id']]['unit_of_measure_text'] = htmlspecialchars($bei_base_unit_of_measure['Text'], ENT_QUOTES);    
                }
            }

            // 1: all line items and images associated to that particular item
            // 2: set tax value in percentage for each item
            // 3: generate one field for all meta data
            // 4: summing coupon discount by TAX category
            $this->bei_get_lines_data_and_meta_data('line_items', $bei_default_unit_of_measure);
           
            if (!empty($this->order_data['line_items'])) {
                foreach ($this->order_data['line_items'] as $bei_item_key => $bei_item_data) {

                    $this->order_data['line_items'][$bei_item_key]['name'] = htmlspecialchars($this->order_data['line_items'][$bei_item_key]['name'], ENT_QUOTES);
                    
                    if ($bei_item_data['product_id']) {
                        $bei_item_image_data = wc_get_product($bei_item_data['product_id'])->get_image(array(20, 20));
                        if (empty(trim($bei_item_image_data))) {
                            $bei_item_image_data = '<img src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-missing-image.svg" alt="' . __('Missing image', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">';
                        }
                    }
                    
                    $this->order_data['line_items'][$bei_item_key]['image'] = $bei_item_image_data;

                    $bei_tax_percentage = $this->order_data['default_tax_rate'];
                    $bei_tax_id = $this->order_data['default_tax_rate_id'];
                    if (!empty($bei_item_data['tax_class'])) {
                        foreach ($this->order_data['all_tax_type'] as $bei_tax_key => $bei_tax_value) {
                            if (trim($bei_item_data['tax_class']) == trim($bei_tax_value['tax_rate_class'])) {
                                $bei_tax_percentage = $bei_tax_value['tax_rate'];
                                $bei_tax_id = $bei_tax_value['tax_rate_id'];
                            }
                        }
                    }

                    // set tax percentage
                    $this->order_data['line_items'][$bei_item_key]['tax_percentage'] = $bei_tax_percentage;
                    // set tax percentage id
                    $this->order_data['line_items'][$bei_item_key]['tax_percentage_id'] = $bei_tax_id;

                    $bei_all_meta_data_concatenated = '';
                    if (!empty($bei_item_data['meta_data'])) {

                        foreach ($bei_item_data['meta_data'] as $bei_item_meta_data_key => $bei_item_meta_data_data) {

                            $bei_all_meta_data_concatenated .= $bei_item_meta_data_data['key'] . ': ' . $bei_item_meta_data_data['value'] . ' | ';

                        }
                        $bei_all_meta_data_concatenated = trim(substr(trim($bei_all_meta_data_concatenated), 0, -1));

                    }

                    // set all meta data field
                    $this->order_data['line_items'][$bei_item_key]['all_meta_data_concatenated'] = htmlspecialchars($bei_all_meta_data_concatenated, ENT_QUOTES);

                    // if there is any coupon discount then summ discount by TAX category
                    if (!empty($this->order_data['discount_per_tax'])) {
                        foreach ($this->order_data['discount_per_tax'] as $bei_tax_type_key => $bei_tax_type_value) {
                            if ($bei_tax_type_value['tax_percentage_id'] == $this->order_data['line_items'][$bei_item_key]['tax_percentage_id']) {
                                $this->order_data['discount_per_tax'][$bei_tax_type_key]['total_discount_per_tax'] += $this->order_data['line_items'][$bei_item_key]['subtotal'] - $this->order_data['line_items'][$bei_item_key]['total'];
                                $this->order_data['discount_per_tax'][$bei_tax_type_key]['total_discount_tax_per_tax'] += $this->order_data['line_items'][$bei_item_key]['subtotal_tax'] - $this->order_data['line_items'][$bei_item_key]['total_tax'];
                            }
                        }
                    }
                }

            }

            // all shipping type
            $bei_all_shipping_methods = WC()->shipping->get_shipping_methods();
            if (!empty($bei_all_shipping_methods)) {
                foreach ($bei_all_shipping_methods as $bei_shipping_method_key => $bei_shipping_method_value) {
                    $this->order_data['all_shipping_methods'][$bei_shipping_method_value->id]['id'] = $bei_shipping_method_value->id;
                    $this->order_data['all_shipping_methods'][$bei_shipping_method_value->id]['method_title'] = htmlspecialchars($bei_shipping_method_value->method_title, ENT_QUOTES);
                }
            }

            // all shipping lines
            $this->bei_get_lines_data_and_meta_data('shipping_lines', $bei_base_unit_of_measure);

            // all shipping names concatenated
            $bei_all_shipping_name = '';
            $bei_has_shipping_lines = false;
            if (!empty($this->order_data['shipping_lines']) && count($this->order_data['shipping_lines']) > 0) {
                foreach ($this->order_data['shipping_lines'] as $bei_shipping_key => $bei_shipping_data) {

                    foreach ($this->order_data['all_shipping_methods'] as $bei_shipping_method_key => $bei_shipping_method_value) {
                        if ($bei_shipping_data['method_id'] == $bei_shipping_method_key) {
                            $bei_has_shipping_lines = true;
                            $bei_shipping_data['method_title'] = htmlspecialchars($bei_shipping_method_value['method_title'], ENT_QUOTES);
                            $bei_shipping_data['name'] = htmlspecialchars($bei_shipping_method_value['method_title'], ENT_QUOTES);
                            $this->order_data['shipping_lines'][$bei_shipping_key]['method_title'] = htmlspecialchars($bei_shipping_method_value['method_title'], ENT_QUOTES);
                            $this->order_data['shipping_lines'][$bei_shipping_key]['name'] = htmlspecialchars($bei_shipping_method_value['method_title'], ENT_QUOTES);
                        }
                    }

                    $bei_all_shipping_name .= $bei_shipping_data['method_title'] . ', ';

                    $this->order_data['shipping_lines'][$bei_shipping_key]['tax_percentage'] = $this->order_data['default_tax_rate'];
                    $this->order_data['shipping_lines'][$bei_shipping_key]['tax_percentage_id'] = $this->order_data['default_tax_rate_id'];
                    foreach ($this->order_data['all_tax_type'] as $bei_tx_key => $bei_tx_data) {
                        if (($this->order_data['shipping_lines'][$bei_shipping_key]['total'] != 0) && ($bei_tx_data['tax_rate'] == ($this->order_data['shipping_lines'][$bei_shipping_key]['total_tax'] * 100 / $this->order_data['shipping_lines'][$bei_shipping_key]['total']))) {
                            $this->order_data['shipping_lines'][$bei_shipping_key]['tax_percentage'] = $bei_tx_data['tax_rate'];
                            $this->order_data['shipping_lines'][$bei_shipping_key]['tax_percentage_id'] = $bei_tx_data['tax_rate_id'];
                        }
                    }
                }
            }

            $this->order_data['has_shipping_lines'] = $bei_has_shipping_lines ? 1 : 0;
            $this->order_data['all_shipping_name'] = htmlspecialchars(substr(trim($bei_all_shipping_name), 0, -1), ENT_QUOTES);

            // all fee lines
            $this->bei_get_lines_data_and_meta_data('fee_lines', $bei_base_unit_of_measure);
            if (!empty($this->order_data['fee_lines']) && count($this->order_data['fee_lines']) > 0) {
                foreach ($this->order_data['fee_lines'] as $bei_fee_key => $bei_fee_value) {

                    $this->order_data['fee_lines'][$bei_fee_key]['tax_percentage'] = $this->order_data['default_tax_rate'];
                    $this->order_data['fee_lines'][$bei_fee_key]['tax_percentage_id'] = $this->order_data['default_tax_rate_id'];
                    foreach ($bei_fee_value['taxes']['total'] as $bei_fee_tax_key => $bei_fee_tax_value) {
                        if (!empty($bei_fee_tax_value)) {
                            $this->order_data['fee_lines'][$bei_fee_key]['tax_percentage'] = $this->order_data['all_tax_type'][$bei_fee_tax_key]['tax_rate'];
                            $this->order_data['fee_lines'][$bei_fee_key]['tax_percentage_id'] = $bei_fee_tax_key;
                        }
                    }
                }
            }

            // all coupon names concatenated
            $bei_all_coupons = '';
            if (!empty($this->order_data['coupon_lines']) && count($this->order_data['coupon_lines']) > 0) {
                foreach ($this->order_data['coupon_lines'] as $bei_coupon) {
                    $bei_all_coupons .= $bei_coupon['code'] . ', ';
                }
            }
            $this->order_data['all_coupon_name'] = htmlspecialchars(substr(trim($bei_all_coupons), 0, -1), ENT_QUOTES);

            // all order meta data lines
            $this->bei_get_lines_data('meta_data');

            // all refund items
            $this->order_data['all_refunds'] = array();
            $this->order_data['there_is_a_refund'] = 'no';
            $this->order_data['there_is_wrong_tax_in_refund'] = 'no';

            if (count($this->order->get_refunds()) > 0) {

                billzone_eu_invoice_logme(__FILE__, 'OK', '#3A: class BEI_ORDER > Refunds START | Order ID: ' . $this->order_data['id']);

                foreach ($this->order->get_refunds() as $bei_refund) {

                    $bei_refund_data = $bei_refund->get_data();
                    
                    // line items refund
                    if (count($bei_refund_data['line_items']) > 0) {

                        $this->order_data['there_is_a_refund'] = 'yes';

                        foreach ($bei_refund_data['line_items'] as $bei_refund_data_item) {

                            $bei_refund_data_detail = array();
                            $bei_refund_data_detail['wrong_tax'] = 'no';
                            $bei_refund_data_detail['id'] = $bei_refund_data_item['order_id'];
                            $bei_refund_data_detail['original_refunded_item_id'] = $bei_refund_data_item->get_meta('_refunded_item_id');
                            $bei_refund_data_detail['order_id'] = $bei_refund_data['parent_id'];
                            $bei_refund_data_detail['name'] = htmlspecialchars($bei_refund_data_item['name'], ENT_QUOTES);
                            $bei_refund_data_detail['reason'] = $bei_refund_data['reason'];
                            $bei_refund_data_detail['quantity'] = $bei_refund_data_item['quantity'];
                            if ($bei_refund_data_detail['quantity'] == 0) {
                                $bei_refund_data_detail['net_unit_price'] = 0;
                                $bei_refund_data_detail['wrong_tax'] = 'yes';
                                $this->order_data['there_is_wrong_tax_in_refund'] = 'yes';
                            } else {
                                $bei_refund_data_detail['net_unit_price'] = abs($bei_refund_data_item['total'] / $bei_refund_data_item['quantity']);
                            }
                            $bei_refund_data_detail['subtotal'] = $bei_refund_data_item['subtotal'];
                            $bei_refund_data_detail['subtotal_tax'] = $bei_refund_data_item['subtotal_tax'];
                            $bei_refund_data_detail['total'] = $bei_refund_data_item['total'];
                            $bei_refund_data_detail['total_tax'] = $bei_refund_data_item['total_tax'];
                            $bei_refund_data_detail['unit_of_measure'] = $this->order_data['line_items'][$bei_refund_data_detail['original_refunded_item_id']]['unit_of_measure'];
                            $bei_refund_data_detail['unit_of_measure_text'] = $this->order_data['line_items'][$bei_refund_data_detail['original_refunded_item_id']]['unit_of_measure_text'];
                            
                            /* if (count($bei_refund_data_item['taxes']['total']) == 1) {
                                $bei_refund_data_detail['tax_percentage'] = $this->order_data['all_tax_type'][array_keys($bei_refund_data_item['taxes']['total'])[0]]['tax_rate'];
                                $bei_refund_data_detail['tax_percentage_id'] = array_keys($bei_refund_data_item['taxes']['total'])[0];
                            } else {
                                $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_tax_rate'];
                                $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_tax_rate_id'];
                                $bei_refund_data_detail['wrong_tax'] = 'yes';
                                $this->order_data['there_is_wrong_tax_in_refund'] = 'yes';
                            } */
                            
                            switch (count($bei_refund_data_item['taxes']['total'])) {
                                case 0:
                                    $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_zero_tax_rate'];
                                    $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_zero_tax_rate_id'];
                                    break;
                                case 1:
                                    $bei_refund_data_detail['tax_percentage'] = $this->order_data['all_tax_type'][array_keys($bei_refund_data_item['taxes']['total'])[0]]['tax_rate'];
                                    $bei_refund_data_detail['tax_percentage_id'] = array_keys($bei_refund_data_item['taxes']['total'])[0];
                                    break;
                                default: 
                                    $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_tax_rate'];
                                    $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_tax_rate_id'];
                                    $bei_refund_data_detail['wrong_tax'] = 'yes';
                                    $this->order_data['there_is_wrong_tax_in_refund'] = 'yes';
                            }

                            array_push($this->order_data['all_refunds'], $bei_refund_data_detail);

                        }

                    } else {

                        // shipping lines refund
                        if (count($bei_refund_data['shipping_lines']) > 0) {

                            $this->order_data['there_is_a_refund'] = 'yes';

                            foreach ($bei_refund_data['shipping_lines'] as $bei_refund_data_item) {

                                $bei_refund_data_detail = array();
                                $bei_refund_data_detail['wrong_tax'] = 'no';
                                $bei_refund_data_detail['id'] = $bei_refund_data_item['order_id'];
                                $bei_refund_data_detail['original_refunded_item_id'] = $bei_refund_data_item->get_meta('_refunded_item_id');
                                $bei_refund_data_detail['order_id'] = $bei_refund_data['parent_id'];
                                $bei_refund_data_detail['name'] = __('Shipping', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' [' . $bei_refund_data_item['method_title'] . ']';
                                $bei_refund_data_detail['reason'] = $bei_refund_data['reason'];
                                $bei_refund_data_detail['quantity'] = -1;
                                $bei_refund_data_detail['net_unit_price'] = abs($bei_refund_data_item['total'] / $bei_refund_data_detail['quantity']);
                                $bei_refund_data_detail['subtotal'] = $bei_refund_data_item['total'];
                                $bei_refund_data_detail['subtotal_tax'] = $bei_refund_data_item['total_tax'];
                                $bei_refund_data_detail['total'] = $bei_refund_data_item['total'];
                                $bei_refund_data_detail['total_tax'] = $bei_refund_data_item['total_tax'];
                                reset($this->order_data['all_unit_of_measure']);
                                $bei_refund_data_detail['unit_of_measure'] = key($this->order_data['all_unit_of_measure']);
                                $bei_refund_data_detail['unit_of_measure_text'] = $this->order_data['all_unit_of_measure'][$bei_refund_data_detail['unit_of_measure']];

                                switch (count($bei_refund_data_item['taxes']['total'])) {
                                    case 0:
                                        $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_zero_tax_rate'];
                                        $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_zero_tax_rate_id'];
                                        break;
                                    case 1:
                                        $bei_refund_data_detail['tax_percentage'] = $this->order_data['all_tax_type'][array_keys($bei_refund_data_item['taxes']['total'])[0]]['tax_rate'];
                                    $bei_refund_data_detail['tax_percentage_id'] = array_keys($bei_refund_data_item['taxes']['total'])[0];
                                        break;
                                    default: 
                                        $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_tax_rate'];
                                        $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_tax_rate_id'];
                                        $bei_refund_data_detail['wrong_tax'] = 'yes';
                                        $this->order_data['there_is_wrong_tax_in_refund'] = 'yes';
                                }

                                array_push($this->order_data['all_refunds'], $bei_refund_data_detail);
                            }
                        }

                        // fee lines refunds
                        if (count($bei_refund_data['fee_lines']) > 0) {

                            $this->order_data['there_is_a_refund'] = 'yes';

                            foreach ($bei_refund_data['fee_lines'] as $bei_refund_data_item) {

                                $bei_refund_data_detail = array();
                                $bei_refund_data_detail['wrong_tax'] = 'no';
                                $bei_refund_data_detail['id'] = $bei_refund_data_item['order_id'];
                                $bei_refund_data_detail['original_refunded_item_id'] = $bei_refund_data_item->get_meta('_refunded_item_id');
                                $bei_refund_data_detail['order_id'] = $bei_refund_data['parent_id'];
                                $bei_refund_data_detail['name'] = $bei_refund_data_item['name'];
                                $bei_refund_data_detail['reason'] = $bei_refund_data['reason'];
                                $bei_refund_data_detail['quantity'] = -1;
                                $bei_refund_data_detail['net_unit_price'] = abs($bei_refund_data_item['total'] / $bei_refund_data_detail['quantity']);
                                $bei_refund_data_detail['subtotal'] = $bei_refund_data_item['total'];
                                $bei_refund_data_detail['subtotal_tax'] = $bei_refund_data_item['total_tax'];
                                $bei_refund_data_detail['total'] = $bei_refund_data_item['total'];
                                $bei_refund_data_detail['total_tax'] = $bei_refund_data_item['total_tax'];
                                reset($this->order_data['all_unit_of_measure']);
                                $bei_refund_data_detail['unit_of_measure'] = key($this->order_data['all_unit_of_measure']);
                                $bei_refund_data_detail['unit_of_measure_text'] = $this->order_data['all_unit_of_measure'][$bei_refund_data_detail['unit_of_measure']];

                                switch (count($bei_refund_data_item['taxes']['total'])) {
                                    case 0:
                                        $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_zero_tax_rate'];
                                        $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_zero_tax_rate_id'];
                                        break;
                                    case 1:
                                        $bei_refund_data_detail['tax_percentage'] = $this->order_data['all_tax_type'][array_keys($bei_refund_data_item['taxes']['total'])[0]]['tax_rate'];
                                    $bei_refund_data_detail['tax_percentage_id'] = array_keys($bei_refund_data_item['taxes']['total'])[0];
                                        break;
                                    default: 
                                        $bei_refund_data_detail['tax_percentage'] = $this->order_data['default_tax_rate'];
                                        $bei_refund_data_detail['tax_percentage_id'] = $this->order_data['default_tax_rate_id'];
                                        $bei_refund_data_detail['wrong_tax'] = 'yes';
                                        $this->order_data['there_is_wrong_tax_in_refund'] = 'yes';
                                }

                                array_push($this->order_data['all_refunds'], $bei_refund_data_detail);
                            }
                        }

                    }

                }

                billzone_eu_invoice_logme(__FILE__, 'OK', '#3A: class BEI_ORDER > Refunds END | Order ID: ' . $this->order_data['id']);
            }

            if ($this->order_data['there_is_a_refund'] == 'yes') {
                $this->order_data['total_tax_after_refund'] = $this->order_data['total_tax'] - $this->order->get_total_tax_refunded();
                $this->order_data['total_after_refund'] = $this->order_data['total'] - $this->order->get_total_refunded();
                $this->order_data['total_gross_refund'] = (-1) * $this->order->get_total_refunded();
            } else {
                $this->order_data['total_tax_after_refund'] = $this->order_data['total_tax'];
                $this->order_data['total_after_refund'] = $this->order_data['total'];
                $this->order_data['total_gross_refund'] = 0;
            }

            $bei_id = $this->order_data['id'];
            $bei_check_if_invoice_exist = $wpdb->get_results(
                "
                SELECT id, invoice_number, invoice_url, invoice_type
                FROM " . BILLZONE_EU_INVOICE_TABLE_NAME .
                " WHERE order_id = $bei_id
                "
            );

            // check if we have already created invoice for this order
            if (empty($bei_check_if_invoice_exist)) {
                $this->order_data['already_created_invoice_number'] = "";
                $this->order_data['already_created_invoice_url'] = "";
                $this->order_data['already_created_invoice_type'] = "";
            } else {
                $this->order_data['already_created_invoice_number'] = $bei_check_if_invoice_exist[0]->invoice_number;
                $this->order_data['already_created_invoice_url'] = $bei_check_if_invoice_exist[0]->invoice_url;
                $this->order_data['already_created_invoice_type'] = $bei_check_if_invoice_exist[0]->invoice_type;
            }

            billzone_eu_invoice_logme(__FILE__, 'OK', '#3: class BEI_ORDER > END | Order ID: ' . $this->order_data['id']);

        }

        // return all country code available in Woocommerce
        public static function bei_get_all_country_code()
        {
            billzone_eu_invoice_logme(__FILE__, 'OK', '#1: class BEI_ORDER::bei_get_all_country_code > START_END');
            return bei_all_country_code_base();
        }

        // return order data
        public function bei_get_order_data()
        {
            billzone_eu_invoice_logme(__FILE__, 'OK', '#1: class BEI_ORDER::bei_get_order_data > START_END');
            return $this->order_data;
        }

        // return order data in JSON format
        public function bei_get_order_data_JSON()
        {
            billzone_eu_invoice_logme(__FILE__, 'OK', '#1: class BEI_ORDER::bei_get_order_data_JSON > START_END');
            return json_encode($this->order_data);
        }

        // return data and meta data of the line defined by $bei_line_name
        private function bei_get_lines_data_and_meta_data($bei_line_name, $bei_uom)
        {

            billzone_eu_invoice_logme(__FILE__, 'OK', '#1: class BEI_ORDER::bei_get_lines_data_and_meta_data > START | Order ID: ' . $this->order_data['id']);
            if ($bei_line_name == 'line_items') {
                $bei_all_items = $this->order->get_items();
            } else {
                $bei_all_items = $this->order->get_items(str_replace('_lines', '', $bei_line_name));
            }

            $this->order_data[$bei_line_name] = null;
            if (isset($bei_all_items) && count($bei_all_items) > 0) {
                foreach ($bei_all_items as $bei_item) {
                    $bei_item_data = $bei_item->get_data();
                    $this->order_data[$bei_line_name][$bei_item_data['id']] = $bei_item_data;
                    $this->order_data[$bei_line_name][$bei_item_data['id']]['meta_data'] = null;

                    if (($bei_uom != '') && !empty($bei_uom)) {
                        $this->order_data[$bei_line_name][$bei_item_data['id']]['unit_of_measure'] = $bei_uom['Code'];
                        $this->order_data[$bei_line_name][$bei_item_data['id']]['unit_of_measure_text'] = $bei_uom['Text'];
                    }

                    if (isset($bei_item_data['meta_data'])) {
                        foreach ($bei_item_data['meta_data'] as $bei_meta_data) {
                            $bei_meta_data_data = $bei_meta_data->get_data();
                            $this->order_data[$bei_line_name][$bei_item_data['id']]['meta_data'][$bei_meta_data_data['id']] = $bei_meta_data_data;
                        }
                    }

                }

                billzone_eu_invoice_logme(__FILE__, 'OK', '#2: class BEI_ORDER::bei_get_lines_data_and_meta_data > all items | Order ID: ' . $this->order_data['id']);
            }

            billzone_eu_invoice_logme(__FILE__, 'OK', '#3: class BEI_ORDER::bei_get_lines_data_and_meta_data > END | Order ID: ' . $this->order_data['id']);
        }

        // return data of the line defined by $bei_line_name
        private function bei_get_lines_data($bei_line_name)
        {

            billzone_eu_invoice_logme(__FILE__, 'OK', '#1: class BEI_ORDER::bei_get_lines_data > START | Order ID: ' . $this->order_data['id']);

            $this->order_data[$bei_line_name] = null;
            $bei_all_items = $this->order->get_data()[$bei_line_name];

            if (isset($bei_all_items) && count($bei_all_items) > 0) {
                foreach ($bei_all_items as $bei_item) {
                    $bei_item_data = $bei_item->get_data();
                    $this->order_data[$bei_line_name][$bei_item_data['id']] = $bei_item_data;
                }
            }

            billzone_eu_invoice_logme(__FILE__, 'OK', '#2: class BEI_ORDER::bei_get_lines_data > END | Order ID: ' . $this->order_data['id']);

        }

    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------
