<?php

/**
 * This file contains all class and functions for log
 *
 * 
 * @link              https://n-ware.hu
 * @since             0.8.1
 * @package           billzone.eu Invoice
 *
 */

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Exit if accessed directly
*/
if ( ! defined( 'ABSPATH' ) ) { 
    exit; 
}
//---------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to save a log row in log database table (xxx_billzone_eu_invoice_log, where xxx is WP db prefix) when logging option is ON [billzone_eu_invoice_log_is_on]
 * (only the last 2000 log row will be saved)
 * @param string $bei_file_name  -> the file name containing the logged code
 * @param string $bei_log_type  -> the log type (ERROR, OK etc)
 * @param string $bei_log_text  -> short description of the log event
 * @access public
*/
function billzone_eu_invoice_logme( $bei_file_name, $bei_log_type, $bei_log_text){

    global $wpdb;
    global $bei_options;
    
    if (isset($bei_options)) {
        $bei_log_is_on = get_option($bei_options['billzone_eu_invoice_log_is_on'],'NO');

        // insert log data only when logging is ON
        if(!empty($bei_log_is_on) && strtoupper($bei_log_is_on) == 'YES') { 
    
            // if the row number is greater than 2000 then the first log row will be deleted
            $bei_all_log_row = $wpdb->get_results('SELECT * FROM '.BILLZONE_EU_INVOICE_LOG_TABLE_NAME);
            if (count($bei_all_log_row) >= 2000) {
                $wpdb->query('DELETE FROM '.BILLZONE_EU_INVOICE_LOG_TABLE_NAME.' LIMIT 1');
            }
            
            date_default_timezone_set("Europe/Budapest");
            $wpdb->insert( 
                BILLZONE_EU_INVOICE_LOG_TABLE_NAME, 
                array( 
                        'log_creation_date' => date('Y-m-d H:i:s',time()), 
                        'log_type' => $bei_log_type,
                        'log_text' => 'BEI v'.BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM.' | File: '.basename($bei_file_name).' | '.$bei_log_text
                    )
                );
        }
    }
      
}
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to check if log is ON via AJAX
 * @param string $bei_file_name  -> the file name containing the logged code
 * @param string $bei_log_type  -> the log type (ERROR, OK etc)
 * @param string $bei_log_text  -> short description of the log event
 * @access public
*/
function billzone_eu_invoice_log_is_on_AJAX(){
    if ( isset($_REQUEST) ) {

        global $bei_options;

        $bei_log_is_on = get_option($bei_options['billzone_eu_invoice_log_is_on']);
        if(!empty($bei_log_is_on) && strtoupper($bei_log_is_on) == 'YES') {
            $bei_log_status = 'on';
        }
        else {
            $bei_log_status = 'off';
        }

        // set response type: JSON
        header("Content-type: application/json");

        // response in JSON format
        echo '{"logstatus":"'.$bei_log_status.'"}';
    }
    die();
}

add_action( 'wp_ajax_billzone_eu_invoice_log_is_on_AJAX', 'billzone_eu_invoice_log_is_on_AJAX' );
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to save a log row in log database table via AJAX (xxx_billzone_eu_invoice_log, where xxx is WP db prefix) when logging option is ON [billzone_eu_invoice_log_is_on]
 * (only the last 1000 log row will be saved)
 * @access public
*/
function billzone_eu_invoice_logme_AJAX(){
    if ( isset($_REQUEST) ) {

        billzone_eu_invoice_logme( 'billzone-eu-invoice.js',$_REQUEST['beilogtype'], $_REQUEST['beilogtext']);

        // set response type: JSON
        header("Content-type: application/json");

        // response in JSON format
        echo '{"message":"Logging succesfully"}';
    }
    die();
}

add_action( 'wp_ajax_billzone_eu_invoice_logme_AJAX', 'billzone_eu_invoice_logme_AJAX' );
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to generate message panel
 * @param string $bei_title  -> the message panel title
 * @param string $bei_body  -> the message panel body
 * @param string $bei_buttons  -> the message panel buttons 
 * @param string $bei_dimension_class  -> CSS class for setting the message panel dimension
 * @access public
*/
function billzone_eu_invoice_message_panel_generator($bei_title,$bei_body,$bei_buttons,$bei_dimension_class){

    $bei_message_panel =  '<div class="billzone-eu-invoice-log-clear-message-panel-wrapper">';
    $bei_message_panel .= ' <div class="billzone-eu-invoice-message-container  '.$bei_dimension_class.'">';
    $bei_message_panel .= '     <div class="billzone-eu-invoice-message-title">';
    $bei_message_panel .= '         '.$bei_title;
    $bei_message_panel .= '         <span class="dashicons dashicons-no-alt"></span>';
    $bei_message_panel .= '     </div>';
    $bei_message_panel .= '     <div class="billzone-eu-invoice-message-body">';
    $bei_message_panel .= '         '.$bei_body;
    $bei_message_panel .= '     </div>';
    $bei_message_panel .= '     <div class="billzone-eu-invoice-message-buttons">';
    $bei_message_panel .= '         '.$bei_buttons;
    $bei_message_panel .= '     </div>';
    $bei_message_panel .= ' </div>';
    $bei_message_panel .= '</div>';

    return $bei_message_panel;
}
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to clear the log database table via AJAX call (xxx_billzone_eu_invoice_log, where xxx is equal to WP db prefix)
 * @access public
*/
function billzone_eu_invoice_clear_log_AJAX(){
    if ( isset($_REQUEST) ) {

        global $wpdb;

        $wpdb->query('TRUNCATE TABLE '.BILLZONE_EU_INVOICE_LOG_TABLE_NAME);

        $bei_title = __( 'billzone.eu Invoice message' ,BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (v' . BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM . ')';
        $bei_body = __( 'All logs data have been deleted!' ,BILLZONE_EU_INVOICE_PLUGIN_NAME);
        $bei_buttons = '<span id="beiMessagePanelOK" class="button button-primary">'.__( 'OK' ,BILLZONE_EU_INVOICE_PLUGIN_NAME).'</span>';
        $bei_message_panel = billzone_eu_invoice_message_panel_generator($bei_title,$bei_body,$bei_buttons,'billzone-eu-invoice-message-container-dimension-1');
        
        // set response type: JSON
        header("Content-type: application/json");

        // response in JSON format
        echo '{"messagepanel":"'.bei_esc_html($bei_message_panel).'"}';
    }
    die();
}

add_action( 'wp_ajax_billzone_eu_invoice_clear_log_AJAX', 'billzone_eu_invoice_clear_log_AJAX' );
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to export the log database table via AJAX call (xxx_billzone_eu_invoice_log, where xxx is equal to WP db prefix)
 * @access public
*/
function billzone_eu_invoice_export_log_AJAX(){
    if ( isset($_REQUEST) ) {

        global $wpdb;
        
        $bei_title = __( 'billzone.eu Invoice log export' ,BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (v' . BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM . ')';

        $bei_all_log_row = $wpdb->get_results("SELECT * FROM ".BILLZONE_EU_INVOICE_LOG_TABLE_NAME);
        if (!empty($bei_all_log_row)) {
            $bei_body = '<textarea readonly id="beiMessagePanelTextarea">';
            foreach($bei_all_log_row as $bei_log_row){
                $bei_row_id = str_repeat("0", (4 - strlen($bei_log_row->id))).$bei_log_row->id;
                $bei_body .= '#'.$bei_row_id.' | '.$bei_log_row->log_creation_date.' | Status: '.$bei_log_row->log_type.' | '.$bei_log_row->log_text.'\n';
            }
            $bei_body .= '</textarea>';
            $bei_buttons = '<span id="beiMessagePanelOK" class="button button-primary">'.__( 'OK' ,BILLZONE_EU_INVOICE_PLUGIN_NAME).'</span>';
            $bei_copyresult_OK = __( 'Copy result: SUCCESFULLY' ,BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_copyresult_error = __( 'Copy result: ERROR' ,BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_buttons .= '<span id="beiMessagePanelCopyToClipboard" class="button button-primary" data-copyok="'.$bei_copyresult_OK.'" data-copyerror="'.$bei_copyresult_error.'">'.__( 'Copy to Clipboard...' ,BILLZONE_EU_INVOICE_PLUGIN_NAME).'</span>';
            $bei_dimension = 'billzone-eu-invoice-message-container-dimension-2';
        }
        else {
            $bei_body = __( 'No log data!' ,BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_buttons = '<span id="beiMessagePanelOK" class="button button-primary">'.__( 'OK' ,BILLZONE_EU_INVOICE_PLUGIN_NAME).'</span>';
            $bei_dimension = 'billzone-eu-invoice-message-container-dimension-1';
        }
        
        $bei_message_panel = billzone_eu_invoice_message_panel_generator($bei_title,$bei_body,$bei_buttons,$bei_dimension);

        // set response type: JSON
        header("Content-type: application/json");

        // response in JSON format
        echo '{"messagepanel":"'.bei_esc_html($bei_message_panel).'"}';
    }
    die();
}

add_action( 'wp_ajax_billzone_eu_invoice_export_log_AJAX', 'billzone_eu_invoice_export_log_AJAX' );
//----------------------------------------------------------------------------------------------------------------------------------------------