<?php

/**
 * Initialize all plugin default settings
 *
 *
 * @link              https://n-ware.hu
 * @since             0.8.1
 * @package           billzone.eu Invoice
 *
 */

include_once 'billzone-eu-invoice-API.php';
include_once 'billzone-eu-invoice-log.php';

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Exit if accessed directly
 */
if (!defined('ABSPATH')) {
    exit;
}
//-----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add new column named "billzone.eu invoce nr." to Woocommerce orders table
 * @access public
 * @param string $columns
 */
function billzone_eu_invoice_add_new_order_column_invoice_number($columns)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_add_new_order_column_invoice_number > START');
    $columns['billzone_eu_invoice_invoice_number'] = __('billzone.eu invoice nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_add_new_order_column_invoice_number > END');
    return $columns;
}
add_filter('manage_edit-shop_order_columns', 'billzone_eu_invoice_add_new_order_column_invoice_number');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Populate newly added "billzone.eu invoce nr." column in Woocommerce orders table
 * @access public
 * @param string $columns
 */
function billzone_eu_invoice_populate_newly_added_invoice_number_column($column)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_populate_newly_added_invoice_number_column > START');

    global $post;
    global $wpdb;

    if ($column == 'billzone_eu_invoice_invoice_number') {
        if (bei_check_if_install_is_OK()) {

            $bei_check_if_invoice_exist = $wpdb->get_results(
                "
                SELECT invoice_number, invoice_type
                FROM " . BILLZONE_EU_INVOICE_TABLE_NAME .
                " WHERE order_id = $post->ID
                "
            );

            $bei_all_invoice_nr = '';
            foreach ($bei_check_if_invoice_exist as $bei_invoice) {

                if (empty($bei_invoice->invoice_type) || $bei_invoice->invoice_type == 'normal') {
                    $bei_img = 'billzone-eu-invoice-invoice-type-normal.svg';
                    $bei_invoice_type_help_txt = __('Normal invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME);
                } else {
                    $bei_img = 'billzone-eu-invoice-invoice-type-void.svg';
                    $bei_invoice_type_help_txt = __('Void invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME);
                }

                $bei_all_invoice_nr .= '<span class="billzone-eu-invoice-invoice-type-wrapper">';
                $bei_all_invoice_nr .= '    <span class="billzone-eu-invoice-invoice-type-help-text">' . $bei_invoice_type_help_txt . '</span>';
                $bei_all_invoice_nr .= '    <img class="billzone-eu-invoice-invoice-type-img" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/' . $bei_img . '">';
                $bei_all_invoice_nr .= $bei_invoice->invoice_number;
                $bei_all_invoice_nr .= '</span>,';
            }
            $bei_all_invoice_nr = trim(substr($bei_all_invoice_nr, 0, -1));
            if ($bei_all_invoice_nr == '') {
                echo '---';
            } else {
                echo $bei_all_invoice_nr;
            }
            billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_populate_newly_added_invoice_number_column > inside IF');
        }
        else {
            echo __('Please reinstall plugin!!!', BILLZONE_EU_INVOICE_PLUGIN_NAME); 
        }
    }
    

    billzone_eu_invoice_logme(__FILE__, 'OK', '#3: function billzone_eu_invoice_populate_newly_added_invoice_number_column > END');

}
add_action('manage_shop_order_posts_custom_column', 'billzone_eu_invoice_populate_newly_added_invoice_number_column');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add new tab on Woocommerce settings area
 * @access public
 */
// function billzone_eu_invoice_add_settings_tab()
// {

//     billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_add_settings_tab > START');

//     $current_tab = (!empty($_GET["a"]) && $_GET['tab'] == BILLZONE_EU_INVOICE_PLUGIN_NAME . '-settings-tab') ? 'nav-tab-active' : '';

//     echo '<a href="admin.php?page=wc-settings&amp;tab=' . BILLZONE_EU_INVOICE_PLUGIN_NAME . '-settings-tab" class="billzone-settings-tab nav-tab ' . $current_tab . '">' . __('billzone.eu Invoice settings', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</a>';

//     billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_add_settings_tab > END');
// }
// add_action('woocommerce_settings_tabs', 'billzone_eu_invoice_add_settings_tab'); // Action to call billzone_eu_invoice_add_settings_tab() function
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add new tab on Woocommerce settings area
 * @access public
 */
class BEI_Add_Billzone_Eu_Invoice_Settings_Tab_To_WooCommerce
{

    public static function init()
    {
        add_filter('woocommerce_settings_tabs_array', __CLASS__ . '::add_settings_tab', 50);
    }

    public static function add_settings_tab($settings_tabs)
    {
        $settings_tabs[BILLZONE_EU_INVOICE_PLUGIN_NAME . '-settings-tab'] = __('billzone.eu Invoice settings', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        return $settings_tabs;
    }
}
BEI_Add_Billzone_Eu_Invoice_Settings_Tab_To_WooCommerce::init();
//----------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add content for the new settings tab in Woocommerce settings area
 * @access public
 */
function billzone_eu_invoice_get_settings()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_get_settings > START');

    global $bei_options;

    // Get all enabled woocommerce payment method
    $bei_payment_method_display_title = '<div><h4>' . __('Woocommerce payment method', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
    $bei_payment_method_display_code = '<div><h4>' . __('Woocommerce payment code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';

    foreach (WC()->payment_gateways->payment_gateways() as $bei_payment_gateway) {
        if (strtoupper($bei_payment_gateway->enabled) == "YES") {
            $bei_payment_method_display_title .= '<div>' . $bei_payment_gateway->title . ':</div>';
            $bei_payment_method_display_code .= '<div><b>' . strtoupper($bei_payment_gateway->id) . '</b></div>';
        }
    }

    $bei_payment_method_table = '<div class="billzone-eu-invoice-settings-help-wrapper"><div>' . $bei_payment_method_display_title . '</div><div>' . $bei_payment_method_display_code . '</div></div>';

    // If there is any error, set a different class to be more visible

    $bei_get_response_status_code = get_option($bei_options['billzone_eu_invoice_security_code_status']);
    if (empty($bei_get_response_status_code)) {
        $bei_get_response_status_code = BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE;
    }

    switch ($bei_get_response_status_code) {

        case BILLZONE_EU_INVOICE_OK_SECURITY_CODE:

            $bei_get_all_billzone_lists = get_option($bei_options['billzone_eu_invoice_all_billzone_data_list']);

            $bei_security_code_message = ' <div class="billzone-eu-invoice-settings-security-code-msg billzone-eu-invoice-ok">';
            $bei_security_code_message .= ' <p>' . __('Security code status:', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' ' . bei_get_API_error($bei_get_response_status_code)['message'] . '</p>';
            if (!empty($bei_get_all_billzone_lists)) {

                $bei_get_all_billzone_lists_obj = json_decode($bei_get_all_billzone_lists);
                $bei_temp_eutaxnr = (empty($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->EUTaxNumber)) ? '---': $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->EUTaxNumber;
                $bei_temp_taxnr = (empty($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->TaxNumber)) ? '---': $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->TaxNumber;
                $bei_temp_regnr = (empty($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->RegistrationNumber)) ? '---': $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->RegistrationNumber;

                $bei_security_code_message .= '<div class="billzone-eu-invoice-invoice-issuer-data-wrapper">';
                $bei_security_code_message .= '     <div>';
                $bei_security_code_message .= '     ' . __('Invoice issuer data', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':';
                $bei_security_code_message .= '         <ul>';
                $bei_security_code_message .= '             <li>' . __('Name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <strong>' . $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->Name . '</strong></li>';
                $bei_security_code_message .= '             <li>' . __('Address', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <strong>' . $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->AddressCountryId . '-' . $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->AddressPostalCode . ' ' . $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->AddressCity . ', ' . $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->AddressLine1 . ' ' . $bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->CompanyData->AddressLine2 . '</strong></li>';
                $bei_security_code_message .= '             <li>' . __('EU tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <strong>' . $bei_temp_eutaxnr . '</strong></li>';
                $bei_security_code_message .= '             <li>' . __('Tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <strong>' . $bei_temp_taxnr . '</strong></li>';
                $bei_security_code_message .= '             <li>' . __('Registration nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <strong>' . $bei_temp_regnr . '</strong></li>';
                $bei_security_code_message .= '         </ul>';
                $bei_security_code_message .= '     </div>';
                $bei_security_code_message .= '</div>';
            }
            $bei_security_code_message .= '</div>';

            if (!empty($bei_get_all_billzone_lists)) {
                $bei_get_all_billzone_lists_obj = json_decode($bei_get_all_billzone_lists);

                $bei_col_left = '<div><h4>' . __('Currency', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                $bei_col_right = '<div><h4>' . __('Available invoice number prefix', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                foreach ($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->AccountBlocks->AccountBlocksQueryResultItem as $bei_invoice_number_prefix) {
                    $bei_col_left .= '<div>' . $bei_invoice_number_prefix->CurrencyShortName . '</div>';
                    $bei_col_right .= '<div><b>' . $bei_invoice_number_prefix->AccountBlockPrefix . '</b></div>';
                }
                $bei_available_invoice_number_prefixes = '<div class="billzone-eu-invoice-settings-help-wrapper"><div>' . $bei_col_left . '</div><div>' . $bei_col_right . '</div></div>';

                $bei_col_left = '<div><h4>' . __('Name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                $bei_col_right = '<div><h4>' . __('Location ID', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                foreach ($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->Issuers->IssuersQueryResultItem as $bei_company_location_id) {
                    $bei_col_left .= '<div>' . $bei_company_location_id->Name . '</div>';
                    $bei_col_right .= '<div><b>' . $bei_company_location_id->InvoiceIssuerIdentifier . '</b></div>';
                }
                $bei_available_company_location_id = '<div class="billzone-eu-invoice-settings-help-wrapper"><div>' . $bei_col_left . '</div><div>' . $bei_col_right . '</div></div>';

                $bei_col_left = '<div><h4>' . __('Name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                $bei_col_right = '<div><h4>' . __('Unit ID', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                foreach ($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->Units->UnitsQueryResultItem as $bei_unit) {
                    $bei_col_left .= '<div>' . $bei_unit->Name . '</div>';
                    $bei_col_right .= '<div><b>' . $bei_unit->UnitIdentifier . '</b></div>';
                }
                $bei_available_default_pricing_unit = '<div class="billzone-eu-invoice-settings-help-wrapper"><div>' . $bei_col_left . '</div><div>' . $bei_col_right . '</div></div>';

                $bei_available_tax_country_code_via_API = '';
                $bei_all_tax_country_code = bei_get_all_API_returned_tax_country_code($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->Taxes->TaxesQueryResultItem);
                $bei_lng = strtoupper(get_option($bei_options['billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code']));
                if (array_key_exists(strtoupper($bei_lng), $bei_all_tax_country_code)) {
                    $bei_col_left = '<div><h4>' . __('Country code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                    $bei_col_middle_right = '<div><h4>' . __('Country name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
                    foreach ($bei_all_tax_country_code as $bei_tax_country_code) {
                        $bei_col_left .= '<div>' . $bei_tax_country_code . '</div>';
                        $bei_col_middle_right .= '<div><b>' . bei_get_all_country_code()[$bei_tax_country_code] . '</b></div>';
                    }
                    $bei_col_middle_left = '<div><h4>' . __('Tax code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (' . $bei_lng . ')</h4></div>';
                    $bei_col_right = '<div><h4>' . __('Tax percentage / Display name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (' . $bei_lng . ')</h4></div>';
                    $bei_all_tax_code_for_selected_country = array();
                    foreach ($bei_get_all_billzone_lists_obj->GetInvoiceDataResult->QueryResult->Taxes->TaxesQueryResultItem as $bei_one_tx) {
                        if ($bei_one_tx->CountryId == $bei_lng) {
                            if (!array_key_exists($bei_one_tx->TaxRateCode, $bei_all_tax_code_for_selected_country)) {
                                $bei_one_tx_detail = array();
                                $bei_one_tx_detail['Percentage'] = $bei_one_tx->Percentage;
                                $bei_one_tx_detail['DisplayName'] = $bei_one_tx->DisplayName;
                                $bei_all_tax_code_for_selected_country[$bei_one_tx->TaxRateCode] = $bei_one_tx_detail;
                            }
                        }
                    }
                    foreach ($bei_all_tax_code_for_selected_country as $bei_tax_code_for_selected_country_key => $bei_tax_code_for_selected_country_value) {
                        $bei_col_middle_left .= '<div>' . $bei_tax_code_for_selected_country_key . '</div>';
                        $bei_col_right .= '<div><b>' . bei_b_round($bei_tax_code_for_selected_country_value['Percentage']) . '%</b> - ' . $bei_tax_code_for_selected_country_value['DisplayName'] . '</div>';
                    }
                    $bei_available_tax_country_code_via_API = '<div class="billzone-eu-invoice-settings-help-wrapper"><div>' . $bei_col_left . '</div><div>' . $bei_col_middle_right . '</div><div>' . $bei_col_middle_left . '</div><div>' . $bei_col_right . '</div></div>';
                } else {
                    $bei_available_tax_country_code_via_API = '';
                }

            } else {
                $bei_available_invoice_number_prefixes = '';
                $bei_available_company_location_id = '';
                $bei_available_default_pricing_unit = '';
                $bei_available_tax_country_code_via_API = '';
            }
            break;

        case BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE:
            $bei_security_code_message = '<div class="billzone-eu-invoice-settings-security-code-msg billzone-eu-invoice-empty">' . __('Security code status:', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' ' . bei_get_API_error($bei_get_response_status_code)['message'] . '</div>';
            $bei_available_invoice_number_prefixes = '';
            $bei_available_company_location_id = '';
            $bei_available_default_pricing_unit = '';
            $bei_available_tax_country_code_via_API = '';
            break;

        default:
            $bei_array = bei_get_API_error($bei_get_response_status_code);
            $bei_security_code_message = '<div class="billzone-eu-invoice-settings-security-code-msg billzone-eu-invoice-error">' . __('Security code status:', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' [' . $bei_get_response_status_code . '] ' . bei_get_API_error($bei_get_response_status_code)['message'] . '</div>';
            $bei_available_invoice_number_prefixes = '';
            $bei_available_company_location_id = '';
            $bei_available_default_pricing_unit = '';
            $bei_available_tax_country_code_via_API = '';

    }

    $bei_wizard = '<div class="billzone-eu-invoice-wizard-wrapper">';
    $bei_wizard .= '    <div>'.__('You can find the plugins documentation <a href="https://www.billzone.eu/hu-en/billing/documents" target="_blank">here</a>.',BILLZONE_EU_INVOICE_PLUGIN_NAME).'</div>';
    $bei_wizard .= '    <div class="billzone-eu-invoice-wizard-call-to-action">' . __('By clicking on the button below a wizard will help you to easily set up billzone.eu Invoice in 3 steps...', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</div>';
    $bei_wizard .= '    <div>';
    $bei_wizard .= '        <span id="billzone_start_setup_wizard" class="billzone-eu-invoice-wizard-button">' . __('Setup wizard', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_wizard .= '    </div>';
    $bei_wizard .= '</div>';

    $bei_export_and_clear_log = '<div class="billzone-eu-invoice-export-and-clear-log-wrapper">';
    $bei_export_and_clear_log .= '    <span id="billzone_eu_invoice_export_log" class="billzone-eu-invoice-export-log">' . __('Export log', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_export_and_clear_log .= '    <span id="billzone_eu_invoice_clear_log" class="billzone-eu-invoice-clear-log">' . __('Clear log', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_export_and_clear_log .= '</div>';

    $bei_settings = array(
        'billzone_eu_invoice_settings_section_title' => array(
            'name' => __('billzone.eu Invoice settings', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (v' . BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM . ')',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_settings_section_title',
        ),
        'billzone_eu_invoice_settings_section_end_1' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_1',
        ),
        'billzone_eu_invoice_wizard' => array(
            'name' => '',
            'type' => 'title',
            'desc' => $bei_wizard,
            'id' => 'billzone_eu_invoice_wizard',
        ),
        'billzone_eu_invoice_settings_section_end_2' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_2',
        ),
        'billzone_eu_invoice_tax_rate_code' => array(
            'name' => '',
            'type' => 'title',
            'desc' => __('The system use UN/EDIFACT tax rate codes. For available tax rate codes, please check MASTER DATA / TAX RATE CODES menu in your billzone.eu account, or read <a href="https://www.billzone.eu/blog/2016/10/10/milyen-afa-kulcsokat-hasznal-a-rendszer/" target="_blank">this</a> article.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => 'billzone_eu_invoice_tax_rate_code',
        ),
        'billzone_eu_invoice_round_rule_warning' => array(
            'name' => '',
            'type' => 'title',
            'desc' => __('Attention! The rounding rules must be set under the [General] tab of the WooCommerce settings. But may differ from banking rounding rules used by billzone.eu', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => 'billzone_eu_invoice_round_rule_warning',
        ),
        $bei_options['billzone_eu_invoice_is_sandbox'] => array(
            'name' => __('Sandbox invoicing', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_is_sandbox'],
            'type' => 'checkbox',
            'desc' => __('If you want to test your billing in the sandbox, check this option.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        $bei_options['billzone_eu_invoice_security_code'] => array(
            'name' => __('Security code', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'text',
            'desc' => __('For your security code please check ACCOUNT SETTINGS / API SETTINGS menu in your billzone.eu account.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_security_code'],
        ),
        'billzone_eu_invoice_settings_section_end_3' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_3',
        ),
        'billzone_eu_invoice_security_code_message' => array(
            'name' => '',
            'type' => 'title',
            'desc' => $bei_security_code_message,
            'id' => 'billzone_eu_invoice_security_code_message',
        ),
        'billzone_eu_invoice_settings_section_end_4' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_4',
        ),
        'billzone_eu_invoice_invoice_number_prefix_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_invoice_number_prefix_title',
        ),
        $bei_options['billzone_eu_invoice_invoice_number_prefix'] => array(
            'name' => __('Invoice sequence prefixes', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'text',
            'desc' => __('For available invoice sequence prefixes, please check MASTER DATA / INVOICE SEQUENCE GROUPS menu in your billzone.eu account.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_invoice_number_prefix'],
        ),
        'billzone_eu_invoice_settings_section_end_5' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_5',
        ),
        'billzone_eu_invoice_available_invoice_number_prefixes' => array(
            'name' => '',
            'type' => 'title',
            'desc' => $bei_available_invoice_number_prefixes,
            'id' => 'billzone_eu_invoice_available_invoice_number_prefixes',
        ),
        'billzone_eu_invoice_settings_section_end_6' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_6',
        ),
        'billzone_eu_invoice_company_location_id_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_company_location_id_title',
        ),
        $bei_options['billzone_eu_invoice_company_location_id'] => array(
            'name' => __('Company location ID', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'text',
            'desc' => __('For available company locations ID, please check MASTER DATA / LOCATIONS menu in your billzone.eu account.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_company_location_id'],
        ),
        'billzone_eu_invoice_settings_section_end_7' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_7',
        ),
        'billzone_eu_invoice_available_company_location_id' => array(
            'name' => '',
            'type' => 'title',
            'desc' => $bei_available_company_location_id,
            'id' => 'billzone_eu_invoice_available_company_location_id',
        ),
        'billzone_eu_invoice_settings_section_end_8' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_8',
        ),
        'billzone_eu_invoice_default_pricing_unit_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_default_pricing_unit_title',
        ),
        $bei_options['billzone_eu_invoice_default_pricing_unit'] => array(
            'name' => __('Default unit of measure', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_default_pricing_unit'],
            'type' => 'text',
            'desc' => __('For available units of measure, please check MASTER DATA / UNITS OF MEASURE menu in your billzone.eu account.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_settings_section_end_9' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_9',
        ),
        'billzone_eu_invoice_available_default_pricing_unit' => array(
            'name' => '',
            'type' => 'title',
            'desc' => $bei_available_default_pricing_unit,
            'id' => 'billzone_eu_invoice_available_default_pricing_unit',
        ),
        'billzone_eu_invoice_settings_section_end_10' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_10',
        ),
        'billzone_eu_invoice_default_payment_methode_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_default_payment_methode_title',
        ),
        $bei_options['billzone_eu_invoice_default_payment_methode'] => array(
            'name' => __('Default payment method', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_default_payment_methode'],
            'type' => 'text',
            'desc' => __('For available payment method ID, please check MASTER DATA / PAYMENT METHOD menu in your billzone.eu account.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_settings_section_end_11' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_11',
        ),
        'billzone_eu_invoice_payment_gateway' => array(
            'name' => __('Payment gateway codes', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => 'billzone_eu_invoice_payment_gateway',
            'type' => 'title',
            'desc' => __('For proper operation add this payment method codes to your billzone.eu account MASTER DATA / PAYMENT METHOD menu:', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_payment_gateway_explain_4' => array(
            'name' => '',
            'id' => 'billzone_eu_invoice_payment_gateway_explain_4',
            'type' => 'title',
            'desc' => __('<small>Use <b>&raquo;New banking info&laquo;</b> link and add this code to <i>&raquo;Payment method identifier corresponding to interface (API)&laquo;</i> field.<br>For each code add <b>&raquo;New banking info&laquo;</b> line.</small>', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_payment_gateway_codes' => array(
            'name' => '',
            'id' => 'billzone_eu_invoice_payment_gateway_codes',
            'type' => 'title',
            'desc' => $bei_payment_method_table,
        ),
        'billzone_eu_invoice_settings_section_end_12' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_12',
        ),

        'billzone_eu_invoice_get_exchange_rate_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_get_exchange_rate_title',
        ),
        $bei_options['billzone_eu_invoice_get_exchange_rate'] => array(
            'name' => __('Get MNB exchange rate for domestic currency billing', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_get_exchange_rate'],
            'type' => 'checkbox',
            'desc' => __('Check this box for getting automatically from Hungarian Central Bank (MNB) the current exchange rate for domestic currency billing', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_settings_section_end_12A' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_12A',
        ),

        'billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code_title',
        ),
        $bei_options['billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code'] => array(
            'name' => __('Default country code for UN/EDIFACT codes', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'text',
            'default' => 'HU',
            'id' => $bei_options['billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code'],
            'desc' => __('If not configured, UN/EDIFACT VAT codes for the default language listed in the WordPress user profile will be used.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_settings_section_end_13' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_13',
        ),
        'billzone_eu_invoice_available_country_for_UN_EDIFACT_tax_code' => array(
            'name' => '',
            'id' => 'billzone_eu_invoice_available_country_for_UN_EDIFACT_tax_code',
            'type' => 'title',
            'desc' => $bei_available_tax_country_code_via_API,
        ),
        'billzone_eu_invoice_settings_section_end_14' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_14',
        ),
        'billzone_eu_invoice_all_remaining_data_title' => array(
            'name' => '',
            'type' => 'title',
            'desc' => '',
            'id' => 'billzone_eu_invoice_all_remaining_data_title',
        ),
        $bei_options['billzone_eu_invoice_date_of_delivery_plus_days'] => array(
            'name' => __('Date of delivery plus days', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'number',
            'default' => 0,
            'desc' => __('Default date of delivery: The date defined by the number of days determined in this field added to today’s date', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_date_of_delivery_plus_days'],
        ),
        $bei_options['billzone_eu_invoice_payment_due_plus_days'] => array(
            'name' => __('Payment due plus days', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'number',
            'default' => 0,
            'desc' => __('Default payment due: The date defined by the number of days determined in this field added to today’s date', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_payment_due_plus_days'],
        ),
        $bei_options['billzone_eu_invoice_invoice_notes'] => array(
            'name' => __('Invoice&nbsp;comment', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_invoice_notes'],
            'type' => 'text',
            'desc' => __('Leave it blank if you do not want to comment on your invoice. Can be overridden when billing.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        $bei_options['billzone_eu_invoice_send_email'] => array(
            'name' => __('Send invoice for customer via e-mail', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_send_email'],
            'type' => 'checkbox',
            'desc' => __('By selecting this option, the customer will receive the bill via email.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_settings_section_end_15' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_15',
        ),
        'billzone_eu_invoice_logging_title' => array(
            'name' => __('Debug section', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'type' => 'title',
            'desc' => __('Enabling the option below allows you to log billzone.eu Invoice plugin activity. After switching on, the order number, event time, type of activity (error or success) and a brief description of the activity will be stored. Clicking on the »EXPORT LOG« button will display the last 1000 row of these data. Clicking on the »CLEAR LOG« button will clear all log data. Use this option for <span style="color: red">DEBUGGING ONLY</span>. After use, <span style="color: red">TURN OFF</span> to avoid overloading the database with unnecessary data.', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => 'billzone_eu_invoice_logging_title',
        ),
        $bei_options['billzone_eu_invoice_log_is_on'] => array(
            'name' => __('Enable logging', BILLZONE_EU_INVOICE_PLUGIN_NAME),
            'id' => $bei_options['billzone_eu_invoice_log_is_on'],
            'type' => 'checkbox',
            'desc' => __('By selecting this option, you can enable logging for debug purposes. <span style="color: red">REMEMBER TO TURN IT OFF AFTER USE!</span>', BILLZONE_EU_INVOICE_PLUGIN_NAME),
        ),
        'billzone_eu_invoice_settings_section_end_16' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_16',
        ),
        'billzone_eu_invoice_log_export_and_clear' => array(
            'name' => '',
            'type' => 'title',
            'desc' => $bei_export_and_clear_log,
            'id' => 'billzone_eu_invoice_log_export_and_clear',
        ),
        'billzone_eu_invoice_settings_section_end_17' => array(
            'type' => 'sectionend',
            'id' => 'billzone_eu_invoice_settings_section_end_17',
        ),
    );

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_get_settings > END');

    return apply_filters('billzone_eu_invoice_settings', $bei_settings);
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Call billzone.eu Invoice billzone_eu_invoice_get_settings() function
 * @access public
 */
function billzone_eu_invoice_settings_tab()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_settings_tab > START');

    global $bei_options;

    woocommerce_admin_fields(billzone_eu_invoice_get_settings());

    // Add options to database
    if (empty(get_option($bei_options['billzone_eu_invoice_security_code_status']))) {
        add_option($bei_options['billzone_eu_invoice_security_code_status'], BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE, '', 'yes');
    }
    if (empty(get_option($bei_options['billzone_eu_invoice_all_billzone_data_list']))) {
        add_option($bei_options['billzone_eu_invoice_all_billzone_data_list'], '', '', 'yes');
    }

    // Add/refresh plugin version
    add_option(BILLZONE_EU_INVOICE_PLUGIN_VERSION_KEY, BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM);

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_settings_tab > END');
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to update billzone.eu Invoice parameters in database
 * @access public
 */
function billzone_eu_invoice_update_settings()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_update_settings > START');

    global $bei_options;

    $bei_security_code = trim($_POST['billzone_eu_invoice_security_code']);

    // check security code if not empty and is valid then retrive data from API
    if (!empty($bei_security_code)) {

        // get API type (sandbox or production)
        if (empty($_POST['billzone_eu_invoice_is_sandbox'])) {
            $bei_API_class_source = 'billzone-eu-invoice-invoicing-service.php';
        } else {
            $bei_API_class_source = 'billzone-eu-invoice-invoicing-service-sandbox.php';
        }

        // create invoice request instance
        $bei_invoice = new BEI_INVOICE($bei_API_class_source);

        // maping data to API data
        $bei_invoice->RequestId = bei_create_GUID();
        $bei_invoice->SecurityToken = $bei_security_code;

        // create API request
        $bei_request_result = $bei_invoice->get_invoice_data();

        $bei_security_code_status = $bei_request_result['GetInvoiceDataResult']->RequestResult->Code;

    } else {
        $bei_security_code_status = BILLZONE_EU_INVOICE_EMPTY_SECURITY_CODE;
    }

    update_option($bei_options['billzone_eu_invoice_security_code_status'], $bei_security_code_status);

    if (($bei_security_code_status == BILLZONE_EU_INVOICE_OK_SECURITY_CODE) &&
        ($bei_request_result['GetInvoiceDataResult']->QueryResult->ResultCode->Code == BILLZONE_EU_INVOICE_OK_SECURITY_CODE)) {
        update_option($bei_options['billzone_eu_invoice_all_billzone_data_list'], json_encode($bei_request_result));
    } else {
        update_option($bei_options['billzone_eu_invoice_all_billzone_data_list'], '');
    }

    update_option($bei_options[BILLZONE_EU_INVOICE_PLUGIN_VERSION_KEY], BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM);

    woocommerce_update_options(billzone_eu_invoice_get_settings());

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_update_settings > END');
}

// Action to call billzone_eu_invoice_settings_tab() function
add_action('woocommerce_settings_tabs_' . BILLZONE_EU_INVOICE_PLUGIN_NAME . '-settings-tab', 'billzone_eu_invoice_settings_tab');
// Action to call billzone_eu_invoice_update_settings() function
add_action('woocommerce_update_options_' . BILLZONE_EU_INVOICE_PLUGIN_NAME . '-settings-tab', 'billzone_eu_invoice_update_settings');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add billzone.eu Invoice menu item on admin page left sidebar
 * @access public
 */
function add_billzone_eu_invoice_menu_item()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function add_billzone_eu_invoice_menu_item > START');

    global $bei_admin_sidebar_menu_order;
    global $menu;

    // Check for WooCommerce sidebar menu item order to insert our plugin link after them
    foreach ($GLOBALS['menu'] as $bei_menu_item) {
        if (strtolower($bei_menu_item[0]) == 'woocommerce') {
            $bei_admin_sidebar_menu_order = array_search($bei_menu_item, $GLOBALS['menu']) + 1;
            break;
        }
    }

    // Add sidebar menu item after WooCommerce sidebar menu
    add_menu_page('billzone_eu_invoice_sidebar_menu_item', BILLZONE_EU_INVOICE_READABLE_PLUGIN_NAME, 'install_plugins', 'billzone_eu_invoice', '', '', $bei_admin_sidebar_menu_order);
    // Add link to newly created sidebar menu item
    $menu[$bei_admin_sidebar_menu_order][2] = "admin.php?page=wc-settings&tab=billzone-eu-invoice-settings-tab";

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function add_billzone_eu_invoice_menu_item > END');

}
add_action('admin_menu', 'add_billzone_eu_invoice_menu_item');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
// Add billzone.eu Invoice meta box to order detail page

/**
 * Helper function to generate UN/EDIFACT <option> tag for a <select> element
 * @param string $bei_items -> mandatory parameter
 * @param string $bei_check_against -> optional parameter, if missing, the first element will be selected by default
 * @access public
 */
function billzone_eu_invoice_generate_UNEDIFACT_items_for_select($bei_items, $bei_check_against = null)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_generate_UNEDIFACT_items_for_select > START');

    $bei_HTML_options = '';

    if ($bei_check_against === null) {
        foreach ($bei_items as $bei_item_key => $bei_items_value) {
            $bei_HTML_options .= '<option value="' . $bei_items_value . '">' . $bei_item_key . '</option>';
        }
    } else {
        foreach ($bei_items as $bei_item_key => $bei_items_value) {
            $bei_selected_option = '';
            if ($bei_items_value == $bei_check_against) {
                $bei_selected_option = 'selected';
            }
            $bei_HTML_options .= '<option value="' . $bei_items_value . '" ' . $bei_selected_option . '>' . $bei_item_key . '</option>';
        }
    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_generate_UNEDIFACT_items_for_select > END');

    return $bei_HTML_options;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to show one invoice item row
 * @param string $bei_special_tax_class -> special CSS class to highlight non-standard VAT lines
 * @param int $bei_row_counter -> item row counter
 * @param string $bei_item_data_image -> the small product or item image
 * @param string $bei_item_data_name -> item name
 * @param string $bei_item_data_all_meta_data_concatenated -> additional metadata
 * @param float $bei_item_data_quantity_value -> item quantity value
 * @param string $bei_item_data_quantity_string -> item quantity value in string format
 * @param string $bei_item_data_unit_of_measure_value -> item unit of measure value
 * @param string $bei_item_data_unit_of_measure_string -> item unit of measure value in string format
 * @param float $bei_item_data_net_unit_price_value -> item net unit price value
 * @param string $bei_item_data_net_unit_price_string -> item net unit price value in string format
 * @param float $bei_item_data_net_price_value -> item total net value
 * @param string $bei_item_data_net_price_string -> item total net value in string format
 * @param float $bei_item_data_tax_percentage_value -> item VAT percentage value
 * @param string $bei_item_data_tax_percentage_string -> item VAT percentage value in string format
 * @param array $bei_order_data_UN_EDIFACT_codes -> all UN/EDIFACT TAX codes
 * @param float $bei_item_data_tax_value -> item total tax value
 * @param string $bei_item_data_tax_string -> item total tax value in string format
 * @param float $bei_item_data_gross_price_value -> item gross price value
 * @param string $bei_item_data_gross_price_string -> item gross price value in string format
 * @access public
 */
function billzone_eu_invoice_one_item_row($bei_special_tax_class, $bei_row_counter,
    $bei_item_data_image, $bei_item_data_name, $bei_item_data_all_meta_data_concatenated,
    $bei_item_data_quantity_value, $bei_item_data_quantity_string,
    $bei_item_data_unit_of_measure_value, $bei_item_data_unit_of_measure_string,
    $bei_item_data_net_unit_price_value, $bei_item_data_net_unit_price_string,
    $bei_item_data_net_price_value, $bei_item_data_net_price_string,
    $bei_item_data_tax_percentage_value, $bei_item_data_tax_percentage_string,
    $bei_order_data_UN_EDIFACT_codes,
    $bei_item_data_tax_value, $bei_item_data_tax_string,
    $bei_item_data_gross_price_value, $bei_item_data_gross_price_string) {

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_one_item_row > START');

    $bei_html = '          <tr class="billzone-eu-invoice-detail-table-data ' . $bei_special_tax_class . '">';
    $bei_html .= '              <td>#' . $bei_row_counter . ' ' . $bei_item_data_image . '</td>';
    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="name" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . strip_tags($bei_item_data_name) . '">';
    $bei_html .= '                  <input data-beifieldname="meta" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_all_meta_data_concatenated . '">';
    $bei_html .= '                  <span>' . $bei_item_data_name . '</span>';
    $bei_html .= '                  <br><small><i>' . $bei_item_data_all_meta_data_concatenated . '</i></small>';
    $bei_html .= '              </td>';
    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="quantity" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_quantity_value . '">';
    $bei_html .= '                  <span>' . $bei_item_data_quantity_string . '</span>';
    $bei_html .= '              </td>';

    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="unit_of_measure" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_unit_of_measure_value . '">';
    $bei_html .= '                  <span>' . $bei_item_data_unit_of_measure_string . '</span>';
    $bei_html .= '              </td>';

    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="net_unit_price" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_net_unit_price_value . '">';
    $bei_html .= '                  <span>' . $bei_item_data_net_unit_price_string . '</span>';
    $bei_html .= '              </td>';
    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="net_price" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_net_price_value . '">';
    $bei_html .= '                  <span>' . $bei_item_data_net_price_string . '</span>';
    $bei_html .= '              </td>';
    $bei_html .= '              <td class="billzone-eu-invoice-item-UN-EDIFACT-cell bei-disabled">';
    $bei_html .= '                  <div class="billzone-eu-invoice-item-UN-EDIFACT-cell-flex">';
    $bei_html .= '                      <div>';
    $bei_html .= '                          <small class="billzone-eu-invoice-item-UN-EDIFACT-warning">' . __('You can enable editing below', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</small>';
    $bei_html .= '                          <span class="billzone-eu-invoice-item-UN-EDIFACT-value" data-originaltaxpercentage="' . $bei_item_data_tax_percentage_string . '">' . $bei_item_data_tax_percentage_string . '</span><span>%</span>';
    $bei_html .= '                      </div>';
    $bei_html .= '                      <div>';
    $bei_html .= '                          <select data-beifieldname="tax_code" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" data-originaltaxpercentage="' . $bei_item_data_tax_percentage_string . '" class="billzone-eu-invoice-item-UN-EDIFACT-codes" disabled>';
    $bei_html .= billzone_eu_invoice_generate_UNEDIFACT_items_for_select($bei_order_data_UN_EDIFACT_codes, $bei_item_data_tax_percentage_value);
    $bei_html .= '                          </select>';
    $bei_html .= '                      </div>';
    $bei_html .= '                  </div>';
    $bei_html .= '              </td>';
    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="tax" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_tax_value . '">';
    $bei_html .= '                  <span class="billzone-eu-invoice-item-VAT-value" data-originaltaxpercentage="' . $bei_item_data_tax_percentage_string . '">' . $bei_item_data_tax_string . '</span>';
    $bei_html .= '              </td>';
    $bei_html .= '              <td>';
    $bei_html .= '                  <input data-beifieldname="gross_price" data-beitype="line" data-beilinenr="' . $bei_row_counter . '" type="hidden" value="' . $bei_item_data_gross_price_value . '">';
    $bei_html .= '                  <span class="billzone-eu-invoice-item-gross-value" data-originaltaxpercentage="' . $bei_item_data_tax_percentage_string . '">' . $bei_item_data_gross_price_string . '</span>';
    $bei_html .= '              </td>';
    $bei_html .= '          </tr>';

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_one_item_row > END');

    return $bei_html;

}

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to generate order detail popup inside elements for #billzone_eu_invoice_Create_Button javascript button event
 * @param string $bei_order_data -> input var containing all order data
 * @access public
 */
function billzone_eu_invoice_add_popup_inside_elements($bei_order_data)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_add_popup_inside_elements > START | Order ID: ' . $bei_order_data['id']);

    global $bei_options;

    $bei_html = '';

    // -- START -- Container header ----------------------------------------------------------------------------------------

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_add_popup_inside_elements > CONTAINER HEADER IN | Order ID: ' . $bei_order_data['id']);

    $bei_html .= '      <div class="billzone-eu-invoice-detail-container-header">';

    $bei_html .= '          <div class="billzone-eu-invoice-transaction-result-message-wrapper">';

    $bei_html .= '              <table class="billzone-eu-invoice-transaction-result-message-table">';
    $bei_html .= '                  <tr>';
    $bei_html .= '                      <td>';
    $bei_html .= '                          <img class="billzone-eu-invoice-transaction-result-succes" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-transaction-succes.svg" alt="' . __('Transaction OK icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">';
    $bei_html .= '                          <img class="billzone-eu-invoice-transaction-result-error" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-transaction-error.svg" alt="' . __('Transaction Error icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">';
    $bei_html .= '                      </td>';
    $bei_html .= '                      <td>';
    $bei_html .= '                          <ul class="billzone-eu-invoice-transaction-result-message-for-succes">';
    $bei_html .= '                              <li>' . __('Your invoice has been successfully submitted!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</li>';
    $bei_html .= '                              <li>' . __('Invoice number:', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '&nbsp;&nbsp<b class="billzone-eu-invoice-transaction-result-invoice-number"></b></li>';
    $bei_html .= '                          </ul>';
    $bei_html .= '                          <ul class="billzone-eu-invoice-transaction-result-message-for-error">';
    $bei_html .= '                              <li>' . __('Invoice submit failed!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</li>';
    $bei_html .= '                              <li class="billzone-eu-invoice-transaction-result-server-message"></li>';
    $bei_html .= '                          </ul>';
    $bei_html .= '                      </td>';
    $bei_html .= '                  </tr>';
    $bei_html .= '              </table>';

    $bei_html .= '              <div class="billzone-eu-invoice-transaction-result-button-wrapper">';
    $bei_html .= '                  <span id="beiTransactionResultMessageOkButton" class="button button-primary">' . __('Ok', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '              </div>';

    $bei_html .= '          </div>';

    $bei_html .= '          <h3 class="billzone-eu-invoice-info">';
    $bei_html .= __('billzone.eu Invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': ';
    $bei_html .= '              <i>' . __('Payment info', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</i> (v' . BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM . ')';
    $bei_html .= '              <span class="dashicons dashicons-info"></span>';
    $bei_html .= '          </h3>';
    $bei_html .= '          <span title="' . __('Close', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '" class="billzone-eu-invoice-close dashicons dashicons-no-alt"></span>';
    $bei_html .= '          <span class="billzone-eu-invoice-order-nr">' . __('Order:', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' #' . $bei_order_data['id'] . '</span>';
    $bei_html .= '          <div class="billzone-eu-invoice-info-text">';
    $bei_html .= '              <span class="dashicons dashicons-info"></span>';
    $bei_html .= '              <small>' . __('Attention! Only the parameters of the account can be changed on this interface. Changes to the original order data are not affected. If you need to change your original ordering information, you can do so on the Woocommerce interface. VAT fields are only used to enter the exact code or indicate the percentage of the code. However, the value of VAT cannot be changed, so you should pay close attention to the VAT setting!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</small>';
    $bei_html .= '              <span class="billzone-eu-invoice-info-text-close dashicons dashicons-no-alt"></span>';
    $bei_html .= '          </div>';
    $bei_html .= '      </div>';

    billzone_eu_invoice_logme(__FILE__, 'OK', '#3: function billzone_eu_invoice_add_popup_inside_elements > CONTAINER HEADER OUT | Order ID: ' . $bei_order_data['id']);

    // -- END ---- Container header ----------------------------------------------------------------------------------------

    $bei_invoice_prefix = sanitize_text_field(trim(get_option($bei_options['billzone_eu_invoice_invoice_number_prefix'])));

    // -- START -- Domestic currency billing warning text ------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#4: function billzone_eu_invoice_add_popup_inside_elements > DOMESTIC CURRENCY BILLING WARNING IN | Order ID: ' . $bei_order_data['id']);
    if (array_key_exists($bei_invoice_prefix, $bei_order_data['all_invoice_prefixes_and_associated_currency'])) {
        if (strtoupper($bei_order_data['invoice_issuer_data']['CountryDefaultCurrency']) != strtoupper($bei_order_data['all_invoice_prefixes_and_associated_currency'][$bei_invoice_prefix])) {

            $bei_html .= '  <div class="billzone-eu-invoice-domestic-currency-billing">';
            $bei_html .= '      ' . __('Domestic currency billing. Exchange rate is required!', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            $bei_html .= '  </div>';

        }
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#5: function billzone_eu_invoice_add_popup_inside_elements > DOMESTIC CURRENCY BILLING WARNING OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Domestic currency billing warning text -------------------------------------------------------------------

    // -- START -- Container body ------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#6: function billzone_eu_invoice_add_popup_inside_elements > CONTAINER BODY IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '      <div class="billzone-eu-invoice-detail-container-items">';

    // -- START -- Billing info --------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#7: function billzone_eu_invoice_add_popup_inside_elements > BILLING INFO IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '          <div class="billzone-eu-invoice-billing-info-wrapper">';

    // -- START -- Invoice issuer data -------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#8: function billzone_eu_invoice_add_popup_inside_elements > ISSUER DATA IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '              <div>';
    $bei_html .= '                  <div class="billzone-eu-invoice-invoice-issuer-wrapper">';

    $bei_html .= '                      <div class="billzone-eu-invoice-invoice-issuer-title" >' . __('Invoice issuer', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</div>';
    $bei_html .= '                      <ul class="billzone-eu-invoice-invoice-issuer-data-1">';
    $bei_html .= '                          <li>' . $bei_order_data['invoice_issuer_data']['Name'] . '</li>';
    $bei_html .= '                          <li>' . $bei_order_data['invoice_issuer_data']['Address'] . '</li>';
    $bei_html .= '                      </ul>';
    $bei_html .= '                      <ul class="billzone-eu-invoice-invoice-issuer-data-2">';
    $bei_html .= '                          <li>' . __('EU tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</li>';

    if (empty($bei_order_data['invoice_issuer_data']['EUTaxNumber'])) {
        $bei_html .= '                      <li>---</li>';
    }
    else {
        $bei_html .= '                      <li>' . $bei_order_data['invoice_issuer_data']['EUTaxNumber'] . '</li>';
    }
    $bei_html .= '                          <li>' . $bei_order_data['invoice_issuer_data']['EUTaxNumber'] . '</li>';
    $bei_html .= '                      </ul>';
    $bei_html .= '                      <ul class="billzone-eu-invoice-invoice-issuer-data-2">';
    $bei_html .= '                          <li>' . __('Tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</li>';
    if (empty($bei_order_data['invoice_issuer_data']['TaxNumber'])) {
        $bei_html .= '                      <li>---</li>';
    }
    else {
        $bei_html .= '                      <li>' . $bei_order_data['invoice_issuer_data']['TaxNumber'] . '</li>';
    }
    $bei_html .= '                      </ul>';
    $bei_html .= '                      <ul class="billzone-eu-invoice-invoice-issuer-data-2">';
    $bei_html .= '                          <li>' . __('Registration nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</li>';
    if (empty($bei_order_data['invoice_issuer_data']['RegistrationNumber'])) {
        $bei_html .= '                      <li>---</li>';
    }
    else {
        $bei_html .= '                      <li>' . $bei_order_data['invoice_issuer_data']['RegistrationNumber'] . '</li>';
    }
    $bei_html .= '                      </ul>';
    $bei_html .= '                      <ul class="billzone-eu-invoice-invoice-issuer-data-2">';
    $bei_html .= '                          <li>' . __('Country default currency', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</li>';
    $bei_html .= '                          <li>' . $bei_order_data['invoice_issuer_data']['CountryDefaultCurrency'] . '</li>';
    $bei_html .= '                      </ul>';

    $bei_html .= '                  </div>';
    $bei_html .= '              </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#9: function billzone_eu_invoice_add_popup_inside_elements > ISSUER DATA OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Invoice issuer data ---------------------------------------------------------------------------------------

    // -- START -- Payment customer name -------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#10: function billzone_eu_invoice_add_popup_inside_elements > PAYMENT CUSTOMER NAME IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '              <div>';
    $bei_html .= '                  <table class="billzone-eu-invoice-customer-info">';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('Customer name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td>';
    $bei_html .= '                              <input data-beifieldname="order_id" data-beitype="header" type="hidden" value="' . $bei_order_data['id'] . '">';
    $bei_html .= '                              <input data-beifieldname="order_key" data-beitype="header" type="hidden" value="' . $bei_order_data['order_key'] . '">';
    $bei_html .= '                              <input data-beifieldname="customer_name" data-beitype="header" type="text" value="' . $bei_order_data['billing']['customer_name'] . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="2" class="billzone-eu-invoice-validation-message" data-validationmessagefor="customer_name">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('E-mail', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td>';
    $bei_html .= '                              <input data-beifieldname="email" data-beitype="header" type="email" value="' . $bei_order_data['billing']['email'] . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="2" class="billzone-eu-invoice-validation-message" data-validationmessagefor="email">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                              <span>' . __('Invalid e-mail format!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>' . __('Phone', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';
    $bei_html .= '                          <td>';
    $bei_html .= '                              <input data-beifieldname="phone" data-beitype="header" type="text" value="' . $bei_order_data['billing']['phone'] . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="2" class="billzone-eu-invoice-validation-message">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>' . __('VAT registration no.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';
    $bei_html .= '                          <td>';
    $bei_html .= '                              <input data-beifieldname="VAT_number" data-beitype="header" type="text" value="">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="2" class="billzone-eu-invoice-validation-message" data-validationmessagefor="VAT_number">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>' . __('EU&nbsp;VAT&nbsp;reg.&nbsp;no.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';
    $bei_html .= '                          <td>';
    $bei_html .= '                              <input data-beifieldname="EU_VAT_number" data-beitype="header" type="text" value="">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="2" class="billzone-eu-invoice-validation-message" data-validationmessagefor="EU_VAT_number">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    
    $bei_html .= '                  </table>';
    $bei_html .= '              </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#11: function billzone_eu_invoice_add_popup_inside_elements > PAYMENT CUSTOMER NAME OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Payment customer name -------------------------------------------------------------------------------------

    // -- START -- Payment address -------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#12: function billzone_eu_invoice_add_popup_inside_elements > PAYMENT ADDRESS IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '              <div>';
    $bei_html .= '                  <table class="billzone-eu-invoice-payment-info">';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>' . __('Country', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td>';

    $bei_html .= '                              <select data-beifieldname="country" data-beitype="header" class="billzone-eu-invoice-country-codes">';
    foreach ($bei_order_data['all_country_codes'] as $bei_country_code_key => $bei_country_code_value) {
        $bei_selected = '';
        if (strtoupper(trim($bei_country_code_key)) == strtoupper($bei_order_data['billing']['country'])) {
            $bei_selected = 'selected';
        }
        $bei_html .= '                              <option value="' . strtoupper(trim($bei_country_code_key)) . '" ' . $bei_selected . '>' . $bei_country_code_value . '</option>';
    }
    $bei_html .= '                              </select>';

    $bei_html .= '                          </td>';
    $bei_html .= '                          <td><strong class="billzone-eu-invoice-country-code-value">' . strtoupper($bei_order_data['billing']['country']) . '</strong></td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('City', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td colspan=2>';
    $bei_html .= '                              <input data-beifieldname="city" data-beitype="header" type="text" value="' . $bei_order_data['billing']['city'] . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message" data-validationmessagefor="city">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('Postal code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td colspan=2>';
    $bei_html .= '                              <input data-beifieldname="postcode" data-beitype="header" type="text" value="' . $bei_order_data['billing']['postcode'] . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message" data-validationmessagefor="postcode">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('Address', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td colspan=2>';
    $bei_html .= '                              <input data-beifieldname="address" data-beitype="header" type="text" value="' . $bei_order_data['billing']['address_concatenated'] . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message" data-validationmessagefor="address">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>';
    $bei_html .= '                          </td>';
    $bei_html .= '                          <td  colspan="2">';
    $bei_is_hu_tax_legal_ent = (bei_Is_Hungarian_Tax_Entity($bei_order_data['invoice_issuer_data'])) ? 'billzone-eu-invoice-has-hu-tax-and-legal-ent-wrapper' : 'billzone-eu-invoice-has-hu-tax-and-legal-ent-wrapper billzone-eu-invoice-hide';

    $bei_html .= '                              <div class="' . $bei_is_hu_tax_legal_ent . '">';
    
    if ((empty($bei_order_data['billing']['VAT_number']))) {
        $bei_legal_entity_wout_tax_nr = 'billzone-eu-invoice-legal-entity-wout-tax-nr-wrapper';
        $bei_legal_entity_wout_tax_nr_disabled = '';
    }
    else {
        $bei_legal_entity_wout_tax_nr = 'billzone-eu-invoice-legal-entity-wout-tax-nr-wrapper billzone-eu-invoice-disable';
        $bei_legal_entity_wout_tax_nr_disabled = 'disabled';
    }
    
    $bei_has_hu_tax_nr = (strtoupper ($bei_order_data['billing']['country']) == 'HU') ? 'billzone-eu-invoice-has-hu-tax-nr-wrapper billzone-eu-invoice-hide' : 'billzone-eu-invoice-has-hu-tax-nr-wrapper';

    $bei_html .= '                                  <div class="' . $bei_legal_entity_wout_tax_nr . '">';
    $bei_html .= '                                      <div>';
    $bei_html .= '                                          <input id="beiLegalEntityWithoutTaxNumber" data-beifieldname="legalEntityWithoutTaxNumber" data-beitype="header" type="checkbox" ' . $bei_legal_entity_wout_tax_nr_disabled . '>';    
    $bei_html .= '                                      </div>';
    $bei_html .= '                                      <div>';
    $bei_html .= '                                          <label for="beiLegalEntityWithoutTaxNumber">'.__('Legal entity without tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME).'</label>';    
    $bei_html .= '                                      </div>';
    $bei_html .= '                                  </div>';
    $bei_html .= '                                  <div class="' . $bei_has_hu_tax_nr . '">';
    $bei_html .= '                                      <div>';
    $bei_html .= '                                          <input id="beiHasHungarianTaxNumber" data-beifieldname="hasHungarianTaxNumber" data-beitype="header" type="checkbox">';
    $bei_html .= '                                      </div>';
    $bei_html .= '                                      <div>';
    $bei_html .= '                                          <label for="beiHasHungarianTaxNumber">'.__('Has Hungarian tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME).'</label>';
    $bei_html .= '                                      </div>';
    $bei_html .= '                                  </div>';
    $bei_html .= '                              </div>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                  </table>';
    $bei_html .= '              </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#12: function billzone_eu_invoice_add_popup_inside_elements > PAYMENT ADDRESS OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Payment address -------------------------------------------------------------------------------------------

    // -- START -- Payment detail --------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#13: function billzone_eu_invoice_add_popup_inside_elements > PAYMENT DETAIL IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '              <div>';
    $bei_html .= '                  <table class="billzone-eu-invoice-payment-info">';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>' . __('Payment method', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td>';

    $bei_html .= '                              <select data-beifieldname="payment_method" data-beitype="header" class="billzone-eu-invoice-payment-method">';

    // Show only enabled payment methode
    foreach ($bei_order_data['all_payment_method_type'] as $bei_payment_method_value) {

        if ($bei_payment_method_value['enabled'] == 1) {
            $bei_selected = '';
            if (strtoupper(trim($bei_payment_method_value['id'])) == strtoupper(trim($bei_order_data['payment_method']))) {
                $bei_selected = 'selected';
            }
            $bei_html .= '                                <option value="' . strtoupper(trim($bei_payment_method_value['id'])) . '" ' . $bei_selected . '>' . $bei_payment_method_value['method_title'] . '</option>';
        }

    }

    $bei_html .= '                              </select>';

    $bei_html .= '                          </td>';
    $bei_html .= '                          <td><strong class="billzone-eu-invoice-payment-method-value">' . strtoupper($bei_order_data['payment_method']) . '</strong></td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>' . __('Date of invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td colspan=2>';
    $bei_html .= '                              <input data-beifieldname="invoice_date" data-beitype="header" class="billzone-eu-invoice-current-date" type="date" value="' . gmdate("Y-m-d", strtotime(date("Y-m-d"))) . '" disabled>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('Date of delivery', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td colspan=2>';
    $bei_html .= '                              <input data-beifieldname="delivery_date" data-beitype="header" type="date" value="' . gmdate("Y-m-d", strtotime(date("Y-m-d")) + get_option($bei_options['billzone_eu_invoice_date_of_delivery_plus_days']) * 24 * 60 * 60) . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message" data-validationmessagefor="delivery_date">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                              <span>' . __('Invalid date! (correct format: YYYY-MM-DD)', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr class="">';
    $bei_html .= '                          <td>' . __('Payment due', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':<sup>*</sup></td>';
    $bei_html .= '                          <td colspan=2>';
    $bei_html .= '                              <input data-beifieldname="payment_due" data-beitype="header" type="date" value="' . gmdate("Y-m-d", strtotime(date("Y-m-d")) + get_option($bei_options['billzone_eu_invoice_payment_due_plus_days']) * 24 * 60 * 60) . '">';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td colspan="3" class="billzone-eu-invoice-validation-message" data-validationmessagefor="payment_due">';
    $bei_html .= '                              <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                              <span>' . __('Invalid date! (correct format: YYYY-MM-DD)', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                          </td>';
    $bei_html .= '                      </tr>';
    $bei_html .= '                  </table>';
    $bei_html .= '              </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#14: function billzone_eu_invoice_add_popup_inside_elements > PAYMENT DETAIL OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Payment detail --------------------------------------------------------------------------------------------

    $bei_html .= '          </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#15: function billzone_eu_invoice_add_popup_inside_elements > BILLING INFO OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Billing info --------------------------------------------------------------------------------------------

    // -- START -- Invoice comment ------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#16: function billzone_eu_invoice_add_popup_inside_elements > INVOICE COMMENT IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '          <hr>';
    $bei_html .= '          <div class="billzone-eu-invoice-invoice-comment-wrapper">';
    $bei_html .= '              <div>';
    $bei_html .= '                  <table class="billzone-eu-invoice-invoice-comment-table">';
    $bei_html .= '                      <tr>';
    $bei_html .= '                          <td>';
    $bei_html .= '                              <label for="beiNotes">' . __('Invoice&nbsp;comment', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</label>';
    $bei_html .= '                              <div class="billzone-eu-invoice-validation-message">';
    $bei_html .= '                              </div>';
    $bei_html .= '                          </td>';
    $bei_html .= '                          <td class="billzone-eu-invoice-notes-wrapper">';
    $bei_html .= '                              <input id="beiNotes" name="beiNotes" data-beifieldname="notes" data-beitype="header" type="text" value="' . htmlspecialchars(trim(get_option($bei_options['billzone_eu_invoice_invoice_notes'])), ENT_QUOTES) . '">';
    $bei_html .= '                              <div class="billzone-eu-invoice-validation-message">';
    $bei_html .= '                              </div>';
    $bei_html .= '                          </td>';

    // In case of domestic currency billing show exchange rate input box
    if (array_key_exists($bei_invoice_prefix, $bei_order_data['all_invoice_prefixes_and_associated_currency'])) {

        if (strtoupper($bei_order_data['invoice_issuer_data']['CountryDefaultCurrency']) != strtoupper($bei_order_data['all_invoice_prefixes_and_associated_currency'][$bei_invoice_prefix])) {

            if (empty($bei_order_data['current_exchange_rate']['EXCHANGERATE'])) {
                $bei_exch_rate = "";
            } else {
                $bei_exch_rate = bei_b_round(bei_number($bei_order_data['current_exchange_rate']['EXCHANGERATE']));
            }

            $bei_html .= '                 <td>';
            $bei_html .= '                     <label for="beiExchangeRate" class="billzone-eu-invoice-bold">' . $bei_order_data['current_exchange_rate']['UNIT'] . '&nbsp;' . $bei_order_data['current_exchange_rate']['CURR'] . '&nbsp;=&nbsp;</label>';
            $bei_html .= '                     <div class="billzone-eu-invoice-validation-message">';
            $bei_html .= '                     </div>';
            $bei_html .= '                 </td>';
            $bei_html .= '                 <td class="billzone-eu-invoice-exchange-rate-input-wrapper">';
            $bei_html .= '                     <span><input id="beiExchangeRate" name="beiExchangeRate" data-beifieldname="exchange_rate" data-beitype="header" class="billzone-eu-invoice-exchange-rate-input" type="text" value="' . $bei_exch_rate . '"></span>';
            $bei_html .= '                     <div class="billzone-eu-invoice-validation-message billzone-eu-invoice-text-center" data-validationmessagefor="exchange_rate">';
            $bei_html .= '                         <span>' . __('The field cannot be empty!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
            $bei_html .= '                         <span>' . __('Only numbers are allowed!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
            $bei_html .= '                      </div>';
            $bei_html .= '                 </td>';
            $bei_html .= '                 <td class="billzone-eu-invoice-bold">';
            $bei_html .= '                      ' . $bei_order_data['invoice_issuer_data']['CountryDefaultCurrency'] . '<sup>*</sup>';
            $bei_html .= '                      <div class="billzone-eu-invoice-validation-message">';
            $bei_html .= '                      </div>';
            $bei_html .= '                 </td>';
        }
    }

    $bei_html .= '                      </tr>';
    $bei_html .= '                  </table>';
    $bei_html .= '              </div>';

    $bei_html .= '              <div class="billzone-eu-invoice-invoice-prefix-wrapper">';
    $bei_html .= '                  <span class="billzone-eu-invoice-text-bold">' . __('Invoice sequence prefix', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</span>';
    $bei_html .= '                  <span class="billzone-eu-invoice-invoice-prefix">' . $bei_invoice_prefix . '</span>';
    $bei_html .= '              </div>';

    $bei_html .= '              <div class="billzone-eu-invoice-invoice-currency-wrapper">';
    $bei_html .= '                  <span class="billzone-eu-invoice-text-bold">' . __('Invoice sequence currency', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</span>';
    if (array_key_exists($bei_invoice_prefix, $bei_order_data['all_invoice_prefixes_and_associated_currency'])) {
        $bei_invoice_prefix_txt = $bei_order_data['all_invoice_prefixes_and_associated_currency'][$bei_invoice_prefix];
    } else {
        $bei_invoice_prefix_txt = '---';
    }
    $bei_html .= '                  <span class="billzone-eu-invoice-invoice-currency">' . $bei_invoice_prefix_txt . '</span>';
    $bei_html .= '              </div>';

    $bei_html .= '              <div class="billzone-eu-invoice-sandbox-email-wrapper">';
    if (strtoupper(get_option($bei_options['billzone_eu_invoice_is_sandbox'])) == 'YES') {
        $bei_html .= '              <span class="billzone-eu-invoice-sandbox-icon">';
        $bei_html .= '                  <span>' . __('Sandbox invoicing is ON', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $bei_html .= '                  <img src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-sandbox.svg" alt="Email send icon">';
        $bei_html .= '              </span>';
    }

    if (strtoupper(get_option($bei_options['billzone_eu_invoice_send_email'])) == 'YES') {
        $bei_html .= '              <span class="billzone-eu-invoice-email-send-icon">';
        $bei_html .= '                  <span>' . __('Send invoice notification email to customer is ON', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $bei_html .= '                  <img src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-send-email-yes.svg" alt="Email send icon">';
        $bei_html .= '              </span>';
    } else {
        $bei_html .= '              <span class="billzone-eu-invoice-email-send-icon">';
        $bei_html .= '                  <span>' . __('Send invoice notification email to customer is OFF', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $bei_html .= '                  <img src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-send-email-no.svg" alt="Email send icon">';
        $bei_html .= '              </span>';
    }

    $bei_html .= '                  <span class="billzone-eu-invoice-countrycode-unedifact">';
    $bei_html .= '                      <span>' . __('Country selected for UN/EDIFACT tax rate codes', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '                        ' . $bei_order_data['UN_EDIFACT_country_code'];
    $bei_html .= '                      <b>%</b>';
    $bei_html .= '                  </span>';
    $bei_html .= '              </div>';

    $bei_html .= '          </div>';

    // -- START -- Innvoice sequence currency and order currency mismatch warning -------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#17: function billzone_eu_invoice_add_popup_inside_elements > CURRENCY MISMATCH WARNING IN | Order ID: ' . $bei_order_data['id']);
    if ($bei_order_data['currency'] != $bei_invoice_prefix_txt) {
        $bei_html .= '      <hr>';
        $bei_html .= '      <div class="billzone-eu-invoice-currency-mismatch-warning">';
        $bei_html .= '          <span class="dashicons dashicons-warning"></span>' . __('Attention! The invoice sequence currency is different from the ordering currency. The invoice items are issued unconverted in the currency of the invoice sequence', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (' . $bei_invoice_prefix_txt . ').';
        $bei_html .= '      </div>';
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#18: function billzone_eu_invoice_add_popup_inside_elements > CURRENCY MISMATCH WARNING OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Innvoice sequence currency and order currency mismatch warning -------------------------------------------

    $bei_html .= '          <hr>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#19: function billzone_eu_invoice_add_popup_inside_elements > INVOICE COMMENT  OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Invoice comment ------------------------------------------------------------------------------------------

    // -- START -- Refund error warning -------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#20: function billzone_eu_invoice_add_popup_inside_elements > REFUND ERROR WARNING IN | Order ID: ' . $bei_order_data['id']);
    if ($bei_order_data['there_is_a_refund'] == 'yes' && $bei_order_data['there_is_wrong_tax_in_refund'] == 'yes') {
        $bei_html .= '      <div class="billzone-eu-invoice-refund-items-error-message">';
        $bei_html .= __('There is an error between the refund items! Check the quantity (must be non-zero) or VAT key (there is no VAT key or multiple VAT keys have been set) for the refund item in question.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        $bei_html .= '      </div>';
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#21: function billzone_eu_invoice_add_popup_inside_elements > REFUND ERROR WARNING OUT | Order ID: ' . $bei_order_data['id']);
    // -- END -- Refund error warning ---------------------------------------------------------------------------------------

    // -- START -- Item table -----------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#22: function billzone_eu_invoice_add_popup_inside_elements > ITEM TABLE IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '          <table class="billzone-eu-invoice-detail-table">';

    // -- START -- Item table header ----------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#23: function billzone_eu_invoice_add_popup_inside_elements > ITEM TABLE HEADER IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '              <tr>';
    $bei_html .= '                  <th colspan=2>' . __('Description', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</th>';
    $bei_html .= '                  <th>' . __('Quantity', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</th>';
    $bei_html .= '                  <th>' . __('Unit of measure', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</th>';
    $bei_html .= '                  <th>' . __('Unit price', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<br>' . $bei_order_data['currency'] . '</th>';
    $bei_html .= '                  <th>' . __('Amount', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<br>' . $bei_order_data['currency'] . '</th>';
    $bei_html .= '                  <th>';
    $bei_html .= __('VAT', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '(%)<br>';
    $bei_html .= __('Code', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $bei_html .= '                  </th>';
    $bei_html .= '                  <th>' . __('VAT amount', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<br>' . $bei_order_data['currency'] . '</th>';
    $bei_html .= '                  <th>' . __('Gross amount', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<br>' . $bei_order_data['currency'] . '</th>';
    $bei_html .= '              </tr>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#24: function billzone_eu_invoice_add_popup_inside_elements > ITEM TABLE HEADER OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Item table header ----------------------------------------------------------------------------------------

    // -- START -- Item detail ----------------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#25: function billzone_eu_invoice_add_popup_inside_elements > ITEM DETAIL IN | Order ID: ' . $bei_order_data['id']);
    $bei_row_counter = 1;
    if (!empty($bei_order_data['line_items'])) {
        foreach ($bei_order_data['line_items'] as $bei_item_key => $bei_item_data) {

            $bei_special_tax_class = ($bei_item_data['tax_percentage'] != $bei_order_data['default_tax_rate']) ? 'billzone-eu-invoice-special-tax' : '';

            $bei_html .= billzone_eu_invoice_one_item_row($bei_special_tax_class, $bei_row_counter,
                $bei_item_data['image'],
                $bei_item_data['name'],
                $bei_item_data['all_meta_data_concatenated'],
                $bei_item_data['quantity'],
                bei_b_round($bei_item_data['quantity']),
                $bei_item_data['unit_of_measure'], $bei_item_data['unit_of_measure_text'],
                ($bei_item_data['subtotal'] / $bei_item_data['quantity']), bei_b_round($bei_item_data['subtotal'] / $bei_item_data['quantity']),
                $bei_item_data['subtotal'], bei_b_round($bei_item_data['subtotal']),
                $bei_item_data['tax_percentage'], bei_z_round($bei_item_data['tax_percentage']),
                $bei_order_data['UN_EDIFACT_codes'],
                $bei_item_data['subtotal_tax'], bei_b_round($bei_item_data['subtotal_tax']),
                ($bei_item_data['subtotal_tax'] + $bei_item_data['subtotal']), bei_b_round($bei_item_data['subtotal_tax'] + $bei_item_data['subtotal']));

            $bei_row_counter++;

        }
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#26: function billzone_eu_invoice_add_popup_inside_elements > ITEM DETAIL OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Item detail ----------------------------------------------------------------------------------------------

    //-- START -- Show shipping if there is any exept "local_pickup" --------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#27: function billzone_eu_invoice_add_popup_inside_elements > SHIPPING IN | Order ID: ' . $bei_order_data['id']);

    if ($bei_order_data['has_shipping_lines']) {
        foreach ($bei_order_data['shipping_lines'] as $bei_shipping_item_key => $bei_shipping_item_data) {

            $bei_special_tax_class = ($bei_shipping_item_data['tax_percentage'] != $bei_order_data['default_tax_rate']) ? 'billzone-eu-invoice-special-tax' : '';
    
            $bei_html .= billzone_eu_invoice_one_item_row($bei_special_tax_class, $bei_row_counter,
                ' <img class="billzone-eu-invoice-item-icon" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-shipping.svg" alt="' . __('Shipping icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">',
                __('Shipping', BILLZONE_EU_INVOICE_PLUGIN_NAME), $bei_shipping_item_data['method_title'],
                '1', '1' . wc_get_price_decimal_separator() . '00',
                $bei_shipping_item_data['unit_of_measure'], $bei_shipping_item_data['unit_of_measure_text'],
                $bei_shipping_item_data['total'], bei_b_round($bei_shipping_item_data['total']),
                $bei_shipping_item_data['total'], bei_b_round($bei_shipping_item_data['total']),
                $bei_shipping_item_data['tax_percentage'], bei_z_round($bei_shipping_item_data['tax_percentage']),
                $bei_order_data['UN_EDIFACT_codes'],
                $bei_shipping_item_data['total_tax'], bei_b_round($bei_shipping_item_data['total_tax']),
                ($bei_shipping_item_data['total'] + $bei_shipping_item_data['total_tax']), bei_b_round(($bei_shipping_item_data['total'] + $bei_shipping_item_data['total_tax'])));
    
            $bei_row_counter++;
    
        }
    }
    
    billzone_eu_invoice_logme(__FILE__, 'OK', '#28: function billzone_eu_invoice_add_popup_inside_elements > SHIPPING OUT | Order ID: ' . $bei_order_data['id']);
    //-- END -- Show shipping if there is any than "local_pickup" -----------------------------------------------------------

    //-- START -- Show coupon discount only if we have coupons --------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#29: function billzone_eu_invoice_add_popup_inside_elements > COUPON IN | Order ID: ' . $bei_order_data['id']);
    if (!empty($bei_order_data['discount_per_tax'])) {

        foreach ($bei_order_data['discount_per_tax'] as $bei_discount_key => $bei_discount_data) {

            $bei_special_tax_class = ($bei_discount_data['tax_percentage'] != $bei_order_data['default_tax_rate']) ? 'billzone-eu-invoice-special-tax' : '';

            $bei_html .= billzone_eu_invoice_one_item_row($bei_special_tax_class, $bei_row_counter,
                ' <img class="billzone-eu-invoice-item-icon" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-discount.svg" alt="' . __('Discount icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">',
                __('Coupon discount (Tax: ', BILLZONE_EU_INVOICE_PLUGIN_NAME) . bei_z_round($bei_discount_data['tax_percentage']) . '%)',
                __('Coupon code: ', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' ' . $bei_order_data['all_coupon_name'],
                '1', '1' . wc_get_price_decimal_separator() . '00',
                $bei_discount_data['unit_of_measure'], $bei_discount_data['unit_of_measure_text'],
                ($bei_discount_data['total_discount_per_tax'] * (-1)), bei_b_round($bei_discount_data['total_discount_per_tax'] * (-1)),
                ($bei_discount_data['total_discount_per_tax'] * (-1)), bei_b_round($bei_discount_data['total_discount_per_tax'] * (-1)),
                $bei_discount_data['tax_percentage'], bei_z_round($bei_discount_data['tax_percentage']),
                $bei_order_data['UN_EDIFACT_codes'],
                ($bei_discount_data['total_discount_tax_per_tax'] * (-1)), bei_b_round($bei_discount_data['total_discount_tax_per_tax'] * (-1)),
                (($bei_discount_data['total_discount_tax_per_tax'] + $bei_discount_data['total_discount_per_tax']) * (-1)), bei_b_round(($bei_discount_data['total_discount_tax_per_tax'] + $bei_discount_data['total_discount_per_tax']) * (-1)));

            $bei_row_counter++;
        }
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#30: function billzone_eu_invoice_add_popup_inside_elements > COUPON OUT | Order ID: ' . $bei_order_data['id']);
    //-- END -- Show coupon discount only if we have coupons ----------------------------------------------------------------

    //-- START -- Show other fees, if any -----------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#31: function billzone_eu_invoice_add_popup_inside_elements > OTHER FEES IN | Order ID: ' . $bei_order_data['id']);
    if (!empty($bei_order_data['fee_lines'])) {
        foreach ($bei_order_data['fee_lines'] as $bei_fee_key => $bei_fee_data) {

            $bei_special_tax_class = ($bei_fee_data['tax_percentage'] != $bei_order_data['default_tax_rate']) ? 'billzone-eu-invoice-special-tax' : '';
            $bei_html .= billzone_eu_invoice_one_item_row($bei_special_tax_class, $bei_row_counter,
                ' <img class="billzone-eu-invoice-item-icon" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-other-fee.svg" alt="' . __('Fee icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">',
                $bei_fee_data['name'],
                '',
                '1', '1' . wc_get_price_decimal_separator() . '00',
                $bei_fee_data['unit_of_measure'], $bei_fee_data['unit_of_measure_text'],
                $bei_fee_data['total'], bei_b_round($bei_fee_data['total']),
                $bei_fee_data['total'], bei_b_round($bei_fee_data['total']),
                $bei_fee_data['tax_percentage'], bei_z_round($bei_fee_data['tax_percentage']),
                $bei_order_data['UN_EDIFACT_codes'],
                $bei_fee_data['total_tax'], bei_b_round($bei_fee_data['total_tax']),
                ($bei_fee_data['total_tax'] + $bei_fee_data['total']), bei_b_round(($bei_fee_data['total_tax'] + $bei_fee_data['total'])));

            $bei_row_counter++;
        }
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#32: function billzone_eu_invoice_add_popup_inside_elements > OTHER FEES OUT | Order ID: ' . $bei_order_data['id']);
    //-- END ---- Show other fees, if any -----------------------------------------------------------------------------------

    //-- START -- Show refunds, if any --------------------------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#33: function billzone_eu_invoice_add_popup_inside_elements > REFUNDS IN | Order ID: ' . $bei_order_data['id']);
    if (!empty($bei_order_data['all_refunds'])) {
        foreach ($bei_order_data['all_refunds'] as $bei_refund_key => $bei_refund_data) {

            if (strtoupper($bei_refund_data['wrong_tax']) == 'YES') {
                $bei_special_class = 'billzone-eu-invoice-refund-error';
                $bei_refund_image = ' <span class="billzone-eu-invoice-item-with-error">';
                $bei_refund_image .= '  <img class="billzone-eu-invoice-item-icon billzone-eu-invoice-item-error-color" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-refund-error.svg" alt="' . __('Refund icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">';
                $bei_refund_image .= '  <span class="billzone-eu-invoice-item-error-message">' . __('Item line refund error! Check the number of pieces (cannot be zero) or the VAT key (there is no VAT key or multiple VAT keys have been set).', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
                $bei_refund_image .= '</span>';
            } elseif ($bei_refund_data['tax_percentage'] != $bei_order_data['default_tax_rate']) {
                $bei_special_class = 'billzone-eu-invoice-refund-special-tax';
                $bei_refund_image = ' <img class="billzone-eu-invoice-item-icon" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-refund.svg" alt="' . __('Refund icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">';
            } else {
                $bei_special_class = 'billzone-eu-invoice-refund';
                $bei_refund_image = ' <img class="billzone-eu-invoice-item-icon" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-refund.svg" alt="' . __('Refund icon', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '">';
            }

            $bei_html .= billzone_eu_invoice_one_item_row($bei_special_class, $bei_row_counter,
                $bei_refund_image,
                '<i>' . __('Refund for: ', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</i>' . $bei_refund_data['name'],
                $bei_refund_data['reason'],
                $bei_refund_data['quantity'],
                bei_b_round($bei_refund_data['quantity']),
                $bei_refund_data['unit_of_measure'], $bei_refund_data['unit_of_measure_text'],
                $bei_refund_data['net_unit_price'], bei_b_round($bei_refund_data['net_unit_price']),
                $bei_refund_data['total'], bei_b_round($bei_refund_data['total']),
                $bei_refund_data['tax_percentage'], bei_z_round($bei_refund_data['tax_percentage']),
                $bei_order_data['UN_EDIFACT_codes'],
                $bei_refund_data['total_tax'], bei_b_round($bei_refund_data['total_tax']),
                ($bei_refund_data['total_tax'] + $bei_refund_data['total']), bei_b_round(($bei_refund_data['total_tax'] + $bei_refund_data['total'])));

            $bei_row_counter++;

        }
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#34: function billzone_eu_invoice_add_popup_inside_elements > REFUNDS OUT | Order ID: ' . $bei_order_data['id']);
    //-- END -- Show refunds, if any ----------------------------------------------------------------------------------------

    //-- START -- Summing all items (product+shipping+discount) -------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#35: function billzone_eu_invoice_add_popup_inside_elements > SUMMING ALL ITEMS IN | Order ID: ' . $bei_order_data['id']);
    if ($bei_order_data['there_is_a_refund'] == 'yes') {
        $bei_final_total_tax = $bei_order_data['total_tax_after_refund'];
        $bei_final_total = $bei_order_data['total_after_refund'];
    } else {
        $bei_final_total_tax = $bei_order_data['total_tax'];
        $bei_final_total = $bei_order_data['total'];
    }

    $bei_html .= '              <tr class="billzone-eu-invoice-detail-table-summ">';
    $bei_html .= '                  <td></td>';
    $bei_html .= '                  <td></td>';
    $bei_html .= '                  <td></td>';
    $bei_html .= '                  <td></td>';
    $bei_html .= '                  <td>&sum;:</td>';
    $bei_html .= '                  <td>' . bei_w_round($bei_final_total - $bei_final_total_tax) . '</td>';
    $bei_html .= '                  <td></td>';
    $bei_html .= '                  <td>';
    $bei_html .= '                      <input data-beifieldname="total_tax" data-beitype="summing" type="hidden" value="' . $bei_final_total_tax . '">';
    $bei_html .= '                      <span class="billzone-eu-invoice-final-sum">' . bei_w_round($bei_final_total_tax) . '</span>';
    $bei_html .= '                  </td>';
    $bei_html .= '                  <td>';
    $bei_html .= '                      <input data-beifieldname="total_gross_price" data-beitype="summing" type="hidden" value="' . $bei_final_total . '">';
    $bei_html .= '                      <span class="billzone-eu-invoice-final-sum">' . bei_w_round($bei_final_total) . '</span>';
    $bei_html .= '                  </td>';
    $bei_html .= '              </tr>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#36: function billzone_eu_invoice_add_popup_inside_elements > SUMMING ALL ITEMS OUT | Order ID: ' . $bei_order_data['id']);
    //-- END ---- Summing all items (product+shipping+discount) -------------------------------------------------------------

    $bei_html .= '          </table>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#37: function billzone_eu_invoice_add_popup_inside_elements > ITEM TABLE OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Item table -----------------------------------------------------------------------------------------------

    $bei_html .= '      </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#38: function billzone_eu_invoice_add_popup_inside_elements > CONTAINER BODY OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Container body -------------------------------------------------------------------------------------------

    // -- START -- Container footer (create invoice button) -----------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#39: function billzone_eu_invoice_add_popup_inside_elements > CONTAINER FOOTER IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '      <div class="billzone-eu-invoice-detail-container-send">';
    $bei_html .= '          <div class="billzone-eu-invoice-flex-row">';
    $bei_html .= '              <div class="billzone-eu-invoice-flex-col">';
    $bei_html .= '                  <div class="billzone-eu-invoice-margin-10">';
    $bei_html .= '                      <input id="billzone_eu_invoice_enable_vat_edit" class="billzone-eu-invoice-align-vertical" type="checkbox">';
    $bei_html .= '                      <label for="billzone_eu_invoice_enable_vat_edit">' . __('Enable VAT code edit', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $bei_html .= '                  </div>';
    $bei_html .= '              </div>';
    $bei_html .= '              <div class="billzone-eu-invoice-flex-col">';
    $bei_html .= '                  <div class="billzone-eu-invoice-margin-10">';
    $bei_html .= '                      <input id="billzone_eu_invoice_same_vat" class="billzone-eu-invoice-align-vertical billzone_eu_invoice_same_vat" type="checkbox" disabled>';
    $bei_html .= '                      <label for="billzone_eu_invoice_same_vat">' . __('Same VAT code for all items', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $bei_html .= '                      <select id="billzone_eu_invoice_all_item_UN_EDIFACT_codes" disabled="disabled">';
    $bei_html .= billzone_eu_invoice_generate_UNEDIFACT_items_for_select($bei_order_data['UN_EDIFACT_codes']);
    $bei_html .= '                      </select>';
    $bei_html .= '                      <strong class="billzone-eu-invoice-val-container"><span class="billzone-eu-invoice-item-UN-EDIFACT-value">&nbsp;' . reset($bei_order_data['UN_EDIFACT_codes']) . '</span>%<sup>**</sup></strong>';
    $bei_html .= '                  </div>';
    $bei_html .= '              </div>';
    $bei_html .= '              <div class="billzone-eu-invoice-flex-col">';
    if ($bei_order_data['there_is_wrong_tax_in_refund'] == 'yes') {
        $bei_html .= '                  <div class="button billzone-eu-invoice-button-refund-error"><span class="dashicons dashicons-no"></span>' . __('Refund error', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</div>';
    } else {
        $bei_html .= '                  <div id="billzoneEuInvoiceCreateInvoice" class="button button-primary"><span class="dashicons dashicons-yes"></span>' . __('Create invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</div>';
    }
    $bei_html .= '                  <div class="button button-secondary billzone-eu-invoice-cancel"><span class="dashicons dashicons-no-alt"></span>' . __('Cancel', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</div>';
    $bei_html .= '              </div>';
    $bei_html .= '          </div>';
    $bei_html .= '          <div class="billzone-eu-invoice-footer-mentions-flex">';
    $bei_html .= '              <div class="billzone-eu-invoice-footer-mentions-col">';
    $bei_html .= '                  *<small>' . __('Mandatory fields', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</small>';
    $bei_html .= '              </div>';
    $bei_html .= '              <div class="billzone-eu-invoice-footer-mentions-col">';
    $bei_html .= '                  **<small>' . __('For available UN/EDIFACT tax rate codes, please check MASTER DATA / TAX RATE CODES menu in your billzone.eu account, or read <a href="https://www.billzone.eu/blog/2016/10/10/milyen-afa-kulcsokat-hasznal-a-rendszer/" target="_blank">this</a> article.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</small>';
    $bei_html .= '              </div>';
    $bei_html .= '          </div>';
    $bei_html .= '      </div>';
    $bei_html .= '      <div class="billzone-eu-invoice-warning-VAT-code-warning-wrapper">';
    $bei_html .= '          <div class="billzone-eu-invoice-warning-VAT-code-warning-box">';
    $bei_html .= '              <h3>' . __('Warning!', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h3>';
    $bei_html .= '              <p>' . __('Changing the VAT codes is only recommended if the system cannot set the UN/EDIFACT VAT rates correctly (typically for 0% VAT rates), as changing the VAT code does not mean recalculating the VAT value (the system will show the changed values highlighted in red). Use the WooCommerce settings interface to set the correct VAT values. The plugin will always send the net values to the billzone.eu invoicer, so the issued invoice will be correct and the invoices will have the values corresponding to the VAT codes set here.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</p>';
    $bei_html .= '              <div id="billzoneEuInvoiceVATCodeChangeConsentButton" class="button button-primary"><span class="dashicons dashicons-yes"></span>' . __('OK', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</div>';
    $bei_html .= '          </div>';
    $bei_html .= '      </div>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#40: function billzone_eu_invoice_add_popup_inside_elements > CONTAINER FOOTER OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Container footer (create invoice button) -----------------------------------------------------------------

    billzone_eu_invoice_logme(__FILE__, 'OK', '#41: function billzone_eu_invoice_add_popup_inside_elements > END | Order ID: ' . $bei_order_data['id']);
    return $bei_html;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to generate order detail popup panel wrapper
 * @param string $bei_order_data -> input var containing all order data
 * @access public
 */
function billzone_eu_invoice_add_popup_panel_wrapper($bei_order_data)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_add_popup_panel_wrapper > START | Order ID: ' . $bei_order_data['id']);

    // -- START -- Container background ------------------------------------------------------------------------------------

    echo '<div class="billzone-eu-invoice-detail-wrapper billzone-eu-invoice-hide-detail">';
    // -- START -- Container -----------------------------------------------------------------------------------------------
    echo '  <div class="billzone-eu-invoice-detail-container billzone-eu-invoice-detail-container-initial-position">';

    // calling beiSetOrderData() javascript function wich will populate the popup container with order data, the data will be passed to Bei_Order_Data javascript global variable
    $bei_order_data_escaped = billzone_eu_invoice_add_popup_inside_elements($bei_order_data);
    $bei_order_data_escaped = bei_esc_html($bei_order_data_escaped);
    echo '      <script class="billzone-eu-invoice-set-order-data-script">beiSetOrderData(' . '\'' . $bei_order_data_escaped . '\'' . ');</script>';

    echo '  </div>';
    // -- END ---- Container -----------------------------------------------------------------------------------------------
    echo '</div>';
    // -- END ---- Container background ------------------------------------------------------------------------------------

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_add_popup_panel_wrapper > END | Order ID: ' . $bei_order_data['id']);
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to add meta box content
 * @param string $bei_order_data -> input var containing all order data
 * @access public
 */
function billzone_eu_invoice_add_meta_box_content($bei_order_data)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_add_meta_box_content > START | Order ID: ' . $bei_order_data['id']);

    global $bei_options;

    $bei_html = '';

    // -- START -- Add meta box to order page sidebar ----------------------------------------------------------------------
    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_add_meta_box_content > META BOX IN | Order ID: ' . $bei_order_data['id']);
    $bei_html .= '<table class="billzone-eu-invoice-invoice-summary-table">';

    // Show warning for wrong refund
    billzone_eu_invoice_logme(__FILE__, 'OK', '#3: function billzone_eu_invoice_add_meta_box_content > REFUND WARNING IN | Order ID: ' . $bei_order_data['id']);
    if ($bei_order_data['there_is_wrong_tax_in_refund'] == 'yes') {
        $bei_html .= '<tr>';
        $bei_html .= ' <td colspan=2 class="billzone-eu-invoice-button-refund-error-text"><span class="dashicons dashicons-warning"></span>' . __('Refund error', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</td>';
        $bei_html .= '</tr>';
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#4: function billzone_eu_invoice_add_meta_box_content > REFUND WARNING OUT | Order ID: ' . $bei_order_data['id']);

    // Show warning text for domestic billing
    billzone_eu_invoice_logme(__FILE__, 'OK', '#5: function billzone_eu_invoice_add_meta_box_content > DOMESTIC BILLING IN | Order ID: ' . $bei_order_data['id']);
    $bei_invoice_prefix = sanitize_text_field(trim(get_option($bei_options['billzone_eu_invoice_invoice_number_prefix'])));
    if (array_key_exists($bei_invoice_prefix, $bei_order_data['all_invoice_prefixes_and_associated_currency'])) {
        if (strtoupper($bei_order_data['invoice_issuer_data']['CountryDefaultCurrency']) != strtoupper($bei_order_data['all_invoice_prefixes_and_associated_currency'][$bei_invoice_prefix])) {

            $bei_html .= '<tr>';
            $bei_html .= ' <td colspan=2 class="billzone-eu-invoice-text-center">' . __('Domestic currency billing', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<hr></td>';
            $bei_html .= '</tr>';

        }
    }

    $bei_html .= '  <tr>';
    $bei_html .= '     <td>' . __('Invoice number', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';

    $bei_html .= '     <td id="beiInvoiceNumberForAlreadyCreatedInvoice">';

    if (empty($bei_order_data['already_created_invoice_number'])) {
        $bei_class_toggler_normal_invoice = 'billzone-eu-invoice-hide';
        $bei_class_toggler_void_invoice = 'billzone-eu-invoice-hide';
        $bei_class_toggler_no_invoice = '';
        $bei_class_toggler_invoice_nr_txt = '';
    } else {

        if (empty($bei_order_data['already_created_invoice_type']) || $bei_order_data['already_created_invoice_type'] == 'normal') {
            $bei_class_toggler_normal_invoice = '';
            $bei_class_toggler_void_invoice = 'billzone-eu-invoice-hide';
            $bei_class_toggler_no_invoice = 'billzone-eu-invoice-hide';
            $bei_class_toggler_invoice_nr_txt = '';
        } else {
            $bei_class_toggler_normal_invoice = 'billzone-eu-invoice-hide';
            $bei_class_toggler_void_invoice = '';
            $bei_class_toggler_no_invoice = 'billzone-eu-invoice-hide';
            $bei_class_toggler_invoice_nr_txt = '';
        }

    }

    $bei_html .= '      <span class="billzone-eu-invoice-invoice-type-wrapper">';

    $bei_html .= '          <span class="billzone-eu-invoice-invoice-type-help-text billzone-eu-invoice-invoice-type-normal-invoice ' . $bei_class_toggler_normal_invoice . '">' . __('Normal invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '          <span class="billzone-eu-invoice-invoice-type-help-text billzone-eu-invoice-invoice-type-void-invoice ' . $bei_class_toggler_void_invoice . '">' . __('Void invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '          <span class="billzone-eu-invoice-invoice-type-help-text billzone-eu-invoice-invoice-type-no-invoice ' . $bei_class_toggler_no_invoice . '">' . __('no invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';

    $bei_html .= '          <img class="billzone-eu-invoice-invoice-type-img2 billzone-eu-invoice-invoice-type-img-normal ' . $bei_class_toggler_normal_invoice . '" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-invoice-type-normal.svg">';
    $bei_html .= '          <img class="billzone-eu-invoice-invoice-type-img2 billzone-eu-invoice-invoice-type-img-void ' . $bei_class_toggler_void_invoice . '" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-invoice-type-void.svg">';
    $bei_html .= '          <img class="billzone-eu-invoice-invoice-type-img2 billzone-eu-invoice-invoice-type-img-no-invoice ' . $bei_class_toggler_no_invoice . '" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-invoice-type-no-invoice.svg">';

    $bei_html .= '          <span class="billzone-eu-invoice-no-invoice-txt ' . $bei_class_toggler_no_invoice . '">' . __('no invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $bei_html .= '          <span class="billzone-eu-invoice-invoice-nr-txt ' . $bei_class_toggler_invoice_nr_txt . '">' . $bei_order_data['already_created_invoice_number'] . '</span>';

    $bei_html .= '      </span>';

    $bei_html .= '     </td>';

    $bei_html .= '  </tr>';

    if (!empty($bei_order_data['already_created_invoice_url'])) {
        $bei_html .= '<tr class="billzone-eu-invoice-invoice-download-link">';
        $bei_html .= '  <td>' . __('Invoice link', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';
        $bei_html .= '  <td><a id="beiInvoiceUrlForAlreadyCreatedInvoice" href="' . $bei_order_data['already_created_invoice_url'] . '" target="_blank">' . __('DOWNLOAD', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</a></td>';
        $bei_html .= '</tr>';
    } else {
        $bei_html .= '<tr class="billzone-eu-invoice-invoice-download-link billzone-eu-invoice-hide">';
        $bei_html .= '  <td>' . __('Invoice link', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';
        $bei_html .= '  <td><a id="beiInvoiceUrlForAlreadyCreatedInvoice" href="" target="_blank">' . __('DOWNLOAD', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</a></td>';
        $bei_html .= '</tr>';
    }

    $bei_html .= '  <tr><td>' . __('Order number', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>#' . $bei_order_data['id'] . '</td></tr>';
    if (empty($bei_order_data['date_created'])) {
        $bei_html .= '  <tr><td>' . __('Order date', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>---</td></tr>';
    } else {
        $bei_html .= '  <tr><td>' . __('Order date', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . date_format($bei_order_data['date_created'], "Y-m-d H:i") . '</td></tr>';
    }

    $bei_html .= '  <tr><td>' . __('Order status', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_order_data['status_title'] . '</td></tr>';
    $bei_html .= '  <tr><td>' . __('Payment method', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_order_data['payment_method_title'] . '</td></tr>';

    empty($bei_order_data['line_items']) ? $bei_row_nr = 0 : $bei_row_nr = count($bei_order_data['line_items']);
    $bei_html .= '  <tr><td>' . __('Total item row nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_row_nr . '</td></tr>';

    $bei_html .= '  <tr><td colspan=2><hr></td></tr>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#6: function billzone_eu_invoice_add_meta_box_content > DOMESTIC BILLING OUT | Order ID: ' . $bei_order_data['id']);

    // Show coupon codes only if we have coupons
    billzone_eu_invoice_logme(__FILE__, 'OK', '#7: function billzone_eu_invoice_add_meta_box_content > COUPONS IN | Order ID: ' . $bei_order_data['id']);
    if (!empty($bei_order_data['coupon_lines'])) {
        $bei_html .= '<tr>';
        $bei_html .= '  <td>' . __('Coupons', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td>';
        $bei_html .= '  <td>';
        $bei_html .= $bei_order_data['all_coupon_name'];
        $bei_html .= '  </td>';
        $bei_html .= '</tr>';
        $bei_html .= '<tr><td colspan=2><hr></td></tr>';
    }

    $bei_html .= '  <tr><td>' . __('Shipping', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_order_data['all_shipping_name'] . '</td></tr>';
    $bei_html .= '  <tr><td colspan=2><hr></td></tr>';
    $bei_html .= '  <tr><td>' . __('Currency', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_order_data['currency'] . '</td></tr>';
    $bei_html .= '  <tr><td colspan=2><hr></td></tr>';

    $bei_invoice_seq_pref = get_option($bei_options['billzone_eu_invoice_invoice_number_prefix']);
    if (!empty($bei_invoice_seq_pref)) {
        $bei_invoice_seq_pref_txt = $bei_invoice_seq_pref;
        if (array_key_exists(strtoupper($bei_invoice_seq_pref), $bei_order_data['all_invoice_prefixes_and_associated_currency'])) {
            $bei_invoice_seq_cur_txt = $bei_order_data['all_invoice_prefixes_and_associated_currency'][$bei_invoice_seq_pref];
        } else {
            $bei_invoice_seq_cur_txt = '---';
        }
    } else {
        $bei_invoice_seq_pref_txt = '---';
        $bei_invoice_seq_cur_txt = '---';
    }

    $bei_html .= '  <tr><td>' . __('Invoice sequence prefix', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_invoice_seq_pref_txt . '</td></tr>';
    $bei_html .= '  <tr><td>' . __('Invoice sequence currency', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . $bei_invoice_seq_cur_txt . '</td></tr>';
    $bei_html .= '  <tr><td colspan=2><hr></td></tr>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#8: function billzone_eu_invoice_add_meta_box_content > COUPONS OUT | Order ID: ' . $bei_order_data['id']);

    // Show coupon total price only if there is coupons
    billzone_eu_invoice_logme(__FILE__, 'OK', '#9: function billzone_eu_invoice_add_meta_box_content > COUPONS TOTAL PRICE IN | Order ID: ' . $bei_order_data['id']);
    if (!empty($bei_order_data['coupon_lines'])) {
        $bei_html .= '<tr class="billzone-eu-invoice-discount-total"><td>' . __('Gross discount', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . bei_b_round(($bei_order_data['discount_total'] + $bei_order_data['discount_tax']) * (-1)) . ' ' . $bei_order_data['currency'] . '</td></tr>';
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#10: function billzone_eu_invoice_add_meta_box_content > COUPONS TOTAL PRICE OUT | Order ID: ' . $bei_order_data['id']);

    // Show shipping total price if needed
    billzone_eu_invoice_logme(__FILE__, 'OK', '#11: function billzone_eu_invoice_add_meta_box_content > SHOPING IN | Order ID: ' . $bei_order_data['id']);
    if ($bei_order_data['has_shipping_lines'] == 1) {
        $bei_html .= '<tr><td>' . __('Gross shipping amount', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . bei_b_round($bei_order_data['shipping_total'] + $bei_order_data['shipping_tax']) . ' ' . $bei_order_data['currency'] . '</td></tr>';
    }
    billzone_eu_invoice_logme(__FILE__, 'OK', '#12: function billzone_eu_invoice_add_meta_box_content > SHOPING OUT | Order ID: ' . $bei_order_data['id']);

    // Show refund summ if any
    billzone_eu_invoice_logme(__FILE__, 'OK', '#13: function billzone_eu_invoice_add_meta_box_content > REFUND SUMM IN | Order ID: ' . $bei_order_data['id']);
    if ($bei_order_data['there_is_a_refund'] == 'yes') {
        $bei_html .= '  <tr class="billzone-eu-invoice-refund-total"><td>' . __('Refunded', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . bei_b_round($bei_order_data['total_gross_refund']) . ' ' . $bei_order_data['currency'] . '</td></tr>';
        $bei_final_total = $bei_order_data['total_after_refund'];
    } else {
        $bei_final_total = $bei_order_data['total'];
    }

    $bei_html .= '  <tr><td>' . __('Gross total amount', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ':</td><td>' . bei_z_round($bei_final_total) . ' ' . $bei_order_data['currency'] . '</td></tr>';
    $bei_html .= '</table>';
    billzone_eu_invoice_logme(__FILE__, 'OK', '#14: function billzone_eu_invoice_add_meta_box_content > REFUND SUMM OUT | Order ID: ' . $bei_order_data['id']);
    // -- END ---- Add meta box to order page sidebar ----------------------------------------------------------------------

    billzone_eu_invoice_logme(__FILE__, 'OK', '#15: function billzone_eu_invoice_add_meta_box_content > END | Order ID: ' . $bei_order_data['id']);

    return $bei_html;
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Function to add meta box and detail popup
 * @access public
 */
function billzone_eu_invoice_add_meta_box_and_popup_content()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_add_meta_box_and_popup_content > START');

    global $wpdb;
    global $bei_options;

    $bei_security_code_status = get_option($bei_options['billzone_eu_invoice_security_code_status']);
    if (empty($bei_security_code_status)) {
        $bei_security_code_status = __('EMPTY', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    }

    if (bei_check_if_install_is_OK()){
        // if security code OK and instalation OK create invoicing box and popup panel otherwise show an error message
        if (!empty($bei_security_code_status) && ($bei_security_code_status == 61001) && bei_check_if_install_is_OK()) {

            $bei_option_billzone_eu_invoice_invoice_number_prefix_is_empty = empty(trim(get_option($bei_options['billzone_eu_invoice_invoice_number_prefix'])));
            $bei_option_billzone_eu_invoice_company_location_id_is_empty = empty(trim(get_option($bei_options['billzone_eu_invoice_company_location_id'])));
            $bei_option_billzone_eu_invoice_default_pricing_unit_is_empty = empty(trim(get_option($bei_options['billzone_eu_invoice_default_pricing_unit'])));
            $bei_option_billzone_eu_invoice_default_payment_methode_is_empty = empty(trim(get_option($bei_options['billzone_eu_invoice_default_payment_methode'])));
            $bei_option_billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code_is_empty = empty(trim(get_option($bei_options['billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code'])));

            if (!$bei_option_billzone_eu_invoice_invoice_number_prefix_is_empty && !$bei_option_billzone_eu_invoice_company_location_id_is_empty && !$bei_option_billzone_eu_invoice_default_pricing_unit_is_empty && !$bei_option_billzone_eu_invoice_default_pricing_unit_is_empty && !$bei_option_billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code_is_empty) {

                $bei_order = new BEI_ORDER(wc_get_order());
                $bei_order_data = $bei_order->bei_get_order_data();

                billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_add_meta_box_and_popup_content > security code OK | Order ID: ' . $bei_order_data['id']);

                // check if we have already created invoice for this order and set class to show button for creation or only plain text
                if (empty($bei_order_data['already_created_invoice_number'])) {
                    $bei_button_type_css_class = 'billzone-eu-invoice-create-btn button button-primary';
                } else {
                    $bei_button_type_css_class = '';
                }

                // -- START -- Add meta box content -----------------------------------------------------------------------------------
                echo '<div class="billzone-eu-invoice-create-button-wrapper">';
                echo '  <div id="billzone_eu_invoice_Create_Button" class="' . $bei_button_type_css_class . '">';
                echo '      <span class="billzone-eu-invoice-create-invoice-button">' . __('Create invoice', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
                echo '      <span class="billzone-eu-invoice-create-invoice-data-text">' . __('Invoice data', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
                echo '  </div>';
                echo '</div>';
                echo '<div class="billzone-eu-invoice-invoice-summary-wrapper">';
                echo billzone_eu_invoice_add_meta_box_content($bei_order_data);
                echo '</div>';
                // -- END ---- Add meta box content -----------------------------------------------------------------------------------

                $bei_order_id = $bei_order_data['id'];
                $bei_check_if_invoice_exist = $wpdb->get_results(
                    "
                    SELECT invoice_number, invoice_type
                    FROM " . BILLZONE_EU_INVOICE_TABLE_NAME .
                    " WHERE order_id =  $bei_order_id
                    "
                );

                if (empty($bei_check_if_invoice_exist)) {
                    // -- START -- Add order popup content ---------------------------------------------------------------------------------
                    billzone_eu_invoice_add_popup_panel_wrapper($bei_order_data);
                    // -- END ---- Add order popup content ---------------------------------------------------------------------------------
                }

                billzone_eu_invoice_logme(__FILE__, 'OK', '#3: function billzone_eu_invoice_add_meta_box_and_popup_content > security code OK | Order ID: ' . $bei_order_data['id']);
            } else {

                // -- START -- Add meta box content -----------------------------------------------------------------------------------
                echo '<div class="billzone-eu-invoice-create-button-wrapper billzone-eu-invoice-woocommerce-missing-error">';
                echo '  <span  class="billzone-eu-invoice-text-white">' . __('ERROR', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
                echo '</div>';
                echo '<div class="billzone-eu-invoice-invoice-summary-wrapper">';
                echo '  <p class="billzone-eu-invoice-text-red">';
                echo __('Not all data is filled in on the billzone.eu Invoice setup page. Billing cannot be initiated.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
                echo '  </p>';
                echo '  <p  class="billzone-eu-invoice-text-red">';
                echo __('Visit the billzone.eu Invoice setup page and fill in all required fields or use the wizard to setup billzone.eu Invoice correctly.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
                echo '  </p>';
                echo '  <p>';
                echo '      <a href="' . admin_url() . 'admin.php?page=wc-settings&tab=billzone-eu-invoice-settings-tab" class="billzone-eu-invoice-button-error button">' . __('billzone.eu Invoice settings', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</a>';
                echo '  </p>';
                echo '</div>';
                // -- END ---- Add meta box content -----------------------------------------------------------------------------------

                billzone_eu_invoice_logme(__FILE__, 'ERROR', '#4A: function billzone_eu_invoice_add_meta_box_and_popup_content > security code ERROR');
            }

        } else {
            // -- START -- Add meta box content -----------------------------------------------------------------------------------
            echo '<div class="billzone-eu-invoice-create-button-wrapper billzone-eu-invoice-woocommerce-missing-error">';
            echo '  <span  class="billzone-eu-invoice-text-white">' . __('ERROR', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
            echo '</div>';
            echo '<div class="billzone-eu-invoice-invoice-summary-wrapper">';
            echo '  <p class="billzone-eu-invoice-text-red">';
            echo __('The billzone.eu security code is incorrect and billing cannot be started.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            echo '  </p>';
            echo '  <p  class="billzone-eu-invoice-text-red">';
            echo __('Security code status:', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' <b>' . $bei_security_code_status . '</b>';
            echo '  </p>';
            echo '  <p  class="billzone-eu-invoice-text-red">';
            echo __('Visit the billzone.eu Invoice setup page and use the wizard to enter the security code correctly.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
            echo '  </p>';
            echo '  <p>';
            echo '      <a href="' . admin_url() . 'admin.php?page=wc-settings&tab=billzone-eu-invoice-settings-tab" class="billzone-eu-invoice-button-error button">' . __('billzone.eu Invoice settings', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</a>';
            echo '  </p>';
            echo '</div>';
            // -- END ---- Add meta box content -----------------------------------------------------------------------------------

            billzone_eu_invoice_logme(__FILE__, 'ERROR', '#4B: function billzone_eu_invoice_add_meta_box_and_popup_content > security code ERROR');
        }
    } else {
        // -- START -- Add meta box content -----------------------------------------------------------------------------------
        echo '<div class="billzone-eu-invoice-create-button-wrapper billzone-eu-invoice-woocommerce-missing-error">';
        echo '  <span  class="billzone-eu-invoice-text-white">' . __('ERROR', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        echo '</div>';
        echo '<div class="billzone-eu-invoice-invoice-summary-wrapper">';
        echo '  <p class="billzone-eu-invoice-text-red">';
        echo __('The requirements for running the billzone.eu Invoice plugin were not met! Billing cannot be started.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        echo '  </p>';
        echo '  <p  class="billzone-eu-invoice-text-red">';
        echo __('Please delete and reinstall the plugin.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        echo '  </p>';
        echo '</div>';
        // -- END ---- Add meta box content -----------------------------------------------------------------------------------
    }
    
    billzone_eu_invoice_logme(__FILE__, 'OK', '#5: function billzone_eu_invoice_add_meta_box_and_popup_content > END');
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Add meta box and detail popup page
 * @access public
 */
function billzone_eu_invoice_add_meta_box()
{
    add_meta_box('billzone-eu-invoice-invoice-box', __('billzone.eu invoice summary', BILLZONE_EU_INVOICE_PLUGIN_NAME), 'billzone_eu_invoice_add_meta_box_and_popup_content', 'shop_order', 'side', 'high');
}
add_action('add_meta_boxes', 'billzone_eu_invoice_add_meta_box');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Action to update order details when change order parameters on admin page via AJAX (meta box data and popup windows order details will be updated)
 * @access public
 */
function billzone_eu_invoice_update_order_via_AJAX()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_update_order_via_AJAX > START');

    if (isset($_REQUEST)) {

        billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_update_order_via_AJAX > REQUEST START');

        // request data
        $bei_order_id = $_REQUEST['id'];

        $bei_order = new BEI_ORDER(wc_get_order($bei_order_id));
        $bei_order_data = $bei_order->bei_get_order_data();

        // set response type: JSON
        header("Content-type: application/json");

        // collect updated order data
        $bei_metabox = billzone_eu_invoice_add_meta_box_content($bei_order_data);
        $bei_metabox = bei_esc_html($bei_metabox);
        $bei_order_details = billzone_eu_invoice_add_popup_inside_elements($bei_order_data);
        $bei_order_details = bei_esc_html($bei_order_details);

        // response in JSON format
        echo '{"metabox":"' . $bei_metabox . '","orderdata":"' . $bei_order_details . '"}';

        billzone_eu_invoice_logme(__FILE__, 'OK', '#3: function billzone_eu_invoice_update_order_via_AJAX > REQUEST END');
    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#4: function billzone_eu_invoice_update_order_via_AJAX > END');

    die();
}
add_action('wp_ajax_billzone_eu_invoice_update_order_via_AJAX', 'billzone_eu_invoice_update_order_via_AJAX');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Action to create invoice via AJAX
 * @access public
 */
function billzone_eu_invoice_create_invoice_via_API()
{

    billzone_eu_invoice_logme(__FILE__, 'API - OK', '#1: function billzone_eu_invoice_create_invoice_via_API > START');

    if (isset($_REQUEST)) {

        billzone_eu_invoice_logme(__FILE__, 'API - OK', '#2: function billzone_eu_invoice_create_invoice_via_API > REQUEST START');

        global $wpdb;
        global $bei_options;

        // set API type (sandbox or production)
        if (strtoupper(get_option($bei_options['billzone_eu_invoice_is_sandbox'])) == 'YES') {
            $bei_API_class_source = 'billzone-eu-invoice-invoicing-service-sandbox.php';
        } else {
            $bei_API_class_source = 'billzone-eu-invoice-invoicing-service.php';
        }

        // request data from form
        $bei_invoice_data_from_form = $_REQUEST['invoicedata'];

        // create invoice request instance
        $bei_invoice = new BEI_INVOICE($bei_API_class_source);

        // mapping data from form data to API data
        $bei_invoice->TransactionId = bei_create_GUID();
        $bei_invoice->SourceSystem = BILLZONE_EU_INVOICE_SYSTEM_SOURCE;
        $bei_invoice->RequestId = bei_create_GUID();
        $bei_invoice->SecurityToken = trim(get_option($bei_options['billzone_eu_invoice_security_code']));
        $bei_invoice->OrderNumber = $bei_invoice_data_from_form['order_id'];
        $bei_invoice->Name = $bei_invoice_data_from_form['customer_name'];
        $bei_invoice->AddressPostalCode = $bei_invoice_data_from_form['postcode'];
        $bei_invoice->AddressCity = $bei_invoice_data_from_form['city'];
        $bei_invoice->AddressCountryId = $bei_invoice_data_from_form['country_value'];
        $bei_invoice->AddressLine1 = $bei_invoice_data_from_form['address'];
        $bei_invoice->TaxNumber = $bei_invoice_data_from_form['VAT_number'];
        $bei_invoice->EUTaxNumber = $bei_invoice_data_from_form['EU_VAT_number'];

        $bei_invoice->HasHungarianTaxNumber = (strtoupper($bei_invoice_data_from_form['hasHungarianTaxNumber']) == 'YES') ? true : false;
        $bei_invoice->LegalEntityWithoutTaxNumber = (strtoupper($bei_invoice_data_from_form['legalEntityWithoutTaxNumber']) == 'YES') ? true : false;

        $bei_invoice->BankIdentifier = $bei_invoice_data_from_form['payment_method_value'];
        $bei_invoice->AccountBlockPrefix = trim(get_option($bei_options['billzone_eu_invoice_invoice_number_prefix']));
        $bei_invoice->InvoiceDescription = $bei_invoice_data_from_form['notes'];
        $bei_invoice->SendInvoiceToCustomer = (strtoupper(trim(get_option($bei_options['billzone_eu_invoice_send_email']))) == 'YES') ? true : false;
        $bei_invoice->SendInvoiceToEmailAddress = trim($bei_invoice_data_from_form['email']);
        if (array_key_exists('exchange_rate', $bei_invoice_data_from_form)) {
            $bei_invoice->LocalForeignCurrencyExchangeRate = $bei_invoice_data_from_form['exchange_rate'];
        } else {
            $bei_invoice->LocalForeignCurrencyExchangeRate = null;
        }

        $bei_invoice->FulfillmentDate = $bei_invoice_data_from_form['delivery_date'];
        $bei_invoice->PaymentDueDate = $bei_invoice_data_from_form['payment_due'];

        $bei_invoice->Lines = array();

        if (!empty($bei_invoice_data_from_form['lines'])) {
            foreach ($bei_invoice_data_from_form['lines'] as $bei_line) {

                $bei_line_data = new InvoiceLine2();
                $bei_line_data->ProductStatisticalCode = '';
                $bei_line_data->ProductName = $bei_line['name'];
                $bei_line_data->NetUnitPrice = $bei_line['net_unit_price'];
                $bei_line_data->Quantity = $bei_line['quantity'];
                $bei_line_data->VatTaxRateCode = $bei_line['tax_code_key'];
                $bei_line_data->UnitIdentifier = $bei_line['unit_of_measure'];
                $bei_line_data->ProductTextIdentifier = '';
                $bei_line_data->PeriodStartDate = null;
                $bei_line_data->PeriodEndDate = null;
                $bei_line_data->InvoiceLineNote = $bei_line['meta'];

                array_push($bei_invoice->Lines, $bei_line_data);

            }
        }

        // create invoice
        $bei_create_invoice_request_result = $bei_invoice->create_invoice();

        if (($bei_create_invoice_request_result->CreateInvoiceResult->RequestResult->Code == BILLZONE_EU_INVOICE_OK_SECURITY_CODE) &&
            ($bei_create_invoice_request_result->CreateInvoiceResult->TransactionResult->ResultCode->Code == BILLZONE_EU_INVOICE_OK_SECURITY_CODE)) {

            $wpdb->insert(
                BILLZONE_EU_INVOICE_TABLE_NAME,
                array(
                    'order_id' => $bei_invoice_data_from_form['order_id'],
                    'order_key' => $bei_invoice_data_from_form['order_key'],
                    'request_id' => $bei_invoice->RequestId,
                    'transaction_id' => $bei_invoice->TransactionId,
                    'invoice_number' => $bei_create_invoice_request_result->CreateInvoiceResult->TransactionResult->InvoiceNumber,
                    'invoice_type' => '',
                    'invoice_url' => $bei_create_invoice_request_result->CreateInvoiceResult->TransactionResult->InvoiceCourierUrl,
                    'invoice_creation_date' => date('Y-m-d H:i:s', time()),
                )
            );

        } else {
            $bei_create_invoice_request_result->CreateInvoiceResult->TransactionResult->ResultCode->Message = bei_get_API_error($bei_create_invoice_request_result->CreateInvoiceResult->TransactionResult->ResultCode->Code)['message'];
        }

        // set response type: JSON
        header("Content-type: application/json");

        // response in JSON format
        echo json_encode($bei_create_invoice_request_result);

        billzone_eu_invoice_logme(__FILE__, 'API - OK', '#3: function billzone_eu_invoice_create_invoice_via_API > REQUEST END');

    }

    billzone_eu_invoice_logme(__FILE__, 'API - OK', '#4: function billzone_eu_invoice_create_invoice_via_API > END');

    die();
}
add_action('wp_ajax_billzone_eu_invoice_create_invoice_via_API', 'billzone_eu_invoice_create_invoice_via_API');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Action to start wizard panel via AJAX
 * @access public
 */
function billzone_eu_invoice_start_wizard_AJAX()
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_start_wizard_AJAX > START');

    if (isset($_REQUEST)) {

        billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_start_wizard_AJAX > REQUEST START');

        global $bei_options;

        // Get all enabled woocommerce payment method
        $bei_payment_method_display_title = '<div><h4>' . __('Woocommerce payment method', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
        $bei_payment_method_display_code = '<div><h4>' . __('Woocommerce payment code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</h4></div>';
        foreach (WC()->payment_gateways->payment_gateways() as $bei_payment_gateway) {
            if (strtoupper($bei_payment_gateway->enabled) == "YES") {
                $bei_payment_method_display_title .= '<div>' . $bei_payment_gateway->title . ':</div>';
                $bei_payment_method_display_code .= '<div><b class="billzone-eu-invoice-text-brand-color">' . strtoupper($bei_payment_gateway->id) . '</b></div>';
            }
        }

        $bei_payment_method_table = '<div class="billzone-eu-invoice-settings-help-wrapper"><div>' . $bei_payment_method_display_title . '</div><div>' . $bei_payment_method_display_code . '</div></div>';

        // set the help image language according to Wordpress language
        $bei_lang = trim(substr(strtolower(get_bloginfo("language")), 0, 2));
        if (($bei_lang != 'en') && ($bei_lang != 'hu')) {
            $bei_lang = 'en';
        }

        // create wizard panel with all element included and translated
        $beiSetupWizardContainer = '';
        $beiSetupWizardContainer .= '<div class="billzone-eu-invoice-wizard-container">';
        $beiSetupWizardContainer .= '    <div class="billzone-eu-invoice-wizard-panel">';
        $beiSetupWizardContainer .= '        <div class="billzone-eu-invoice-wizard-title">';
        $beiSetupWizardContainer .= BILLZONE_EU_INVOICE_READABLE_PLUGIN_NAME . ' ' . __('setup wizard', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ' (v' . BILLZONE_EU_INVOICE_PLUGIN_VERSION_NUM . ')';
        $beiSetupWizardContainer .= '           <span class="billzone-eu-invoice-show-wizard-help dashicons dashicons-info"></span>';
        $beiSetupWizardContainer .= '           <span class="billzone-eu-invoice-close-wizard dashicons dashicons-no-alt"></span>';
        $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-wizard-help-wrapper">';
        $beiSetupWizardContainer .= '               <div class="billzone-eu-invoice-wizard-help-wrapper-title">';
        $beiSetupWizardContainer .= '                   <span class="billzone-eu-invoice-help-image-1">' . __('Payment method can be created in this menu', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $beiSetupWizardContainer .= '                   <span class="billzone-eu-invoice-help-image-2 billzone-eu-invoice-hide">' . __('API code can be found in this menu', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $beiSetupWizardContainer .= '                   <span class="billzone-eu-invoice-close-wizard-help dashicons dashicons-no-alt"></span>';
        $beiSetupWizardContainer .= '               </div>';
        $beiSetupWizardContainer .= '               <img class="billzone-eu-invoice-help-image-1" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-add-payment-' . $bei_lang . '.png" alt="billzone.eu Invoice help image">';
        $beiSetupWizardContainer .= '               <img class="billzone-eu-invoice-help-image-2 billzone-eu-invoice-hide" src="' . BILLZONE_EU_INVOICE_PLUGIN_DIR . '/img/billzone-eu-invoice-api-code-' . $bei_lang . '.png" alt="billzone.eu Invoice help image">';
        $beiSetupWizardContainer .= '           </div>';
        $beiSetupWizardContainer .= '        </div>';
        $beiSetupWizardContainer .= '        <div class="billzone-eu-invoice-wizard-body">';
        $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-wizard-slideframe-wrapper">';

        // WIZARD FRAME --START---------------------------------------------------------
        $beiSetupWizardContainer .= '               <div class="billzone-eu-invoice-wizard-slideframe" style="left: 0;">';

        // WIZARD STEP 0 --START--------------------------------------------------------
        $beiSetupWizardContainer .= '                   <div class="billzone-eu-invoice-wizard-step-wrapper billzone-eu-invoice-wizard-step-0">';
        $beiSetupWizardContainer .= '                       <div class="billzone-eu-invoice-wizard-step-counter">';
        $beiSetupWizardContainer .= '                           <span>0</span>';
        $beiSetupWizardContainer .= '                       </div>';
        $beiSetupWizardContainer .= '                       <div class="billzone-eu-invoice-wizard-step-content">';
        $beiSetupWizardContainer .= '                           <p class="billzone-eu-invoice-text-justify">';
        $beiSetupWizardContainer .= __('Before you set up the program, add the codes for the payment methods listed here in billzone.eu account MASTER DATA / PAYMENT METHOD, otherwise billzone.eu Invoice will work incorrectly. For each of the codes listed here, create a new form of payment using the <b>&raquo;New banking info&laquo;</b> and enter the codes below under <i>&raquo;Payment method identifier corresponding to interface (API)&laquo;</i> (for help click on info icon on panel title).', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        $beiSetupWizardContainer .= '                           </p>';
        $beiSetupWizardContainer .= $bei_payment_method_table;
        $beiSetupWizardContainer .= '                           <span class="button button-primary billzone-eu-invoice-start-wizard">' . __('Start wizard', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<span class="dashicons dashicons-arrow-right-alt2"></span></span>';
        $beiSetupWizardContainer .= '                       </div>';
        $beiSetupWizardContainer .= '                   </div>';
        // WIZARD STEP 0 --END----------------------------------------------------------

        // WIZARD STEP 1 --START--------------------------------------------------------
        $beiSetupWizardContainer .= '                   <div class="billzone-eu-invoice-wizard-step-wrapper billzone-eu-invoice-wizard-step-1">';
        $beiSetupWizardContainer .= '                       <div class="billzone-eu-invoice-wizard-step-counter">';
        $beiSetupWizardContainer .= '                           <span>1</span>';
        $beiSetupWizardContainer .= '                       </div>';
        $beiSetupWizardContainer .= '                       <div class="billzone-eu-invoice-wizard-step-content">';
        $beiSetupWizardContainer .= '                           <div>';
        $beiSetupWizardContainer .= '                               <label for="beiWizardSecurityCode">' . __('Security code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
        $beiSetupWizardContainer .= '                           </div>';
        $beiSetupWizardContainer .= '                           <div>';
        $beiSetupWizardContainer .= '                               <input id="beiWizardSecurityCode" name="beiWizardSecurityCode" data-mapfor="' . $bei_options['billzone_eu_invoice_security_code'] . '" class="bei-wizard-form-element bei-wizard-form-element-input" type="text" value="">';
        $beiSetupWizardContainer .= '                           </div>';
        $beiSetupWizardContainer .= '                           <div class="billzone-eu-invoice-margin-top-10px">';
        $beiSetupWizardContainer .= '                               <input id="beiWizardIsSandbox" name="beiWizardIsSandbox" data-mapfor="' . $bei_options['billzone_eu_invoice_is_sandbox'] . '" class="bei-wizard-form-element" type="checkbox" value="">';
        $beiSetupWizardContainer .= '                               <label for="beiWizardIsSandbox">' . __('Sandbox invoicing', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
        $beiSetupWizardContainer .= '                           </div>';
        $beiSetupWizardContainer .= '                           <p class="billzone-eu-invoice-text-justify">';
        $beiSetupWizardContainer .= __('For your security code please check ACCOUNT SETTINGS / API SETTINGS menu in your billzone.eu account (for help click on info icon on panel title).', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        $beiSetupWizardContainer .= '                           </p>';
        $beiSetupWizardContainer .= '                           <p class="billzone-eu-invoice-text-justify">';
        $beiSetupWizardContainer .= __('Attention! Different security codes are used on the LIVE and SANDBOX servers, so take care which one you write here.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        $beiSetupWizardContainer .= '                           </p>';
        $beiSetupWizardContainer .= '                           <p class="billzone-eu-invoice-text-justify">';
        $beiSetupWizardContainer .= __('Test the entered code, if you have entered it correctly, you can go to the next step.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
        $beiSetupWizardContainer .= '                           </p>';
        $beiSetupWizardContainer .= '                           <hr>';

        $beiSetupWizardContainer .= '                           <div class="billzone-eu-invoice-api-response">';
        $beiSetupWizardContainer .= '                               <ul>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Server', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-server"></span></li>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Status', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-response-text"></span></li>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Status code', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-response-code"></span></li>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Name', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-response-companyname"></span></li>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Address', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-response-address"></span></li>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Tax nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-response-taxnumber"></span></li>';
        $beiSetupWizardContainer .= '                                   <li>' . __('Registration nr.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ': <span class="billzone-eu-invoice-api-response-registrationnumber"></span></li>';
        $beiSetupWizardContainer .= '                               </ul>';
        $beiSetupWizardContainer .= '                           </div>';

        $beiSetupWizardContainer .= '                           <span class="button button-primary billzone-eu-invoice-prev-step"><span class="dashicons dashicons-arrow-left-alt2"></span>' . __('Previous', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $beiSetupWizardContainer .= '                           <span class="button button-primary billzone-eu-invoice-test">' . __('Test connection', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
        $beiSetupWizardContainer .= '                           <span class="button button-primary billzone-eu-invoice-next-step billzone-eu-invoice-hide">' . __('Next', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<span class="dashicons dashicons-arrow-right-alt2"></span></span>';
        $beiSetupWizardContainer .= '                       </div>';
        $beiSetupWizardContainer .= '                   </div>';
        // WIZARD STEP 1 --END----------------------------------------------------------

        $beiSetupWizardContainer .= '               </div>';
        // WIZARD FRAME --END---------------------------------------------------------

        $beiSetupWizardContainer .= '           </div>';
        $beiSetupWizardContainer .= '        </div>';
        $beiSetupWizardContainer .= '    </div>';
        $beiSetupWizardContainer .= '</div>';

        // set response type: JSON
        //header("Content-type: application/json");
        header("Content-type: text/plain");

        // response in JSON format
        echo '{"panel":"' . bei_esc_html($beiSetupWizardContainer) . '"}';

        billzone_eu_invoice_logme(__FILE__, 'OK', '#3: function billzone_eu_invoice_start_wizard_AJAX > REQUEST END');

    }

    billzone_eu_invoice_logme(__FILE__, 'OK', '#4: function billzone_eu_invoice_start_wizard_AJAX > END');

    wp_die();
}
add_action('wp_ajax_billzone_eu_invoice_start_wizard_AJAX', 'billzone_eu_invoice_start_wizard_AJAX');
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Action to retrive billzone.eu API response for wizard panel via AJAX
 * @access public
 */
function billzone_eu_invoice_next_steps($bei_request_result)
{

    billzone_eu_invoice_logme(__FILE__, 'OK', '#1: function billzone_eu_invoice_next_steps > START');

    global $bei_options;

    // WIZARD STEP 2 --START--------------------------------------------------------
    $beiSetupWizardContainer = ' <div class="billzone-eu-invoice-wizard-step-wrapper billzone-eu-invoice-wizard-step-2">';
    $beiSetupWizardContainer .= '   <div class="billzone-eu-invoice-wizard-step-counter">';
    $beiSetupWizardContainer .= '       <span>2</span>';
    $beiSetupWizardContainer .= '   </div>';
    $beiSetupWizardContainer .= '   <div class="billzone-eu-invoice-wizard-step-content">';

    $beiSetupWizardContainer .= '       <div>';
    $beiSetupWizardContainer .= __('Invoice sequence prefixes', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-flex">';
    $beiSetupWizardContainer .= '           <div>';
    $beiSetupWizardContainer .= '               <select id="beiWizardAccountBlock" data-mapfor="' . $bei_options['billzone_eu_invoice_invoice_number_prefix'] . '" data-defaultcurrency="' . $bei_request_result['InvoiceIssuerCountryDefaultCurrency'] . '" data-issuercountrycode="' . $bei_request_result['GetInvoiceDataResult']->QueryResult->CompanyData->AddressCountryId . '" class="bei-wizard-form-element bei-wizard-form-element-select">';
    foreach ($bei_request_result['GetInvoiceDataResult']->QueryResult->AccountBlocks->AccountBlocksQueryResultItem as $bei_accountblock) {
        $beiSetupWizardContainer .= '               <option value="' . $bei_accountblock->CurrencyShortName . '">' . $bei_accountblock->AccountBlockPrefix . '</option>';
    }
    $beiSetupWizardContainer .= '               </select>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-pl3">';
    $beiSetupWizardContainer .= '               <label for="beiWizardAccountBlock"></label>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div>';
    $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-get-exchange-rate-wrapper">';
    $beiSetupWizardContainer .= '               <input id="beiGetExchangeRate" name="beiGetExchangeRate" data-mapfor="' . $bei_options['billzone_eu_invoice_get_exchange_rate'] . '" class="bei-wizard-form-element" type="checkbox" checked="checked">';
    $beiSetupWizardContainer .= '               <label for="beiGetExchangeRate">' . __('Get MNB exchange rate for domestic currency billing', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $beiSetupWizardContainer .= '               <div class="billzone-eu-invoice-text-justify">';
    $beiSetupWizardContainer .= '                   <small>(' . __('In the case of domestic currency billing, the exchange rate is automatically filled in according to the current exchange rate of the MNB. Can be overridden when the invoice is issued.', BILLZONE_EU_INVOICE_PLUGIN_NAME) . ')</small>';
    $beiSetupWizardContainer .= '               </div>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= __('Company location ID', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-flex">';
    $beiSetupWizardContainer .= '           <div>';
    $beiSetupWizardContainer .= '               <select id="beiWizardCompanyLocationId" data-mapfor="' . $bei_options['billzone_eu_invoice_company_location_id'] . '" class="bei-wizard-form-element bei-wizard-form-element-select">';
    foreach ($bei_request_result['GetInvoiceDataResult']->QueryResult->Issuers->IssuersQueryResultItem as $bei_companylocationid) {
        $beiSetupWizardContainer .= '               <option value="' . $bei_companylocationid->Name . '">' . $bei_companylocationid->InvoiceIssuerIdentifier . '</option>';
    }
    $beiSetupWizardContainer .= '               </select>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-pl3">';
    $beiSetupWizardContainer .= '               <label for="beiWizardCompanyLocationId"></label>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= __('Default pricing unit', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-flex">';
    $beiSetupWizardContainer .= '           <div>';
    $beiSetupWizardContainer .= '               <select id="beiWizardDefaultPricingUnit" data-mapfor="' . $bei_options['billzone_eu_invoice_default_pricing_unit'] . '" class="bei-wizard-form-element bei-wizard-form-element-select">';
    foreach ($bei_request_result['GetInvoiceDataResult']->QueryResult->Units->UnitsQueryResultItem as $bei_defaultpricingunit) {
        $beiSetupWizardContainer .= '               <option value="' . $bei_defaultpricingunit->Name . '">' . $bei_defaultpricingunit->UnitIdentifier . '</option>';
    }
    $beiSetupWizardContainer .= '               </select>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-pl3">';
    $beiSetupWizardContainer .= '               <label for="beiWizardDefaultPricingUnit"></label>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= __('Default payment method', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-flex">';
    $beiSetupWizardContainer .= '           <div>';
    $beiSetupWizardContainer .= '               <select id="beiWizardDefaultPaymentMethod" data-mapfor="' . $bei_options['billzone_eu_invoice_default_payment_methode'] . '" class="bei-wizard-form-element bei-wizard-form-element-select">';
    foreach ($bei_request_result['GetInvoiceDataResult']->QueryResult->PaymentMethods->PaymentMethodsQueryResultItem as $bei_defaultpaymentmethod) {
        $beiSetupWizardContainer .= '               <option value="' . $bei_defaultpaymentmethod->PaymentMeansType . '">' . $bei_defaultpaymentmethod->BankIdentifier . '</option>';
    }
    $beiSetupWizardContainer .= '               </select>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-pl3">';
    $beiSetupWizardContainer .= '               <label for="beiWizardDefaultPaymentMethod"></label>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <p class="billzone-eu-invoice-text-justify">';
    $beiSetupWizardContainer .= __('These codes came from your billzone.eu account, if you are missing some data, navigate to the appropriate MASTER DATA menu in your billzone.eu account and create them.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </p>';

    $beiSetupWizardContainer .= '       <span class="button button-primary billzone-eu-invoice-prev-step"><span class="dashicons dashicons-arrow-left-alt2"></span>' . __('Previous', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $beiSetupWizardContainer .= '       <span class="button button-primary billzone-eu-invoice-next-step">' . __('Next', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<span class="dashicons dashicons-arrow-right-alt2"></span></span>';

    $beiSetupWizardContainer .= '   </div>';
    $beiSetupWizardContainer .= '</div>';
    // WIZARD STEP 2 --END----------------------------------------------------------

    // WIZARD STEP 3 --START--------------------------------------------------------
    $beiSetupWizardContainer .= '<div class="billzone-eu-invoice-wizard-step-wrapper billzone-eu-invoice-wizard-step-3">';
    $beiSetupWizardContainer .= '   <div class="billzone-eu-invoice-wizard-step-counter">';
    $beiSetupWizardContainer .= '       <span>3</span>';
    $beiSetupWizardContainer .= '   </div>';
    $beiSetupWizardContainer .= '   <div class="billzone-eu-invoice-wizard-step-content">';

    $beiSetupWizardContainer .= '       <div>';
    $beiSetupWizardContainer .= __('Default country code for UN/EDIFACT codes', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-flex">';
    $beiSetupWizardContainer .= '           <div>';
    $beiSetupWizardContainer .= '               <select id="beiWizardDefaultCountryCodeForUNEDIFACTCodes" data-mapfor="' . $bei_options['billzone_eu_invoice_default_country_for_UN_EDIFACT_tax_code'] . '" class="bei-wizard-form-element bei-wizard-form-element-select">';

    $beiAllTaxCountryCode = bei_get_all_API_returned_tax_country_code($bei_request_result['GetInvoiceDataResult']->QueryResult->Taxes->TaxesQueryResultItem);
    foreach ($beiAllTaxCountryCode as $beiTaxCountryCode) {
        $beiWordpressLang = trim(substr(strtoupper(get_bloginfo("language")), 0, 2));
        // If the language code of the Wordpress is EN then we will change it to GB (because of the API constrain)
        if ($beiWordpressLang == 'EN') {
            $beiWordpressLang = 'GB';
        }
        ($beiTaxCountryCode == $beiWordpressLang) ? $beiOptionSelected = 'selected="selected"' : $beiOptionSelected = '';
        $beiSetupWizardContainer .= '               <option ' . $beiOptionSelected . 'value="' . bei_get_all_country_code()[$beiTaxCountryCode] . '">' . $beiTaxCountryCode . '</option>';
    }

    $beiSetupWizardContainer .= '               </select>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '           <div class="billzone-eu-invoice-pl3">';
    $beiSetupWizardContainer .= '               <label for="beiWizardDefaultCountryCodeForUNEDIFACTCodes"></label>';
    $beiSetupWizardContainer .= '           </div>';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= '           <label for="beiWizardDateOfDeliveryPlusDays">' . __('Date of delivery plus days', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div>';
    $beiSetupWizardContainer .= '           <input id="beiWizardDateOfDeliveryPlusDays" name="beiWizardDateOfDeliveryPlusDays" data-mapfor="' . $bei_options['billzone_eu_invoice_date_of_delivery_plus_days'] . '" class="bei-wizard-form-element bei-wizard-form-element-input" type="number" value="0" min="0">';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= '           <label for="beiWizardPaymentDuePlusDays">' . __('Payment due plus days', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div>';
    $beiSetupWizardContainer .= '           <input id="beiWizardPaymentDuePlusDays" name="beiWizardPaymentDuePlusDays" data-mapfor="' . $bei_options['billzone_eu_invoice_payment_due_plus_days'] . '" class="bei-wizard-form-element bei-wizard-form-element-input" type="number" value="0" min="0">';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= '           <label for="beiWizardInvoiceComment">' . __('Invoice&nbsp;comment', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $beiSetupWizardContainer .= '       </div>';
    $beiSetupWizardContainer .= '       <div>';
    $beiSetupWizardContainer .= '           <input id="beiWizardInvoiceComment" name="beiWizardInvoiceComment" data-mapfor="' . $bei_options['billzone_eu_invoice_invoice_notes'] . '" class="bei-wizard-form-element bei-wizard-form-element-input" type="text" value="">';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <div class="billzone-eu-invoice-margin-top-10px">';
    $beiSetupWizardContainer .= '           <input id="beiWizardSendInvoiceForCustomerViaEmail" name="beiWizardSendInvoiceForCustomerViaEmail" data-mapfor="' . $bei_options['billzone_eu_invoice_send_email'] . '" class="bei-wizard-form-element" type="checkbox" value="">';
    $beiSetupWizardContainer .= '           <label for="beiWizardSendInvoiceForCustomerViaEmail">' . __('Send invoice for customer via e-mail', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</label>';
    $beiSetupWizardContainer .= '       </div>';

    $beiSetupWizardContainer .= '       <p class="billzone-eu-invoice-text-justify billzone-eu-invoice-text-brand-color">';
    $beiSetupWizardContainer .= __('Your settings have been successfully completed. Clicking on the Save button saves the entered data, and the billing can now be used. For invoicing enter in Woocommerce Orders menu, and click on order detail. You will see an invoicing settings panel on right side.', BILLZONE_EU_INVOICE_PLUGIN_NAME);
    $beiSetupWizardContainer .= '       </p>';

    $beiSetupWizardContainer .= '       <span class="button button-primary billzone-eu-invoice-prev-step"><span class="dashicons dashicons-arrow-left-alt2"></span>' . __('Previous', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '</span>';
    $beiSetupWizardContainer .= '       <span class="button button-primary billzone-eu-invoice-save-settings">' . __('Save settings', BILLZONE_EU_INVOICE_PLUGIN_NAME) . '<span class="dashicons dashicons-yes"></span></span>';

    $beiSetupWizardContainer .= '   </div>';
    $beiSetupWizardContainer .= '</div>';
    // WIZARD STEP 3 --END----------------------------------------------------------

    billzone_eu_invoice_logme(__FILE__, 'OK', '#2: function billzone_eu_invoice_next_steps > END');

    return '"' . bei_esc_html($beiSetupWizardContainer) . '"';
}
//----------------------------------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------------------------------
/**
 * Check API token and return response via AJAX
 * @access public
 */
function billzone_eu_invoice_check_API_token_and_return_response_AJAX()
{

    billzone_eu_invoice_logme(__FILE__, 'API - OK', '#1: function billzone_eu_invoice_check_API_token_and_return_response_AJAX > START');

    if (isset($_REQUEST)) {

        billzone_eu_invoice_logme(__FILE__, 'API - OK', '#2: function billzone_eu_invoice_check_API_token_and_return_response_AJAX > REQUEST START');

        // request data from form
        $bei_wizard_data_from_form = $_REQUEST;

        // set API type (sandbox or production)
        $bei_server_path = BILLZONE_EU_INVOICE_API_WSDL_PATH;
        if (strtolower($bei_wizard_data_from_form['issandbox']) == 'true') {
            $bei_API_class_source = 'billzone-eu-invoice-invoicing-service-sandbox.php';
            $bei_server_path = BILLZONE_EU_INVOICE_SANDBOX_API_WSDL_PATH;
        } else {
            $bei_API_class_source = 'billzone-eu-invoice-invoicing-service.php';
        }

        // create invoice request instance
        $bei_invoice = new BEI_INVOICE($bei_API_class_source);

        // mapping data to API data
        $bei_invoice->RequestId = bei_create_GUID();
        $bei_invoice->SecurityToken = $bei_wizard_data_from_form['securitycode'];

        // create API request
        $bei_request_result = $bei_invoice->get_invoice_data();

        if ($bei_request_result['GetInvoiceDataResult']->RequestResult->Code == BILLZONE_EU_INVOICE_OK_SECURITY_CODE) {
            $bei_next_steps_html = billzone_eu_invoice_next_steps($bei_request_result);
        } else {
            $bei_next_steps_html = '""';
        }

        // set response type: JSON
        header("Content-type: application/json");

        // response in JSON format
        echo '{"requestresult":' . json_encode($bei_request_result) . ',"status":' . json_encode(bei_get_API_error($bei_request_result['GetInvoiceDataResult']->RequestResult->Code)) . ',"nextstepshtml":' . $bei_next_steps_html . ',"serverpath": "'.$bei_server_path.'"}';

        billzone_eu_invoice_logme(__FILE__, 'API - OK', '#3: function billzone_eu_invoice_check_API_token_and_return_response_AJAX > REQUEST END');

    }

    billzone_eu_invoice_logme(__FILE__, 'API - OK', '#4: function billzone_eu_invoice_check_API_token_and_return_response_AJAX > END');

    die();
}
add_action('wp_ajax_billzone_eu_invoice_check_API_token_and_return_response_AJAX', 'billzone_eu_invoice_check_API_token_and_return_response_AJAX');
//----------------------------------------------------------------------------------------------------------------------------------------------
