/*
 * All javascript code related to billzone.eu Invoice plugin
 *
 *
 * @link              https://n-ware.hu
 * @since             0.8.1
 * @package           billzone.eu Invoice
 *
 */

// API transaction succes code
const BEI_REQUEST_SUCCES_CODE = 61001;

// global variable to store invoice data
var Bei_Order_Data;

// global variable to check if log is ON
var Bei_Log_Is_Enabled = false;
jQuery.ajax({
    url: ajaxurl,
    type: 'POST',
    dataType: 'json',
    data: {
        'action': 'billzone_eu_invoice_log_is_on_AJAX',
    },
    success: function (data) {
        // check if log is enabled via AJAX
        (data.logstatus.toUpperCase() == "ON") ? Bei_Log_Is_Enabled = true : Bei_Log_Is_Enabled = false;
    },
    error: function (errorThrown) {
        beiWriteErrorToConsole('billzone.eu Invoice AJAX error! See detail bellow:', errorThrown);
    }
});



// event listener for different order update actions via AJAX, to show the changed order data in meta box and popup window
// TODO is it possible to use woocommerce API instead of jQuery?
jQuery(document).ajaxComplete(function (event, xhr, settings) {

    var beiUrl = new URL(window.location.href.split('?')[0] + '?' + settings.data);
    var beiAction = beiUrl.searchParams.get('action');

    // run only on specific woocommerce order update action
    if (beiAction == 'woocommerce_save_order_items' ||
        beiAction == 'woocommerce_add_order_item' ||
        beiAction == 'woocommerce_remove_order_item' ||
        beiAction == 'woocommerce_add_coupon_discount' ||
        beiAction == 'woocommerce_remove_order_coupon' ||
        beiAction == 'woocommerce_refund_line_items' ||
        beiAction == 'woocommerce_add_order_tax' ||
        beiAction == 'woocommerce_add_order_fee' ||
        beiAction == 'woocommerce_delete_refund' ||
        beiAction == 'woocommerce_load_order_items' ||
        beiAction == 'woocommerce_add_order_shipping' ||
        beiAction == 'woocommerce_calc_line_taxes') {

        // get order ID
        var beiId = document.getElementById('post_ID').value;

        // add ajax loader animation
        document.getElementById('billzone-eu-invoice-invoice-box').classList.add('ajax-load');

        // action when order already created

        jQuery.ajax({
            url: ajaxurl,
            type: 'POST',
            dataType: 'json',
            data: {
                'action': 'billzone_eu_invoice_update_order_via_AJAX',
                'id': beiId
            },
            success: function (data) {

                // get meta box invoice summary wrapper element
                var beiInvoiceSummaryDataWrapperElement = document.querySelector('.billzone-eu-invoice-invoice-summary-wrapper');

                // empty meta box invoice summary wrapper
                while (beiInvoiceSummaryDataWrapperElement.firstChild) {
                    beiInvoiceSummaryDataWrapperElement.removeChild(beiInvoiceSummaryDataWrapperElement.firstChild);
                }

                // add the new summary data to summary wrapper
                beiInvoiceSummaryDataWrapperElement.insertAdjacentHTML('afterbegin', data.metabox);

                // add new order detail data for popup window
                Bei_Order_Data = data.orderdata;

                // remove ajax loader animation
                document.getElementById('billzone-eu-invoice-invoice-box').classList.remove('ajax-load');

            },
            error: function (errorThrown) {
                beiWriteErrorToConsole('billzone.eu Invoice AJAX error! See detail bellow:', errorThrown);
            }
        });

    }

});


jQuery(document).ready(function () {

    // event listener for create invoice button and shop popup window fullfiled with initial data
    var beiCreateButton = document.querySelector('#billzone_eu_invoice_Create_Button');
    if (document.body.contains(beiCreateButton)) {

        beiCreateButton.addEventListener('click', function () {

            if (beiCreateButton.classList.contains('billzone-eu-invoice-create-btn')) {
                var beiElementToAttach = document.querySelector('.billzone-eu-invoice-detail-container');

                // empty container
                while (beiElementToAttach.firstChild) {
                    beiElementToAttach.removeChild(beiElementToAttach.firstChild);
                }
                // add invoice elements
                beiElementToAttach.insertAdjacentHTML('afterbegin', Bei_Order_Data);

                beiEventAttacher();

                document.querySelector('.billzone-eu-invoice-detail-wrapper').classList.remove('billzone-eu-invoice-hide-detail');
                setTimeout(function () {
                    beiElementToAttach.classList.remove('billzone-eu-invoice-detail-container-initial-position');
                }, 100);

                beiAttachFieldValidation();
                beiValidate();

            }
        });

    }

    // event listener for starting setup wizard
    var beiSetupWizardStartElement = document.querySelector('#billzone_start_setup_wizard');
    if (document.body.contains(beiSetupWizardStartElement)) {
        beiSetupWizardStartElement.addEventListener('click', function () {
            beiStartSetupWizard();
        });
    }

    // event listener for clear log database
    var beiClearLogElement = document.querySelector('#billzone_eu_invoice_clear_log');
    if (document.body.contains(beiClearLogElement)) {
        beiClearLogElement.addEventListener('click', function () {
            beiShowLogActionMessagePanel('billzone_eu_invoice_clear_log_AJAX', 'billzone_eu_invoice_clear_log');
        });
    }

    // event listener for export log database
    var beiExportLogElement = document.querySelector('#billzone_eu_invoice_export_log');
    if (document.body.contains(beiExportLogElement)) {
        beiExportLogElement.addEventListener('click', function () {
            beiShowLogActionMessagePanel('billzone_eu_invoice_export_log_AJAX', 'billzone_eu_invoice_export_log');
        });
    }
});

function beiWriteErrorToConsole(beiErrorText = 'billzone.eu Invoice AJAX error! See detail bellow:', errorThrown) {
    console.log(beiErrorText);
    console.log('Response status code: ' + errorThrown.status);
    console.log('Response status text: ' + errorThrown.statusText);
    console.log(errorThrown);
}

// all event listener
function beiEventAttacher() {

    // event listener for all detail panel close buttons
    var beiCloseButton = document.querySelectorAll('.billzone-eu-invoice-close, .billzone-eu-invoice-cancel');
    if (beiCloseButton.length > 0) {
        for (var i = 0; i < beiCloseButton.length; i++) {
            beiCloseButton[i].addEventListener('click', function () {
                beiClosePopupPanel();
            });
        }
    }

    //event listener to show warning info text
    var beiInfoText = document.querySelector('.billzone-eu-invoice-info');
    if (document.body.contains(beiInfoText)) {
        beiInfoText.addEventListener('click', function () {
            document.querySelector('.billzone-eu-invoice-info-text').classList.toggle('billzone-eu-invoice-show');
        });
    }

    // event listener for info text wrapper close button
    var beiInfoTextCloseButton = document.querySelector('.billzone-eu-invoice-info-text-close');
    if (document.body.contains(beiInfoTextCloseButton)) {
        beiInfoTextCloseButton.addEventListener('click', function () {
            this.parentNode.classList.remove('billzone-eu-invoice-show');
        });
    }

    // event listener for all VAT code changer
    var beiUNEDIFACTElements = document.querySelectorAll('.billzone-eu-invoice-item-UN-EDIFACT-codes');
    if (beiUNEDIFACTElements.length > 0) {
        for (var i = 0; i < beiUNEDIFACTElements.length; i++) {
            beiUNEDIFACTElements[i].addEventListener('change', function () {
                this.parentNode.parentNode.querySelector('.billzone-eu-invoice-item-UN-EDIFACT-value').innerHTML = this.options[this.selectedIndex].value;
                beiCheckOriginalVATandApplyWarningColor(this);
                beiCheckIfAnyOriginalVATandApplyWarningColor();
            });
        }
    }

    // event listener for UN/EDIFACT code changer for all elements
    var beiUNEDIFACTForAllItemElement = document.querySelector('#billzone_eu_invoice_all_item_UN_EDIFACT_codes');
    if (document.body.contains(beiUNEDIFACTForAllItemElement)) {
        beiUNEDIFACTForAllItemElement.addEventListener('change', function () {
            this.parentNode.querySelector('.billzone-eu-invoice-item-UN-EDIFACT-value').innerHTML = this.options[this.selectedIndex].value;
            beiChangeAllVATToSameValue(this.selectedIndex);
        });
    }

    // event listener for country codes changer
    var beiCountryCodes = document.querySelector('.billzone-eu-invoice-country-codes');
    if (document.body.contains(beiCountryCodes)) {
        beiCountryCodes.addEventListener('change', function () {
            this.parentNode.parentNode.querySelector('.billzone-eu-invoice-country-code-value').innerHTML = this.options[this.selectedIndex].value;
        });
    }

    // event listener for payment method codes changer
    var beiPaymentMethodCodes = document.querySelector('.billzone-eu-invoice-payment-method');
    if (document.body.contains(beiPaymentMethodCodes)) {
        beiPaymentMethodCodes.addEventListener('change', function () {
            this.parentNode.parentNode.querySelector('.billzone-eu-invoice-payment-method-value').innerHTML = this.options[this.selectedIndex].value;
        });
    }

    // event listener for ESC key to hide popup detail panel
    var beiPopUpContainerElement = document.querySelector('.billzone-eu-invoice-detail-wrapper');
    if (document.body.contains(beiPopUpContainerElement)) {
        document.onkeydown = function (e) {
            e = e || window.event;
            var beiIsEscape = false;
            if ('key' in e) {
                beiIsEscape = (e.key == 'Escape' || e.key == 'Esc');
            } else {
                beiIsEscape = (e.keyCode == 27);
            }
            if (!beiPopUpContainerElement.classList.contains('billzone-eu-invoice-hide-detail') && beiIsEscape) {
                beiClosePopupPanel();
            }
        };
    }

    // event listener to enable/disable editing for all VAT code checkbox
    var beiEnableDisableVATEdit = document.querySelector('#billzone_eu_invoice_enable_vat_edit');
    if (document.body.contains(beiEnableDisableVATEdit)) {
        beiEnableDisableVATEdit.addEventListener('change', function () {
            var beiSelectorElement = this.parentNode.parentNode.parentNode.querySelector('#billzone_eu_invoice_all_item_UN_EDIFACT_codes');
            var beiSameVATCheckboxElement = this.parentNode.parentNode.parentNode.querySelector('#billzone_eu_invoice_same_vat');
            if (this.checked) {
                beiSameVATCheckboxElement.removeAttribute('disabled');
                beiShowVATCodeWarningPanel();
            } else {
                beiSelectorElement.setAttribute('disabled', 'disabled');
                beiSameVATCheckboxElement.setAttribute('disabled', 'disabled');
            }
            beiUNEDIFACTCodeChangerDisableEnableToggler();
        });
    }

    // event listener for same VAT code checkbox
    var beiSameVAT = document.querySelector('#billzone_eu_invoice_same_vat');
    if (document.body.contains(beiSameVAT)) {
        beiSameVAT.addEventListener('change', function () {
            var beiSelectorElement = this.parentNode.querySelector('#billzone_eu_invoice_all_item_UN_EDIFACT_codes');
            if (this.checked) {
                beiSelectorElement.removeAttribute('disabled');
            } else {
                beiSelectorElement.setAttribute('disabled', 'disabled');
            }
            beiUNEDIFACTCodeChangerDisableEnableToggler();
            beiChangeAllVATToSameValue(beiSelectorElement.selectedIndex);
        });
    }

    // event listener for create invoice button
    var beiCreateInvoice = document.getElementById('billzoneEuInvoiceCreateInvoice');
    if (document.body.contains(beiCreateInvoice)) {
        beiCreateInvoice.addEventListener('click', function () {

            if (beiValidate()) {
                beiSendInvoiceDataViaAPI();
            }

        });
    }
}

// function to show VAT code change warning panel
function beiShowVATCodeWarningPanel() {
    var beiShowVATCodeWarningPanelElement = document.querySelector('.billzone-eu-invoice-warning-VAT-code-warning-wrapper');
    if (document.body.contains(beiShowVATCodeWarningPanelElement)) {
        beiShowVATCodeWarningPanelElement.classList.add('billzone-eu-invoice-show-warning');
    }
    document.getElementById('billzoneEuInvoiceVATCodeChangeConsentButton').addEventListener('click', function () {
        document.querySelector('.billzone-eu-invoice-warning-VAT-code-warning-wrapper').classList.remove('billzone-eu-invoice-show-warning');
    });
}

// Toggler function to disable/enable items UN/EDIFACT Tax codesvalue
function beiUNEDIFACTCodeChangerDisableEnableToggler() {
    var beiAllUNEDIFACTCodeChangerElement = document.querySelectorAll('.billzone-eu-invoice-item-UN-EDIFACT-codes');
    var beiEnableVATEditElement = document.querySelector('#billzone_eu_invoice_enable_vat_edit');
    var beiSameVAT = document.querySelector('#billzone_eu_invoice_same_vat');
    if (beiAllUNEDIFACTCodeChangerElement.length > 0) {
        for (var i = 0; i < beiAllUNEDIFACTCodeChangerElement.length; i++) {
            if (beiEnableVATEditElement.checked) {
                if (beiSameVAT.checked) {
                    beiAllUNEDIFACTCodeChangerElement[i].setAttribute('disabled', 'disabled')
                } else {
                    beiAllUNEDIFACTCodeChangerElement[i].removeAttribute('disabled')
                }
            } else {
                beiAllUNEDIFACTCodeChangerElement[i].setAttribute('disabled', 'disabled')
            }
        }
    }
    var beiAllbeiAllUNEDIFACTCodeChangerElementParentCell = document.querySelectorAll('.billzone-eu-invoice-item-UN-EDIFACT-cell');
    if (beiAllbeiAllUNEDIFACTCodeChangerElementParentCell.length > 0) {
        for (var i = 0; i < beiAllbeiAllUNEDIFACTCodeChangerElementParentCell.length; i++) {
            if (beiEnableVATEditElement.checked) {
                beiAllbeiAllUNEDIFACTCodeChangerElementParentCell[i].classList.remove('bei-disabled');
            } else {
                beiAllbeiAllUNEDIFACTCodeChangerElementParentCell[i].classList.add('bei-disabled');
            }
        }
    }
}

// Function to set all item VAT value to same value defined by #billzone_eu_invoice_all_item_UN_EDIFACT_codes element
function beiChangeAllVATToSameValue(billzoneEuInvoiceIndexValue) {
    var beiAllUNEDIFACTCodeChangerElement = document.querySelectorAll('.billzone-eu-invoice-item-UN-EDIFACT-codes');
    if (beiAllUNEDIFACTCodeChangerElement.length > 0) {
        for (var i = 0; i < beiAllUNEDIFACTCodeChangerElement.length; i++) {
            beiAllUNEDIFACTCodeChangerElement[i][billzoneEuInvoiceIndexValue].selected = true;
            beiAllUNEDIFACTCodeChangerElement[i].parentNode.parentNode.querySelector('.billzone-eu-invoice-item-UN-EDIFACT-value').innerHTML = beiAllUNEDIFACTCodeChangerElement[i].value;
            beiCheckOriginalVATandApplyWarningColor(beiAllUNEDIFACTCodeChangerElement[i]);
        }
        beiCheckIfAnyOriginalVATandApplyWarningColor();
    }
}

// Function to close popup panel
function beiClosePopupPanel() {
    document.querySelector('.billzone-eu-invoice-detail-container').classList.remove('ajax-end');
    document.querySelector('.billzone-eu-invoice-detail-container').classList.remove('ajax-load');
    document.querySelector('.billzone-eu-invoice-detail-container').classList.add('billzone-eu-invoice-detail-container-initial-position');
    setTimeout(function () {
        document.querySelector('.billzone-eu-invoice-detail-wrapper').classList.add('billzone-eu-invoice-hide-detail');
    }, 500);
}

// Function to remove wizard panel
function beiRemoveWizardPanel() {
    var beiPanelContainer = document.querySelector('.billzone-eu-invoice-wizard-container');
    beiPanelContainer.parentNode.removeChild(beiPanelContainer);
}

// Function to check if VAT rate code was changed and highlight VAT and gross value
function beiCheckOriginalVATandApplyWarningColor(beiDataElementToCheck) {
    var beiNewVATpercentage = beiDataElementToCheck.options[beiDataElementToCheck.selectedIndex].value;
    var beiItemOriginalVATPercentage = beiDataElementToCheck.dataset.originaltaxpercentage;
    var beiItemParentElement = beiDataElementToCheck.parentNode.parentNode.parentNode.parentNode;
    var beiVATPercentageTextElement = beiItemParentElement.querySelector('.billzone-eu-invoice-item-UN-EDIFACT-value');
    var beiVATValueTextElement = beiItemParentElement.querySelector('.billzone-eu-invoice-item-VAT-value');
    var beiGrossValueTextElement = beiItemParentElement.querySelector('.billzone-eu-invoice-item-gross-value');
    if (beiNewVATpercentage != beiItemOriginalVATPercentage) {
        beiVATPercentageTextElement.classList.add('billzone-eu-invoice-warning-highlighted-text');
        beiVATPercentageTextElement.nextSibling.classList.add('billzone-eu-invoice-warning-highlighted-text');
        beiVATValueTextElement.classList.add('billzone-eu-invoice-warning-highlighted-line-through-text');
        beiGrossValueTextElement.classList.add('billzone-eu-invoice-warning-highlighted-line-through-text');
    } else {
        beiVATPercentageTextElement.classList.remove('billzone-eu-invoice-warning-highlighted-text');
        beiVATPercentageTextElement.nextSibling.classList.remove('billzone-eu-invoice-warning-highlighted-text');
        beiVATValueTextElement.classList.remove('billzone-eu-invoice-warning-highlighted-line-through-text');
        beiGrossValueTextElement.classList.remove('billzone-eu-invoice-warning-highlighted-line-through-text');
    }

}

// Function to check if any VAT rate code was changed and highlight VAT and gross value
function beiCheckIfAnyOriginalVATandApplyWarningColor() {
    var beiChangedVATCodeItemNr = document.querySelectorAll(".billzone-eu-invoice-warning-highlighted-text").length;
    var beiFinalSumElemnts = document.querySelectorAll('.billzone-eu-invoice-final-sum');

    if (beiChangedVATCodeItemNr > 0) {
        for (var i = 0; i < beiFinalSumElemnts.length; i++) {
            beiFinalSumElemnts[i].classList.add('billzone-eu-invoice-warning-highlighted-line-through-text');
        }
    } else {
        for (var i = 0; i < beiFinalSumElemnts.length; i++) {
            beiFinalSumElemnts[i].classList.remove('billzone-eu-invoice-warning-highlighted-line-through-text');
        }
    }
}

// Function for banker's rounding
function beiBRound(beiNum, beiDecimalPlaces) {
    var beiD = beiDecimalPlaces || 0;
    var beiM = Math.pow(10, beiD);
    var beiN = +(beiD ? beiNum * beiM : beiNum).toFixed(8); // Avoid rounding errors
    var beiI = Math.floor(beiN);
    var beiF = beiN - beiI;
    var beiE = 1e-8; // Allow for rounding errors in beiF
    var beiR = (beiF > 0.5 - beiE && beiF < 0.5 + beiE) ?
        ((beiI % 2 == 0) ? beiI : beiI + 1) : Math.round(beiN);
    return beiD ? beiR / beiM : beiR;
}

// Function to set order data for popup display
function beiSetOrderData(beiDataIn) {
    Bei_Order_Data = beiDataIn;
}

// Function to return simple data for INPUT end other text base HTML tag, and return dual data for SELECT elemnts (value and key)
function beiValueKeyData(beiVKIn) {

    var beiVK = {};
    beiVK.value = null;
    beiVK.key = null;

    switch (beiVKIn.tagName.toUpperCase()) {

        case 'INPUT':
            if (beiVKIn.type.toUpperCase() == "CHECKBOX") {
                beiVK.value = (beiVKIn.checked == true) ? 'YES' : 'NO';
                beiVK.key = null;
            }
            else {
                beiVK.value = beiVKIn.value;
                beiVK.key = null;
            }
            break;
        case 'SELECT':
            beiVK.value = beiVKIn.options[beiVKIn.selectedIndex].value;
            beiVK.key = beiVKIn.options[beiVKIn.selectedIndex].text.trim();
            break;
        default:
            beiVK.value = beiVKIn.textContent.trim();
            beiVK.key = null;
            break;
    }

    return beiVK;
}


// Function to collect and send invoice data via AJAX to billzone.eu API
function beiSendInvoiceDataViaAPI() {

    var beiDOM;
    var beiFormElement = document.querySelectorAll('[data-beitype="header"], [data-beitype="summing"], [data-beitype="line"]');
    var beiAllInvoiceData = {};

    // collect all form data for invoice creation
    if (beiFormElement.length > 0) {

        for (var i = 0; i < beiFormElement.length; i++) {

            beiDOM = null;
            beiDOM = beiValueKeyData(beiFormElement[i]);

            switch (beiFormElement[i].dataset.beitype) {
                case 'header':
                case 'summing':
                    if (beiDOM.key === null) {
                        beiAllInvoiceData[beiFormElement[i].dataset.beifieldname] = beiDOM.value;
                    } else {
                        beiAllInvoiceData[beiFormElement[i].dataset.beifieldname + '_value'] = beiDOM.value;
                        beiAllInvoiceData[beiFormElement[i].dataset.beifieldname + '_key'] = beiDOM.key;
                    }
                    break;

                case 'line':

                    if (!beiAllInvoiceData.hasOwnProperty('lines')) {
                        beiAllInvoiceData['lines'] = {};
                    }

                    if (!beiAllInvoiceData['lines'].hasOwnProperty(beiFormElement[i].dataset.beilinenr)) {
                        beiAllInvoiceData['lines'][beiFormElement[i].dataset.beilinenr] = {};
                    }

                    if (beiDOM.key === null) {
                        beiAllInvoiceData['lines'][beiFormElement[i].dataset.beilinenr][beiFormElement[i].dataset.beifieldname] = beiDOM.value;
                    } else {
                        beiAllInvoiceData['lines'][beiFormElement[i].dataset.beilinenr][beiFormElement[i].dataset.beifieldname + '_value'] = beiDOM.value;
                        beiAllInvoiceData['lines'][beiFormElement[i].dataset.beilinenr][beiFormElement[i].dataset.beifieldname + '_key'] = beiDOM.key;
                    }
                    break;
            }

        }

    }

    document.querySelector('.billzone-eu-invoice-detail-container').classList.add('ajax-load');
    var beiMessageWrapper = document.querySelector('.billzone-eu-invoice-transaction-result-message-wrapper');
    var beiMessageWrapperOkButton = document.querySelector('#beiTransactionResultMessageOkButton');

    // send invoice data
    jQuery.ajax({
        url: ajaxurl,
        type: 'POST',
        dataType: 'json',
        data: {
            'action': 'billzone_eu_invoice_create_invoice_via_API',
            'invoicedata': beiAllInvoiceData
        },
        success: function (data) {

            document.querySelector('.billzone-eu-invoice-detail-container').classList.add('ajax-end');
            document.querySelector('.billzone-eu-invoice-detail-container').classList.remove('ajax-load');

            if ((data.CreateInvoiceResult.TransactionResult.ResultCode.Code == BEI_REQUEST_SUCCES_CODE) && (data.CreateInvoiceResult.RequestResult.Code == BEI_REQUEST_SUCCES_CODE)) {

                // show succes message
                beiMessageWrapper.classList.add('billzone-eu-invoice-succes');
                beiMessageWrapper.classList.remove('billzone-eu-invoice-error');
                beiMessageWrapper.classList.add('billzone-eu-invoice-show-message');
                document.querySelector('.billzone-eu-invoice-transaction-result-invoice-number').innerHTML = data.CreateInvoiceResult.TransactionResult.InvoiceNumber;
                var beiBtn = document.querySelector('#billzone_eu_invoice_Create_Button');
                beiBtn.classList.remove('billzone-eu-invoice-create-btn');
                beiBtn.classList.remove('button');
                beiBtn.classList.remove('button-primary');

                //show newly created invoice number in billzone.eu Invoice right sidebox
                document.querySelector('.billzone-eu-invoice-invoice-type-normal-invoice').classList.remove('billzone-eu-invoice-hide');
                document.querySelector('.billzone-eu-invoice-invoice-type-no-invoice').classList.add('billzone-eu-invoice-hide');
                document.querySelector('.billzone-eu-invoice-invoice-type-img-normal').classList.remove('billzone-eu-invoice-hide');
                document.querySelector('.billzone-eu-invoice-invoice-type-img-no-invoice').classList.add('billzone-eu-invoice-hide');
                document.querySelector('.billzone-eu-invoice-no-invoice-txt').classList.add('billzone-eu-invoice-hide');
                document.querySelector('.billzone-eu-invoice-invoice-nr-txt').classList.remove('billzone-eu-invoice-hide');
                document.querySelector('.billzone-eu-invoice-invoice-nr-txt').innerHTML = data.CreateInvoiceResult.TransactionResult.InvoiceNumber;

                //show the download link if email notification was set to OFF
                if (data.CreateInvoiceResult.TransactionResult.InvoiceCourierUrl != null && data.CreateInvoiceResult.TransactionResult.InvoiceCourierUrl.trim() != '') {
                    document.querySelector('#beiInvoiceUrlForAlreadyCreatedInvoice').href = data.CreateInvoiceResult.TransactionResult.InvoiceCourierUrl;
                    document.querySelector('.billzone-eu-invoice-invoice-download-link').classList.remove('billzone-eu-invoice-hide');
                }

                // event listener for succes message OK button
                beiMessageWrapperOkButton.addEventListener('click', function () {
                    beiMessageWrapper.classList.remove('billzone-eu-invoice-show-message');
                    beiClosePopupPanel();
                    document.querySelector('.billzone-eu-invoice-create-button-wrapper').classList.remove('billzone-eu-invoice-create-btn');
                });

            } else {

                // show error message
                beiMessageWrapper.classList.add('billzone-eu-invoice-error');
                beiMessageWrapper.classList.remove('billzone-eu-invoice-succes');
                beiMessageWrapper.classList.add('billzone-eu-invoice-show-message');
                document.querySelector('.billzone-eu-invoice-transaction-result-server-message').innerHTML = '[' + data.CreateInvoiceResult.TransactionResult.ResultCode.Code + '] ' + data.CreateInvoiceResult.TransactionResult.ResultCode.Message;

                // event listener for succes message OK button
                beiMessageWrapperOkButton.addEventListener('click', function () {
                    beiMessageWrapper.classList.remove('billzone-eu-invoice-show-message');
                    document.querySelector('.billzone-eu-invoice-detail-container').classList.remove('ajax-end');
                });

            }



        },
        error: function (errorThrown) {

            document.querySelector('.billzone-eu-invoice-detail-container').classList.add('ajax-end');
            document.querySelector('.billzone-eu-invoice-detail-container').classList.remove('ajax-load');

            // show error message
            beiMessageWrapper.classList.add('billzone-eu-invoice-error');
            beiMessageWrapper.classList.remove('billzone-eu-invoice-succes');
            beiMessageWrapper.classList.add('billzone-eu-invoice-show-message');
            document.querySelector('.billzone-eu-invoice-transaction-result-server-message').innerHTML = '[' + errorThrown.status + '] ' + errorThrown.statusText;

            // event listener for error message OK button
            beiMessageWrapperOkButton.addEventListener('click', function () {
                beiMessageWrapper.classList.remove('billzone-eu-invoice-show-message');
                document.querySelector('.billzone-eu-invoice-detail-container').classList.remove('ajax-end');
            });

            beiWriteErrorToConsole('billzone.eu Invoice create invoice error! See detail bellow:', errorThrown);

        }
    });

}

// Function to start setup wizard
function beiStartSetupWizard() {

    document.getElementById('billzone_start_setup_wizard').classList.add('ajax-load');
    jQuery.ajax({
        url: ajaxurl,
        type: 'POST',
        data: {
            'action': 'billzone_eu_invoice_start_wizard_AJAX',
        },
        success: function (data) {

            var adat;
            var jsonStartIndex = data.indexOf('{"panel":');

            //START--------------------------------------------
            //Unicredit payment gateway error message excision from response string, to parse the result as a valid JSON object. Works only if the plugin is installed.
            if (jsonStartIndex == 0) {
                adat = data;
            } else {
                adat = data.slice(jsonStartIndex);

            }
            var pnl;
            pnl = JSON.parse(adat);
            //Unicredit payment gateway error message excision from response string, to parse the result as a valid JSON object. Works only if the plugin is installed.
            //END----------------------------------------------

            document.getElementById('billzone_start_setup_wizard').classList.remove('ajax-load');
            var beiSetupWizardWrapperElement = document.querySelector('.billzone-eu-invoice-wizard-wrapper');
            beiSetupWizardWrapperElement.insertAdjacentHTML('beforeend', pnl.panel);


            var beiCloseWizardButton = document.querySelector('.billzone-eu-invoice-close-wizard');
            if (document.body.contains(beiCloseWizardButton)) {

                // event listener for wizard panel close buttons
                beiCloseWizardButton.addEventListener('click', function () {
                    beiRemoveWizardPanel();
                });

                // event listener for ESC key to hide wizard panel
                document.onkeydown = function (e) {
                    e = e || window.event;
                    var beiIsEscape = false;
                    if ('key' in e) {
                        beiIsEscape = (e.key == 'Escape' || e.key == 'Esc');
                    } else {
                        beiIsEscape = (e.keyCode == 27);
                    }
                    if (beiIsEscape) {
                        beiRemoveWizardPanel();
                    }
                };

                // event listener to show wizard API code finder help
                var beiShowAPICodeFinderHelpElement = document.querySelector('.billzone-eu-invoice-show-wizard-help');
                beiShowAPICodeFinderHelpElement.addEventListener('click', function () {
                    document.querySelector('.billzone-eu-invoice-wizard-help-wrapper').classList.add('billzone-eu-invoice-show');
                });

                // event listener to close wizard API code finder help
                var beiCloseAPICodeFinderHelpElement = document.querySelector('.billzone-eu-invoice-close-wizard-help');
                beiCloseAPICodeFinderHelpElement.addEventListener('click', function () {
                    document.querySelector('.billzone-eu-invoice-wizard-help-wrapper').classList.remove('billzone-eu-invoice-show');
                });

                // event listener for start wizard button
                var beiHlp1 = document.querySelectorAll('.billzone-eu-invoice-help-image-1');
                var beiHlp2 = document.querySelectorAll('.billzone-eu-invoice-help-image-2');
                var beiStartWizardButton = document.querySelector('.billzone-eu-invoice-start-wizard');
                if (document.body.contains(beiStartWizardButton)) {
                    beiStartWizardButton.addEventListener('click', function () {
                        beiSlideNext();
                        for (var i = 0; i < beiHlp1.length; i++) {
                            beiHlp1[i].classList.add('billzone-eu-invoice-hide');
                        }
                        for (var i = 0; i < beiHlp1.length; i++) {
                            beiHlp2[i].classList.remove('billzone-eu-invoice-hide');
                        }
                    });
                }

                // event listener for start wizard button
                var beiStartWizardButton = document.querySelector('.billzone-eu-invoice-wizard-step-1 .billzone-eu-invoice-prev-step');
                if (document.body.contains(beiStartWizardButton)) {
                    beiStartWizardButton.addEventListener('click', function () {
                        beiSlidePrev();
                        for (var i = 0; i < beiHlp1.length; i++) {
                            beiHlp1[i].classList.remove('billzone-eu-invoice-hide');
                        }
                        for (var i = 0; i < beiHlp1.length; i++) {
                            beiHlp2[i].classList.add('billzone-eu-invoice-hide');
                        }
                    });
                }

                // event listener for test API code button
                var beiAPICodeTestButton = document.querySelector('.billzone-eu-invoice-test');
                if (document.body.contains(beiAPICodeTestButton)) {
                    beiAPICodeTestButton.addEventListener('click', function () {
                        beiContinueSetupWizard();
                    });
                }
            }

        },
        error: function (errorThrown) {
            beiWriteErrorToConsole('billzone.eu Invoice AJAX error! See detail bellow:', errorThrown);
        }
    });

}

// function to set and style API status check response
function beiStyledStatusResult(beiStatusResultIsSuccess, beiStatusResultCompanyName = '---', beiStatusResultAddress = '---', beiStatusResultTaxNumber = '---', beiStatusResultRegistrationNumber = '---') {

    if (beiStatusResultIsSuccess === false) {
        document.querySelector('.billzone-eu-invoice-api-response-text').classList.add('billzone-eu-invoice-error');
        document.querySelector('.billzone-eu-invoice-api-response-code').classList.add('billzone-eu-invoice-error');
        document.querySelector('.billzone-eu-invoice-wizard-step-1 .billzone-eu-invoice-next-step').classList.add('billzone-eu-invoice-hide');
        document.querySelector('.billzone-eu-invoice-wizard-step-1 .billzone-eu-invoice-test').classList.remove('billzone-eu-invoice-hide');

        document.querySelector('.billzone-eu-invoice-api-response-companyname').classList.add('billzone-eu-invoice-error');
        document.querySelector('.billzone-eu-invoice-api-response-address').classList.add('billzone-eu-invoice-error');
        document.querySelector('.billzone-eu-invoice-api-response-taxnumber').classList.add('billzone-eu-invoice-error');
        document.querySelector('.billzone-eu-invoice-api-response-registrationnumber').classList.add('billzone-eu-invoice-error');
    }
    else {
        document.querySelector('.billzone-eu-invoice-api-response-text').classList.add('billzone-eu-invoice-ok');
        document.querySelector('.billzone-eu-invoice-api-response-code').classList.add('billzone-eu-invoice-ok');
        document.querySelector('.billzone-eu-invoice-wizard-step-1 .billzone-eu-invoice-next-step').classList.remove('billzone-eu-invoice-hide');
        document.querySelector('.billzone-eu-invoice-wizard-step-1 .billzone-eu-invoice-test').classList.add('billzone-eu-invoice-hide');

        document.querySelector('.billzone-eu-invoice-api-response-companyname').classList.add('billzone-eu-invoice-ok');
        document.querySelector('.billzone-eu-invoice-api-response-address').classList.add('billzone-eu-invoice-ok');
        document.querySelector('.billzone-eu-invoice-api-response-taxnumber').classList.add('billzone-eu-invoice-ok');
        document.querySelector('.billzone-eu-invoice-api-response-registrationnumber').classList.add('billzone-eu-invoice-ok');
    }

    document.querySelector('.billzone-eu-invoice-api-response-companyname').innerHTML = beiStatusResultCompanyName;
    document.querySelector('.billzone-eu-invoice-api-response-address').innerHTML = beiStatusResultAddress;
    document.querySelector('.billzone-eu-invoice-api-response-taxnumber').innerHTML = beiStatusResultTaxNumber;
    document.querySelector('.billzone-eu-invoice-api-response-registrationnumber').innerHTML = beiStatusResultRegistrationNumber;
}

// function to continue wizard after checking API security code and the response is all right (code 61001)
function beiContinueSetupWizard() {

    document.querySelector('#beiWizardSecurityCode').value;

    document.querySelector('.billzone-eu-invoice-test').classList.add('ajax-load');
    jQuery.ajax({
        url: ajaxurl,
        type: 'POST',
        dataType: 'json',
        data: {
            'action': 'billzone_eu_invoice_check_API_token_and_return_response_AJAX',
            'issandbox': document.querySelector('#beiWizardIsSandbox').checked,
            'securitycode': document.querySelector('#beiWizardSecurityCode').value
        },
        success: function (data) {
            document.querySelector('.billzone-eu-invoice-test').classList.remove('ajax-load');
            if (data.status.code != 61001) {
                beiStyledStatusResult(false);
            }
            else {

                beiStyledStatusResult(true, data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.Name, data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.AddressCountryId + '-' + data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.AddressPostalCode + ', ' + data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.AddressCity + ', ' + data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.AddressLine1 + " " + data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.AddressLine2, data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.TaxNumber, data.requestresult.GetInvoiceDataResult.QueryResult.CompanyData.RegistrationNumber);
                beiWizardAddNextTwoStep(data.nextstepshtml);

            }
            let beiAPIResponseWrapper = document.querySelectorAll('.billzone-eu-invoice-api-response');
            if (beiAPIResponseWrapper.length > 0) {
                for (let i = 0; i < beiAPIResponseWrapper.length; i++) {
                    beiAPIResponseWrapper[i].classList.add('billzone-eu-invoice-show');
                }
            }
            document.querySelector('.billzone-eu-invoice-api-server').innerHTML = data.serverpath;
            document.querySelector('.billzone-eu-invoice-api-response-text').innerHTML = data.status.message;
            document.querySelector('.billzone-eu-invoice-api-response-code').innerHTML = data.status.code;

        },
        error: function (errorThrown) {
            document.querySelector('.billzone-eu-invoice-test').classList.remove('ajax-load');

            let beiAPIResponseWrapper = document.querySelectorAll('.billzone-eu-invoice-api-response');
            if (beiAPIResponseWrapper.length > 0) {
                for (let i = 0; i < beiAPIResponseWrapper.length; i++) {
                    beiAPIResponseWrapper[i].classList.add('billzone-eu-invoice-show');
                }
            }
            beiStyledStatusResult(false);
            document.querySelector('.billzone-eu-invoice-api-server').innerHTML = data.serverpath;
            document.querySelector('.billzone-eu-invoice-api-response-text').innerHTML = errorThrown.statusText;
            document.querySelector('.billzone-eu-invoice-api-response-code').innerHTML = errorThrown.status;
            beiWriteErrorToConsole('billzone.eu Invoice AJAX error! See detail bellow:', errorThrown);
        }
    });
}

// function to show/hide help info icon on wizard panel title
function beiWizardInfoIconToggler() {

    var beiLeftVal = parseInt(document.querySelector('.billzone-eu-invoice-wizard-slideframe').style.left);
    if (beiLeftVal == 0 || beiLeftVal == -100) {
        document.querySelector('.billzone-eu-invoice-show-wizard-help').classList.remove('billzone-eu-invoice-hide');
    } else {
        document.querySelector('.billzone-eu-invoice-show-wizard-help').classList.add('billzone-eu-invoice-hide');
    }

}

// function to slide wizard to left
function beiSlideNext() {

    var beiLeftVal = parseInt(document.querySelector('.billzone-eu-invoice-wizard-slideframe').style.left) - 100;
    document.querySelector('.billzone-eu-invoice-wizard-slideframe').style.left = beiLeftVal + '%';
    beiWizardInfoIconToggler();

}

// function to slide wizard to right
function beiSlidePrev() {

    var beiLeftVal = parseInt(document.querySelector('.billzone-eu-invoice-wizard-slideframe').style.left) + 100;
    document.querySelector('.billzone-eu-invoice-wizard-slideframe').style.left = beiLeftVal + '%';
    beiWizardInfoIconToggler();

}

// function to add the next two step to wizard after checkin API security code
function beiWizardAddNextTwoStep(beiNextStepsHTML) {

    // add the next two step
    document.querySelector('.billzone-eu-invoice-wizard-slideframe').insertAdjacentHTML('beforeend', beiNextStepsHTML);

    // event listener for next step wizard button
    var beiNext = document.querySelectorAll('.billzone-eu-invoice-wizard-step-1 .billzone-eu-invoice-next-step, .billzone-eu-invoice-wizard-step-2 .billzone-eu-invoice-next-step');
    if (beiNext.length > 0) {
        for (var i = 0; i < beiNext.length; i++) {
            beiNext[i].addEventListener('click', function () {
                beiSlideNext();
            });
        }
    }

    // event listener for next step wizard button
    var beiPrev = document.querySelectorAll('.billzone-eu-invoice-wizard-step-2 .billzone-eu-invoice-prev-step, .billzone-eu-invoice-wizard-step-3 .billzone-eu-invoice-prev-step');
    if (beiPrev.length > 0) {
        for (var i = 0; i < beiPrev.length; i++) {
            beiPrev[i].addEventListener('click', function () {
                beiSlidePrev();
            });
        }
    }

    // event listener for wizard save settings button
    var beiAPICodeTestButton = document.querySelector('.billzone-eu-invoice-save-settings');
    if (document.body.contains(beiAPICodeTestButton)) {
        beiAPICodeTestButton.addEventListener('click', function () {

            beiMappingWizardDataToSettings();
            this.classList.add('ajax-load');

            // fire submit event for setting form submit button
            var beiSaveSettingsElement = document.querySelector('.woocommerce-save-button[type="submit"]');
            beiSaveSettingsElement.click();
        });
    }

    // event listener for wizard select tag change
    var beiAllSelect = document.querySelectorAll('.bei-wizard-form-element-select');
    if (beiAllSelect.length > 0) {
        for (var i = 0; i < beiAllSelect.length; i++) {
            beiAllSelect[i].addEventListener('click', function () {
                beiShowValueForSelected();
            });
        }
    }

    beiShowValueForSelected();

}

function beiShowValueForSelected() {

    var beiAllSelect = document.querySelectorAll('.bei-wizard-form-element-select');

    if (beiAllSelect.length > 0) {
        for (var i = 0; i < beiAllSelect.length; i++) {
            document.querySelector('label[for="' + beiAllSelect[i].id + '"]').innerHTML = beiAllSelect[i].options[beiAllSelect[i].selectedIndex].value;
            if (beiAllSelect[i].id == 'beiWizardAccountBlock') {
                var beiGetExchangeRateElement = document.querySelector('.billzone-eu-invoice-get-exchange-rate-wrapper');
                if (beiAllSelect[i].dataset.defaultcurrency.toUpperCase() != beiAllSelect[i].options[beiAllSelect[i].selectedIndex].value.toUpperCase() && beiAllSelect[i].dataset.issuercountrycode.toUpperCase() == 'HU') {
                    beiGetExchangeRateElement.classList.add('billzone-eu-invoice-show');
                } else {
                    beiGetExchangeRateElement.classList.remove('billzone-eu-invoice-show');
                }
            }
        }
    }

}

function beiMappingWizardDataToSettings() {

    var beiAllWizardFormElements = document.querySelectorAll('.bei-wizard-form-element');

    if (beiAllWizardFormElements.length > 0) {

        var beiFormElement;
        for (var i = 0; i < beiAllWizardFormElements.length; i++) {

            beiFormElement = document.getElementById(beiAllWizardFormElements[i].dataset.mapfor);
            switch (beiAllWizardFormElements[i].tagName.toLowerCase()) {
                case 'input':
                    switch (beiAllWizardFormElements[i].type.toLowerCase()) {
                        case 'text':
                        case 'number':
                            beiFormElement.value = beiAllWizardFormElements[i].value;
                            break;
                        case 'checkbox':
                            if (beiAllWizardFormElements[i].checked) {
                                beiFormElement.checked = true;
                            } else {
                                beiFormElement.checked = false;
                            }
                            break;
                    }
                    break;
                case 'select':
                    beiFormElement.value = beiAllWizardFormElements[i].options[beiAllWizardFormElements[i].selectedIndex].text;
                    break;
            }

        }
    }

}

// function to check if required field is empty
function beiCheckFieldIsEmpty(beiFieldToBeChecked) {

    if (beiFieldToBeChecked.value.trim() !== '') {
        beiFieldToBeChecked.parentNode.parentNode.classList.remove('billzone-eu-invoice-validation-error');
        document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:first-of-type').classList.remove('billzone-eu-invoice-show');
        return true;
    } else {
        beiFieldToBeChecked.parentNode.parentNode.classList.add('billzone-eu-invoice-validation-error');
        document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:first-of-type').classList.add('billzone-eu-invoice-show');
        return false;
    }

}

// function to check email format
function beiCheckEmailFormat(beiFieldToBeChecked) {

    if (beiFieldToBeChecked.value.trim() !== '') {

        var beiEmailRegex = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

        if (beiEmailRegex.test(String(beiFieldToBeChecked.value.trim()).toLowerCase())) {
            beiFieldToBeChecked.parentNode.parentNode.classList.remove('billzone-eu-invoice-validation-error');
            document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:last-of-type').classList.remove('billzone-eu-invoice-show');
            return true;
        } else {
            beiFieldToBeChecked.parentNode.parentNode.classList.add('billzone-eu-invoice-validation-error');
            document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:last-of-type').classList.add('billzone-eu-invoice-show');
            return false;
        }

    }

    return true;
}

// function to check date format (YYYY-MM-DD)
function beiCheckDateFormat(beiFieldToBeChecked) {

    if (beiFieldToBeChecked.value.trim() !== '') {

        var beiDateRegex = /^\d{4}-\d{2}-\d{2}$/;

        if (beiDateRegex.test(String(beiFieldToBeChecked.value.trim()).toLowerCase())) {
            beiFieldToBeChecked.parentNode.parentNode.classList.remove('billzone-eu-invoice-validation-error');
            document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:last-of-type').classList.remove('billzone-eu-invoice-show');
            return true;
        } else {
            beiFieldToBeChecked.parentNode.parentNode.classList.add('billzone-eu-invoice-validation-error');
            document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:last-of-type').classList.add('billzone-eu-invoice-show');
            return false;
        }

    }

    return true;

}

// function to check if entered value can be converted to number
function beiCheckFieldIsNumber(beiFieldToBeChecked) {

    var beiIsNaN = isNaN(Number(beiFieldToBeChecked.value.replace(',', '.')));

    if (beiIsNaN) {
        beiFieldToBeChecked.parentNode.parentNode.classList.add('billzone-eu-invoice-validation-error');
        document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:last-of-type').classList.add('billzone-eu-invoice-show');
    } else {
        beiFieldToBeChecked.parentNode.parentNode.classList.remove('billzone-eu-invoice-validation-error');
        document.querySelector('[data-validationmessagefor="' + beiFieldToBeChecked.dataset.beifieldname + '"] span:last-of-type').classList.remove('billzone-eu-invoice-show');
    }

    return !beiIsNaN;
}

// function to check TAX field
function beiCheckTaxField(beiFieldToBeChecked) {

    var beiLegEnt = document.querySelector('.billzone-eu-invoice-legal-entity-wout-tax-nr-wrapper');
    var beiLegEntInput = document.querySelector('#beiLegalEntityWithoutTaxNumber');

    if (!beiFieldToBeChecked.value.trim() == '') {
        beiLegEntInput.checked = false;
        beiLegEntInput.disabled = true;
        beiLegEnt.classList.add('billzone-eu-invoice-disable');
    }
    else {
        beiLegEntInput.disabled = false;
        beiLegEnt.classList.remove('billzone-eu-invoice-disable');
    }
}

// function to check country field
function beiCheckCountryField(beiFieldToBeChecked) {

    var beiHasHUTaxNr = document.querySelector('.billzone-eu-invoice-has-hu-tax-nr-wrapper');
    var beiHasHUTaxNrInput = document.querySelector('#beiHasHungarianTaxNumber');
    var beiVATNrInput = document.querySelector('input[data-beifieldname="VAT_number"]');
    var beiEUVATNrInput = document.querySelector('input[data-beifieldname="EU_VAT_number"]');
    var beiValidationMsg = document.querySelector('[data-validationmessagefor="' + beiVATNrInput.dataset.beifieldname + '"] span:last-of-type');
    var beiIsOK = true;

    if (beiFieldToBeChecked.value.toUpperCase() == "HU") {
        beiHasHUTaxNr.classList.add('billzone-eu-invoice-hide');
        beiHasHUTaxNrInput.checked = false;
        beiVATNrInput.parentNode.parentNode.classList.remove('billzone-eu-invoice-validation-error');
        beiValidationMsg.classList.remove('billzone-eu-invoice-show');
    }
    else {
        beiHasHUTaxNr.classList.remove('billzone-eu-invoice-hide');

        if (beiHasHUTaxNrInput.checked && beiVATNrInput.value.trim() == '') {
            beiVATNrInput.parentNode.parentNode.classList.add('billzone-eu-invoice-validation-error');
            beiValidationMsg.classList.add('billzone-eu-invoice-show');
            document.querySelector("#beiLegalEntityWithoutTaxNumber").checked = false;
            beiIsOK = false;
        }
        else {
            beiVATNrInput.parentNode.parentNode.classList.remove('billzone-eu-invoice-validation-error');
            beiValidationMsg.classList.remove('billzone-eu-invoice-show');
        }

        if (beiHasHUTaxNrInput.checked) {
            beiEUVATNrInput.value = '';
        }
    }

    return beiIsOK;
}

// function to validate invoice submit form fields
function beiValidate() {

    //If we want to check only when form is visible we should use this: if (!document.querySelector('.billzone-eu-invoice-detail-container').classList.contains('billzone-eu-invoice-detail-container-initial-position')) {}

    var beiCustomerName = document.querySelector('input[data-beifieldname="customer_name"]');
    var beiEmail = document.querySelector('input[data-beifieldname="email"]');
    var beiCity = document.querySelector('input[data-beifieldname="city"]');
    var beiPostcode = document.querySelector('input[data-beifieldname="postcode"]');
    var beiAddress = document.querySelector('input[data-beifieldname="address"]');
    var beiDeliveryDate = document.querySelector('input[data-beifieldname="delivery_date"]');
    var beiPaymentDue = document.querySelector('input[data-beifieldname="payment_due"]');
    var beiExchangeRate = document.querySelector('input[data-beifieldname="exchange_rate"]');
    var beiVATNr = document.querySelector('input[data-beifieldname="VAT_number"]');
    var beiCountry = document.querySelector('select[data-beifieldname="country"]');

    beiCheckTaxField(beiVATNr);

    var beiAllTrue = [];

    beiAllTrue.push(beiCheckCountryField(beiCountry));

    beiAllTrue.push(beiCheckFieldIsEmpty(beiCustomerName));

    beiAllTrue.push(beiCheckFieldIsEmpty(beiEmail));
    beiAllTrue.push(beiCheckEmailFormat(beiEmail));

    beiAllTrue.push(beiCheckFieldIsEmpty(beiCity));
    beiAllTrue.push(beiCheckFieldIsEmpty(beiPostcode));
    beiAllTrue.push(beiCheckFieldIsEmpty(beiAddress));

    beiAllTrue.push(beiCheckFieldIsEmpty(beiDeliveryDate));
    beiAllTrue.push(beiCheckDateFormat(beiDeliveryDate));

    beiAllTrue.push(beiCheckFieldIsEmpty(beiPaymentDue));
    beiAllTrue.push(beiCheckDateFormat(beiPaymentDue));

    if (document.body.contains(beiExchangeRate)) {
        beiAllTrue.push(beiCheckFieldIsEmpty(beiExchangeRate));
        beiAllTrue.push(beiCheckFieldIsNumber(beiExchangeRate));
    }

    var beiIsTrue = function (element) {
        // checks whether an element is true
        return element === true;
    };

    if (beiAllTrue.every(beiIsTrue)) {
        document.getElementById('billzoneEuInvoiceCreateInvoice').classList.remove('billzone-eu-invoice-button-disabled');
    } else {
        document.getElementById('billzoneEuInvoiceCreateInvoice').classList.add('billzone-eu-invoice-button-disabled');
    }

    return beiAllTrue.every(beiIsTrue);

}

// function to attach event for validation in invoice submit form
function beiAttachFieldValidation() {

    var beiAllField = document.querySelectorAll('input[data-beitype="header"], select[data-beitype="header"]');
    if (beiAllField.length > 0) {
        for (var i = 0; i < beiAllField.length; i++) {
            beiAllField[i].addEventListener('change', beiValidate, false);
            beiAllField[i].addEventListener('keyup', beiValidate, false);
        }
    }

}

// function to show message for log table actions
function beiShowLogActionMessagePanel(beiAJAXActionName, beiAJAXSpinerElement) {

    document.getElementById(beiAJAXSpinerElement).classList.add('ajax-load');
    jQuery.ajax({
        url: ajaxurl,
        type: 'POST',
        dataType: 'json',
        data: {
            'action': beiAJAXActionName
        },
        success: function (data) {

            // get message box wrapper element
            beiMessagePanelWrapper = document.querySelector('.billzone-eu-invoice-export-and-clear-log-wrapper');

            // remove message panel if any
            beiCloseMessagePanel();

            // add the message panel
            beiMessagePanelWrapper.insertAdjacentHTML('beforeend', data.messagepanel);

            // attach event to message panel buttons
            beiEventListenerForSettingsMessagePanel(beiAJAXSpinerElement);

        },
        error: function (errorThrown) {
            beiWriteErrorToConsole('billzone.eu Invoice AJAX error! See detail bellow:', errorThrown);
        }
    });
}

// function to attach event to newly created message panel buttons
function beiEventListenerForSettingsMessagePanel(beiAJAXSpinerElement) {

    beiMessagePanelWrapper = document.querySelector('.billzone-eu-invoice-export-and-clear-log-wrapper');
    beiOK = beiMessagePanelWrapper.querySelector('#beiMessagePanelOK');
    beiClose = beiMessagePanelWrapper.querySelector('.billzone-eu-invoice-message-title .dashicons');
    beiCopyToClipboard = beiMessagePanelWrapper.querySelector('#beiMessagePanelCopyToClipboard');

    if (document.body.contains(beiMessagePanelWrapper)) {

        // event to close message panel using ESC 
        document.onkeydown = function (e) {
            e = e || window.event;
            var beiIsEscape = false;
            if ('key' in e) {
                beiIsEscape = (e.key == 'Escape' || e.key == 'Esc');
            } else {
                beiIsEscape = (e.keyCode == 27);
            }
            if (beiIsEscape) {
                beiCloseMessagePanel(beiAJAXSpinerElement);
            }
        };

        // event to close message panel using OK button
        if (document.body.contains(beiOK)) {
            beiOK.addEventListener('click', function () {
                beiCloseMessagePanel(beiAJAXSpinerElement);
            });
        }

        // event to close message panel using X 
        if (document.body.contains(beiClose)) {
            beiClose.addEventListener('click', function () {
                beiCloseMessagePanel(beiAJAXSpinerElement);
            });
        }

        // event to copy log data to clipboard 
        if (document.body.contains(beiCopyToClipboard)) {
            beiCopyToClipboard.addEventListener('click', function () {
                beiTextarea = document.querySelector('#beiMessagePanelTextarea');
                if (document.body.contains(beiTextarea)) {
                    beiTextarea.focus();
                    beiTextarea.select();
                    var beiCopySuccesfully = document.execCommand('copy');
                    if (beiCopySuccesfully) {
                        beiCopyToClipboard.innerHTML = beiCopyToClipboard.dataset.copyok;
                        beiCopyToClipboard.classList.add('billzone-eu-invoice-copy-ok');
                    } else {
                        beiCopyToClipboard.innerHTML = beiCopyToClipboard.dataset.copyerror;
                        beiCopyToClipboard.classList.add('billzone-eu-invoice-copy-error');
                    }
                }
            });
        }
    }
}

// function to close message panel
function beiCloseMessagePanel(beiAJAXSpinerElement) {
    beiMessagePanelWrapper = document.querySelector('.billzone-eu-invoice-export-and-clear-log-wrapper');
    beiMessagePanel = beiMessagePanelWrapper.querySelector('.billzone-eu-invoice-log-clear-message-panel-wrapper');
    if (document.body.contains(beiMessagePanel)) {
        beiMessagePanel.parentNode.removeChild(beiMessagePanel);
        beiSpinerElement = document.getElementById(beiAJAXSpinerElement);
        if (document.body.contains(beiSpinerElement)) {
            beiSpinerElement.classList.remove('ajax-load');
        }
    }
}

// function to log all JS error and send to backend
function beiLogMe(beiLogType, beiLogText) {

    if (Bei_Log_Is_Enabled) {
        jQuery.ajax({
            url: ajaxurl,
            type: 'POST',
            dataType: 'json',
            data: {
                'action': 'billzone_eu_invoice_logme_AJAX',
                'beilogtype': beiLogType,
                'beilogtext': beiLogText
            },
            success: function () {
                console.log('billzone.eu Invoice log data succesfully saved to database');
            },
            error: function (errorThrown) {
                beiWriteErrorToConsole('Error: billzone.eu Invoice log data CAN NOT be saved to database!', errorThrown);
            }
        });
    }

}

// function to catch all JS error
window.onerror = function (err, file, line) {
    beiLogMe("JS ERROR', 'Line: " + line + " | Error message: |" + err);
    return false;
}